/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.assemblies;

import ist.palcom.collections.LinearSet;
import ist.palcom.device.RemoteConnectHelper;
import ist.palcom.discovery.CommandEvent;
import ist.palcom.discovery.InfoEvent;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PeriodicEvent;
import ist.palcom.resource.descriptor.Command;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.LocalSID;
import ist.palcom.resource.descriptor.PRDAssemblyVer;
import ist.palcom.resource.descriptor.PRDAssemblyD;
import ist.palcom.resource.descriptor.PRDConnection;
import ist.palcom.resource.descriptor.PRDService;
import ist.palcom.resource.descriptor.PRDServiceList;
import ist.palcom.resource.descriptor.ServiceProxy;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.services.ServiceProxyEvent;
import ist.palcom.xml.XMLException;

import java.io.IOException;

public class UpdateStateMachine extends StateContext {

	final State initialState;
	final State discoveredState;
	final State connectingState;
	final State connectedState;
	final State fullUpdateState;
	final State establishedState;
	final State disconnectState;

	URN providerURN;
	final URN customerURN;
	final AssemblyDeviceContext deviceContext;
	final PalcomPriorityThread stateThread;
	final RemoteConnectHelper rch;

	private LinearSet ignoreDeviceURNs;

	ServiceProxy amProxy;

	public UpdateStateMachine(URN thisURN, AssemblyDeviceContext dc, PalcomPriorityThread t) {
		providerURN = null;
		customerURN = thisURN;
		deviceContext = dc;
		stateThread = t;
		ignoreDeviceURNs = new LinearSet();
		ignoreDeviceURNs.add(customerURN.getDeviceURN());
		//hack to make test case run w/o device
		if(dc != null) {
			rch = new RemoteConnectHelper(dc);
		} else {
			rch = null;
		}
		initialState = new State() {
			UpdateStateMachine context = UpdateStateMachine.this;

			public void enterState() {
				if(providerURN != null) {
					deviceContext.getResourceManager2().removePRDConnectionReceiver(
							new DeviceID(providerURN.getDeviceURN().toString()), stateThread);
					deviceContext.getResourceManager2().removePRDConnectionReceiver(
							new DeviceID(customerURN.getDeviceURN().toString()), stateThread);
					providerURN = null;
				}

				if(amProxy != null) {
					//TODO: any cleanup needed?
					amProxy = null;
				}
			}

			public State handleEvent(Object o) {
				if(o instanceof InfoEvent && ((InfoEvent)o).getInfo() instanceof PRDServiceList) 
				{
					PRDServiceList sl = (PRDServiceList) ((InfoEvent)o).getInfo();
					for(int i=0; i < sl.getNumPRDServiceListItem(); i++) {
						if(sl.getPRDServiceListItem(i) instanceof PRDService) {
							PRDService si = (PRDService)sl.getPRDServiceListItem(i); 		

							//Don't connect to the AssemblyManager on the same device
//							if(si.getName().equals(AssemblyManager.SERVICE_NAME) &&
//							!si.getDeviceURN().equals(customerURN.getDeviceURN())) {
							if(si.getName().equals(IAssemblyManager.SERVICE_NAME) &&
									checkIfURNisAllowed(si)) {
//								System.out.println("got ServiceInfo");

								System.err.println("UpdateStateMachine.handleEvent broken by API changes");
								throw new Error("UpdateStateMachine.handleEvent broken by API changes");
								//XXX adapt code below to new addressing
//								providerURN = si.getURN();
//								ignoreDeviceURNs.add(si.getURN().getDeviceURN());
//								return context.discoveredState;
							}
						}
					}
				} 
				return this;
			}
			public String toString() { return "initialState";}
		};

		discoveredState = new State() {
			UpdateStateMachine context = UpdateStateMachine.this;
			public void enterState() {
				//TODO: connect to providerURN
//				System.out.println("try to connect");
				context.transition(context.connectingState);
			}
			public String toString() { return "discoveredState";}
		};

		connectingState = new State() {
			UpdateStateMachine context = UpdateStateMachine.this;

			public void enterState() {
				//hack to make test case run w/o device
				if(deviceContext != null) {
					if(!rch.doConnect(new PRDConnection(new LocalSID(providerURN.toString()), new LocalSID(customerURN.toString())), 5000, 20000)) {
						System.out.println("failed to connect");
						context.transition(initialState);
					}
				}
			}

			public State handleEvent(Object o) {
				throw new Error("UpdateStateMachine broken by API changes");
//				if(o instanceof InfoEvent && 
//						((InfoEvent)o).getInfo() instanceof PRDConnection && 
//						((InfoEvent)o).infoShouldBeKept() &&
//						((PRDConnection)((InfoEvent)o).getInfo()).getProviderLocalSID().equals(
//								new LocalSID(providerURN.toString())) &&
//						((PRDConnection)((InfoEvent)o).getInfo()).getCustomerLocalSID().equals(
//								new LocalSID(customerURN.toString()))) {
////					System.out.println("got ConnectionInfo");
//					return context.connectedState;
//				}else if(o instanceof ServiceProxyEvent && 
//						((ServiceProxyEvent)o).getServiceProxy().getProviderURN().equals(providerURN)) {
////					System.out.println("got ServiceProxy");
//					amProxy = ((ServiceProxyEvent)o).getServiceProxy();
//					return context.fullUpdateState;
//				} else {
//					return this;
//				}
			}

			public String toString() { return "connectingState";}
		};

		connectedState = new State() {
			UpdateStateMachine context = UpdateStateMachine.this;
			public void enterState() {
				//TODO: listen for and handle disconnect in relevant states
				deviceContext.getResourceManager2().updatePRDConnectionReceiver(
						new DeviceID(providerURN.getDeviceURN().toString()), stateThread);
				deviceContext.getResourceManager2().updatePRDConnectionReceiver(
						new DeviceID(customerURN.getDeviceURN().toString()), stateThread);
				stateThread.enablePeriodicEvents(5000);
			}
			public State handleEvent(Object o) {
				throw new Error("UpdateStateMachine broken by API changes");
//XXX				if(o instanceof ServiceProxyEvent && 
//						((ServiceProxyEvent)o).getServiceProxy().getProviderURN().equals(providerURN)) {
////					System.out.println("got ServiceProxy");
//					amProxy = ((ServiceProxyEvent)o).getServiceProxy();
//
//					return context.fullUpdateState;
//				}else if(o instanceof PeriodicEvent) {
//					//timeout
//					return context.disconnectState;
//				}else if(isDisconnect(o)) {
//					return context.initialState; 
//				}else {
//					return this;
//				}
			}

			public void exitState() {
				stateThread.disablePeriodicEvents();
			}

			public String toString() { return "connectedState";}
		};

		fullUpdateState = new State() {
			UpdateStateMachine context = UpdateStateMachine.this;
			private int count;

			public void enterState() {
				Command cmd = (Command) amProxy.findItem(IAssemblyManager.COMMAND_LOADED_COUNT_IN);
				if(cmd != null) {
					amProxy.addOutCommandReceiver(stateThread);
					stateThread.enablePeriodicEvents(10000);
					cmd.invoke();
				} else {
					System.out.println("UpdateStateMachine: Error: proxy has no count command");
					context.transition(context.disconnectState);
				}
			}

			public State handleEvent(Object o) {
//				System.out.println("FullUpdateState:handleEvent");
				if(o instanceof CommandEvent) {
					Command c = ((CommandEvent)o).getCommand();
					if(c.getID().equals(IAssemblyManager.COMMAND_LOADED_COUNT_OUT)) {
						count = Integer.parseInt(new String(c.getParam(0).getData()));
//						System.out.println("FullUpdateState: count == "+count);

						if(count > 0) {
							Command rc = (Command) amProxy.findItem(IAssemblyManager.COMMAND_READ_LOADED);
							rc.getParam(0).setData(Integer.toString(count-1).getBytes());
							rc.invoke();
						}
					} else if(c.getID().equals(IAssemblyManager.COMMAND_ASSEMBLYINFO)) {
						byte[] xmlData = c.getParam(0).getData();
//						String aiStr = new String(xmlData);
//						System.out.println(aiStr);
						PRDAssemblyVer ai = parseAssemblyXML(xmlData);
//						System.out.println("FullUpdateState: ai == "+ai.getAssemblyLabel());
						doCheckUpdate(ai,((CommandEvent)o).getSource());
						if(--count > 0) {
							Command rc = (Command) amProxy.findItem(IAssemblyManager.COMMAND_READ_LOADED);
							rc.getParam(0).setData(Integer.toString(count-1).getBytes());
							rc.invoke();
						}
					}
					if(count == 0) {
						// keep the connection to the assembly manager
						return context.establishedState;
					}
				}else if(o instanceof PeriodicEvent) {
					//timeout
					return context.disconnectState;
				}else if(isDisconnect(o)) {
					return context.initialState; 
				}
				return this;
			}

			public void exitState() {
				//TODO cleanup: remove listeners, etc.
				stateThread.disablePeriodicEvents();
			}

			public String toString() { return "fullUpdateState";}
		};
		establishedState = new State() {
			UpdateStateMachine context = UpdateStateMachine.this;
			//TODO: move this event handler to a separate thread
			//      (i.e., one thread per remote assembly manager, which
			//       lives for the duration of the connection)
			// sketch:
			public void enterState() {
				// start new event handler thread
				UpdateThread ut = new UpdateThread(UpdateStateMachine.this, deviceContext, providerURN, customerURN);
				deviceContext.getScheduler().scheduleThread(ut);
				amProxy.addOutCommandReceiver(ut);
				amProxy.removeOutCommandReceiver(stateThread);
				deviceContext.getResourceManager2().updatePRDConnectionReceiver(
						new DeviceID(providerURN.getDeviceURN().toString()), ut);
				deviceContext.getResourceManager2().updatePRDConnectionReceiver(
						new DeviceID(customerURN.getDeviceURN().toString()), ut);
				//				deviceContext.getDiscoveryManager().removeConnectionInfoReceiver(providerURN.getDeviceURN(), stateThread);

				// go to initial state
				context.transition(context.initialState);
			}

//			public State handleEvent(Object o) {
//			if(o instanceof CommandEvent) {
//			Command c = ((CommandEvent)o).getCommand();
//			if(c.getID().equals(AssemblyManager.COMMAND_UPDATE)) {
//			System.out.println("Assembly Update Command");
//			byte[] xmlData = c.getParam(0).getData();
//			AssemblyInfo ai = parseAssemblyXML(xmlData);
//			doCheckUpdate(ai);
//			}
//			} else if(o instanceof InfoEvent && 
//			((InfoEvent)o).getInfo() instanceof ConnectionInfo &&
//			!((InfoEvent)o).infoShouldBeKept() &&
//			((ConnectionInfo)((InfoEvent)o).getInfo()).getProviderURN().equals(providerURN) &&
//			((ConnectionInfo)((InfoEvent)o).getInfo()).getCustomerURN().equals(customerURN)) {
//			System.out.println("got disconnected");
//			return context.initialState;
//			}
//			return this;
//			}

			public String toString() { return "establishedState";}
		};

		disconnectState = new State() {
			public void enterState() {
				//TODO more cleanup needed?
				try {
					deviceContext.getRemoteConnectManager().requestDisconnect(new PRDConnection(
							new LocalSID(providerURN.toString()), new LocalSID(customerURN.toString())));
				} catch (IOException e) {
					e.printStackTrace();
				}
				allowURN(providerURN.getDeviceURN());
				transition(initialState);
			}

			public String toString() { return "disconnectState";}
		};

		setCurrentState(initialState);
	}

	void doCheckUpdate(PRDAssemblyVer ai, Object src) {
		//XXX Only handle released (non released versions should not really
		//be broadcast...
		if(!ai.getReleased()) return;

		Assembly tmp = deviceContext.getAssemblyManager().checkIfUpdateToRunning(ai);
		if(tmp != null) {
			//TODO: actually do something here.
			System.out.println("Found update for running assembly "+tmp.assemblyVer.getName()+" : "+tmp.assemblyVer.getLogicalVersion() + "("+ai.getLogicalVersion()+")");
//			if(src instanceof ServiceProxy) {
//				ServiceProxy sp = (ServiceProxy)src;
//				System.out.println("   from "+sp.getURN());
//			}
		}

		PRDAssemblyD tmp2 = deviceContext.getAssemblyManager().checkIfUpdateToStored(ai);
		if(tmp2 != null) {
			//TODO: actually do something here.
			System.out.println("Found update for stored assembly "+tmp2.getName()+" : "+"("+ai.getLogicalVersion()+")");
//			if(src instanceof ServiceProxy) {
//				ServiceProxy sp = (ServiceProxy)src;
//				System.out.println("   from "+sp.getURN());
//			}
		}

	}

	private boolean checkIfURNisAllowed(PRDService si) {
		throw new Error("UpdateStateMachine broken by API changes");
//		boolean result = !ignoreDeviceURNs.contains(si.getDeviceURN());
//		return result;
	}

	public void allowURN(URN deviceURN) {
		ignoreDeviceURNs.remove(deviceURN);
	}

	private boolean isDisconnect(Object o) {
//		System.out.println("isDisconnect?");
		boolean result =  o instanceof InfoEvent && 
		((InfoEvent)o).getInfo() instanceof PRDConnection && 
		!((InfoEvent)o).infoShouldBeKept() &&
		((PRDConnection)((InfoEvent)o).getInfo()).getLocalSID1().equals(new LocalSID(providerURN.toString())) &&
		((PRDConnection)((InfoEvent)o).getInfo()).getLocalSID2().equals(new LocalSID(customerURN.toString()));

//		if(result) {System.out.println("isDisconnect() returns true");}
		return result;
	}

	public static PRDAssemblyVer parseAssemblyXML(byte[] xmlData) {
		AssemblyXMLFactory x = new AssemblyXMLFactory();
		try {
			PRDAssemblyVer ai = (PRDAssemblyVer) x.createFromXMLData(xmlData, 0, xmlData.length);
			return ai;
		} catch (XMLException e) {
			e.printStackTrace();
		}
		return null;
	}
}
