/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.assembly.manager;

import ist.palcom.assemblies.Assembly;
import ist.palcom.assemblies.IResourceManager;
import ist.palcom.assemblies.ResourceRequestEvent;
import ist.palcom.assemblies.ResourceResponseEvent;
import ist.palcom.collections.LinearSet;
import ist.palcom.debugging.DebugDispatcher;
import ist.palcom.device.DeviceContext;
import ist.palcom.device.RemoteConnectHelper;
import ist.palcom.discovery.InfoEvent;
import ist.palcom.info.InfoListener;
import ist.palcom.info.ModifiableInfoHandler;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.AbstractServiceDecl;
import ist.palcom.resource.descriptor.AltServiceDecl;
import ist.palcom.resource.descriptor.AltServiceDeclList;
import ist.palcom.resource.descriptor.Command;
import ist.palcom.resource.descriptor.ConnectionDecl;
import ist.palcom.resource.descriptor.ConnectionDeclList;
import ist.palcom.resource.descriptor.ControlItem;
import ist.palcom.resource.descriptor.DeviceDecl;
import ist.palcom.resource.descriptor.LocalSID;
import ist.palcom.resource.descriptor.PRDAssemblyVer;
import ist.palcom.resource.descriptor.PRDConnection;
import ist.palcom.resource.descriptor.PRDData;
import ist.palcom.resource.descriptor.PRDDevice;
import ist.palcom.resource.descriptor.PRDServiceList;
import ist.palcom.resource.descriptor.ServiceDecl;
import ist.palcom.resource.descriptor.ServiceExp;
import ist.palcom.resource.descriptor.ServiceProxy;
import ist.palcom.resource.descriptor.ServiceUse;
import ist.palcom.resource.descriptor.SingleServiceDecl;
import ist.palcom.resource.descriptor.ThisService;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.resource.descriptor.URNLeaf;
import ist.palcom.services.AbstractService;
import ist.palcom.services.ServiceManagerConstants;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * 
 * A class responsible for providing Assemblies with resources
 * 
 * Currently, only connections are supported: An assembly sends requests
 * for connections, and the ResourceManager responds with an ConnectionInfo,
 * if the connection is possible (i.e., the required services have been
 * discovered)
 * 
 * If established resources go down, dependent assemblies are informed
 * 
 * For alternative resources, if a better alternative appears, dependent assemblies
 * are informed that a change is possible 
 * 
 * @author sven
 * @deprecated - use the multi-threaded version, ResourceManager3
 *
 */
public class ResourceManager { //extends AbstractService implements IResourceManager {

//	/** The default delay between trying to load software components. */
//	static final long DEFAULT_PERIOD = 10000L;
//
//	private ResourceThread resourceThread;
//	
//	private InfoThread infoThread;
//	
//	private ModifiableInfoHandler remoteInfoHandler;
//
////	protected ArrayList pendingConnectionRequests;
////	protected ArrayList establishedConnectionRequests;
//	protected LinearSet pendingConnectionRequests;
//	protected LinearSet establishedConnectionRequests;
//
//	private RemoteConnectHelper rch;
//	
//	public ResourceManager(DeviceContext context, String urnLeaf,
//            String serviceName) throws IOException {
//        super(context, urnLeaf, serviceName, createServiceProxy(serviceName));
//		resourceThread = new ResourceThread(DEFAULT_PERIOD);
//		infoThread = new InfoThread();
//		remoteInfoHandler = new ModifiableInfoHandler();
//		pendingConnectionRequests = new LinearSet();
//		establishedConnectionRequests = new LinearSet();
//		context.getResourceManager2().updatePRDDeviceReceiver(infoThread);
//		context.getScheduler().scheduleThread(resourceThread);
//		context.getScheduler().scheduleThread(infoThread);
//		rch = new RemoteConnectHelper(context);
//	}
//		
//	/* (non-Javadoc)
//	 * @see ist.palcom.assemblies.IResourceManager#start()
//	 */
//	public void start() throws IOException {
//		super.start();
//	}
//	
//	/* (non-Javadoc)
//	 * @see ist.palcom.assemblies.IResourceManager#stop()
//	 */
//	public void stop() throws IOException {
//		resourceThread.terminate();
//		infoThread.terminate();
//		super.stop();
//	}
//
//	/** tryLoadingSoftwareComponent 
//	 * 
//	 * Connect to the SoftwareComponentManager on a device with a missing service
//	 * and attempt to load the service with the specified URN.
//	 */
//	
//	protected void tryLoadingSoftwareComponentAsync(Assembly assembly, PRDAssembly ai, URN urn) {
//		// Lookup device declaration in assembly info
//		DeviceDecl dev = ai.lookupDevice(urn.getDeviceURN());
//		if(dev != null) {
//			URN swcmanurn = dev.getURN().createSubUrn(
//                    new URNLeaf(ServiceManagerConstants.URN_SUFFIX));
//			PRDConnection ci = new PRDConnection(swcmanurn, assembly.getServiceInfo().getLocalSID());
//			PRDConnection current = (PRDConnection) remoteInfoHandler
//			.getInfoThatIsAboutTheSameEntity(ci);
//			// check for or establish connection to the SoftwareComponentManager
//			if (current == null) {
//			    // Establish connection
//			    try {
//                    context.getRemoteConnectManager().requestConnect(ci);
//                } catch (IOException e) {
//                    e.printStackTrace();
//                }
//			    //FIXME: this should be divided into two handlers, the second part
//			    //triggered by discovery of the requested connection...
//			}
//			// Send the load command
//			ServiceProxy sp = assembly.getServiceProxyForUrn(swcmanurn);
//			if (sp != null) {
//			    ControlItem tmp = sp.findItem("load");
//			    if(tmp instanceof Command) {
//			        Command command = (Command) tmp;
//			        command.getParam(0).setData(urn.getString().getBytes());
////			        logMsg("[OUT] trying to load SWC "+urn);
//			        DebugDispatcher.debug("ResourceManager trying to load (async) SWC "+urn, DebugDispatcher.CAT_RESOURCE);
//			        command.invoke();
//			    } else {
//			        DebugDispatcher.debug("tryLoading (async)...:Not a command "+tmp, DebugDispatcher.CAT_RESOURCE);
//			    }
//			}
//            // Close connection
//			try {
//                context.getRemoteConnectManager().requestDisconnect(ci);
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//		}
//	}
//	protected void tryLoadingSoftwareComponent(Assembly assembly, PRDAssembly ai, URN urn) {
//		// Lookup device declaration in assembly info
//		DeviceDecl dev = ai.lookupDevice(urn.getDeviceURN());
//		if(dev != null) {
//			URN swcmanurn = dev.getURN().createSubUrn(
//                    new URNLeaf(ServiceManagerConstants.URN_SUFFIX));
//			PRDConnection ci = new PRDConnection(swcmanurn, assembly.getServiceInfo().getLocalSID());
//			
//			// This call is blocking...
//			rch.doConnect(ci, 5000, 20000);
//			
//			// Send the load command
//			ServiceProxy sp = assembly.getServiceProxyForUrn(swcmanurn);
//			if (sp != null) {
//			    ControlItem tmp = sp.findItem("load");
//			    if(tmp instanceof Command) {
//			        Command command = (Command) tmp;
//			        command.getParam(0).setData(urn.getString().getBytes());
////			        logMsg("[OUT] trying to load SWC "+urn);
//			        DebugDispatcher.debug("ResourceManager trying to load SWC "+urn, DebugDispatcher.CAT_RESOURCE);
//			        command.invoke();
//			    } else {
//			        DebugDispatcher.debug("tryLoading...:Not a command "+tmp, DebugDispatcher.CAT_RESOURCE);
//			    }
//			}
//            // Close connection
//			try {
//                context.getRemoteConnectManager().requestDisconnect(ci);
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//		}
//	}
//
//	
//	/** Aux method: move a request from the established set to the pending set
//	 * 
//	 * @param r -t the request
//	 */
//	private void requestUnsatisfied(ConnectionRequest r) {
//		r.unsatisfied();
//		establishedConnectionRequests.remove(r);
//		pendingConnectionRequests.add(r);
//	}
//
//	/** Aux method: move a request from the pending set to the established set
//	 * 
//	 * @param r -t the request
//	 */
//
//	private void requestSatisfied(ConnectionRequest r) {
//		r.satisfied();
//		pendingConnectionRequests.remove(r);
//		establishedConnectionRequests.add(r);
//	}
//		
//	/**
//	 * A thread that receives ResourceRequests, and tries to establish
//	 * connections when new Services are discovered.
//	 */
//	private class ResourceThread extends PalcomPriorityThread {
//		private long period;
//		
//		ResourceThread(long period) {
//			this.period = period;
//		}
//		
//		public void run() {
//			enablePeriodicEvents(period);
//			while (true) {
//				Event tmpe = waitEvent();
//				if(tmpe instanceof ResourceRequestEvent) {
//					DebugDispatcher.debug("ResourceManager :"+tmpe,DebugDispatcher.CAT_RESOURCE);
//					processResourceRequest((ResourceRequestEvent) tmpe);
//				}else if(tmpe instanceof InfoEvent) {
////					DebugDispatcher.debug("ResourceManager :"+((InfoEvent)tmpe).getInfo(),DebugDispatcher.CAT_RESOURCE);
//					InfoEvent ie = (InfoEvent)tmpe;	
//					if(ie.infoShouldBeKept()) {
////						DebugDispatcher.debug("ResourceThread: infoShouldBeKept", DebugDispatcher.CAT_RESOURCE);						
//						findPossibleConnectionsForInfo(ie);
//					} else {
//						handleResourceDown(ie);
//					}
//				} 
////					else if (tmpe instanceof PeriodicEvent) {
////					//retry periodically by fireing all infos
//////					if(!pendingConnectionRequests.isEmpty()) {
////					DebugDispatcher.debug("ResourceThread.periodicEvent", DebugDispatcher.CAT_RESOURCE);
//////					if(!(pendingConnectionRequests.toArray().length>0)) {					
////					if((pendingConnectionRequests.iterator().hasNext())) {
////						DebugDispatcher.debug("ResourceThread fireAllInfos", DebugDispatcher.CAT_RESOURCE);						
////						remoteInfoHandler.fireAllInfos();
////					}
////				}				
//
////				Param p = statusCmd.getParam(0);
////				byte [] val = statusStr.getBytes();
////				p.setData(val);
////				statusCmd.invoke();
//			}
//		}
//
//		private void handleResourceDown(InfoEvent ie) {
//			ArrayList connectionsDown = new ArrayList();
//			for (Iterator iter = establishedConnectionRequests.iterator(); iter
//					.hasNext();) {
//				ConnectionRequest rre = (ConnectionRequest) iter.next();
//				PRDConnection ci;
//				if((ci = rre.getMatchingConnectionInfo(ie.getInfo())) != null) {
//					//XXX This piece of code re-establishes connections brought
//					//down when a higher priority alternative was found. That has
//					//to be sorted out before this is put back in.
////					if(remoteInfoHandler.connectionIsPossible(ci)) {
////						//Connection still possible, tell Assembly to retry.
////						DebugDispatcher.debug("ResourceManager.handleResourceDown: "+ci+" still possible-RETRY", DebugDispatcher.CAT_RESOURCE);
////						rre.getReceiver().putEvent(new ResourceResponseEvent(ci,ResourceResponseEvent.ADD));
////					} else 
//					{
//						//Some service down, notify assembly
//						//XXX this should (also?) be a contingency event: resource no longer available!
//						rre.getReceiver().putEvent(new ResourceResponseEvent(ci,ResourceResponseEvent.REMOVE));
//					}
//					connectionsDown.add(rre);	
//				}
//			}
//			// Keep trying (at a later point) to reestablish connections that are currently not possible
////			establishedConnectionRequests.removeAll(connectionsDown);
////			pendingConnectionRequests.addAll(connectionsDown);
//			for (Iterator iter2 = connectionsDown.iterator(); iter2.hasNext();) {
//				ConnectionRequest o = (ConnectionRequest) iter2.next();
//				requestUnsatisfied(o);
//			}
//		}
//
//		private void findPossibleConnectionsForInfo(InfoEvent ie) {
//			ArrayList discoveredConnections = new ArrayList();
//			for (Iterator iter = pendingConnectionRequests.iterator(); iter.hasNext();) {
//				ConnectionRequest rre = (ConnectionRequest) iter.next();
//
//				ConnectionRequest tmp = rre.checkAgainstInfo(ie);
//				if(tmp != null) discoveredConnections.add(tmp);			
//			}
//			for (Iterator iter = establishedConnectionRequests.iterator(); iter.hasNext();) {
//				ConnectionRequest rre = (ConnectionRequest) iter.next();
//
//				if(rre.isAlternative()) {
//					ConnectionRequest tmp = rre.checkAgainstInfo(ie);
//					if(tmp != null) discoveredConnections.add(tmp);			
//				}
//			}
//
////			pendingConnectionRequests.removeAll(discoveredConnections);
////			establishedConnectionRequests.addAll(discoveredConnections);
//			for (Iterator iter2 = discoveredConnections.iterator(); iter2.hasNext();) {
//				ConnectionRequest o = (ConnectionRequest) iter2.next();
//				requestSatisfied(o);
//			}
//
//		}
//	}
//	
//	private class InfoThread extends PalcomPriorityThread {
//		public void run() {
//			//XXX Hack: the remoteInfoHandler uses callbacks. Forward events
//			//to the resourceThread
//			remoteInfoHandler.addInfoListener( new InfoListener() {
//
//				public void infoReceived(InfoEvent event) {
//					resourceThread.putEvent(event);
//				}				
//			});
//			while (true) {
//				InfoEvent event = (InfoEvent) waitEvent();
//				remoteInfoHandler.infoReceived(event);
//				if(event.getInfo() instanceof PRDDevice) {
//					String deviceID = ((PRDDevice)event.getInfo()).getDeviceID();
//					context.getResourceManager2().updatePRDServiceListReceiver(new LocalSID(deviceID, null), this);
//					context.getResourceManager2().updatePRDConnectionReceiver(deviceID, this);
//				}
//			}
//		}
//	}
//	
//	/** aux method
//	 * 
//	 * take care of differences between single and alternative declarations
//	 */
//	private void doProcessConnectionDecl(ConnectionDecl connectionDecl, ResourceRequestEvent rre) {
//		ServiceExp prov = connectionDecl.getProvider();
//		ServiceExp cust = connectionDecl.getCustomer();
//		try {
//			DebugDispatcher.debug("ResourceManager.doProcessConnectionDecl: "+connectionDecl+"("+rre.isActive()+")", DebugDispatcher.CAT_RESOURCE);
//			if(rre.isActive()) {
//				if(servicesAreSingle(prov, cust)) {
//					pendingConnectionRequests.add(new SingleConnectionRequest(connectionDecl, rre.getAssembly(), rre.getReceiver()));
//				} else {
//					pendingConnectionRequests.add(new AltConnectionRequest(connectionDecl, rre.getAssembly(), rre.getReceiver()));
//				}
//			} else {
//					cancelConnectionRequest(connectionDecl);
//			} 
//		} catch(IllegalArgumentException e) {
//			DebugDispatcher.debug("ResourceManager.doProcessConnectionDecl: "+e.getMessage(), DebugDispatcher.CAT_RESOURCE);
//		} catch(Exception e) {
//			DebugDispatcher.debug("ResourceManager.doProcessConnectionDecl: ("+prov+", "+cust+")"+e, DebugDispatcher.CAT_RESOURCE);
//			e.printStackTrace();
//		}
//	}
//
//	protected void processResourceRequest(ResourceRequestEvent rre) {
//		PRDData d = rre.getDeclaration();
//    	if(d instanceof ConnectionDecl) {
//    		doProcessConnectionDecl((ConnectionDecl)d, rre);
//    	} else if( d instanceof ConnectionDeclList) {
////    		System.out.println("ResourceManager.addResourceRequest:"+d);
//    		ConnectionDeclList cdl = (ConnectionDeclList) d;
//			for(int i=0; i<cdl.getNumConnectionDecl(); i++){
//	    		ConnectionDecl connectionDecl = cdl.getConnectionDecl(i);
//	    		doProcessConnectionDecl(connectionDecl, rre);
//			}
//    	} else {
//    		DebugDispatcher.debug("ResourceManager.addResourceRequest: unknown type:"+d, DebugDispatcher.CAT_RESOURCE);
//    	}
////		System.out.println("pendingConnectionRequests:");
////    	for (Iterator iter = pendingConnectionRequests.iterator(); iter.hasNext();) {
////			Object element = (Object) iter.next();
////			System.out.println(element);
////		}
//		// We want events also for already discovered services
//    	//Only do this if a request was actually added
//		remoteInfoHandler.fireAllInfos();
//	}
//
//	private void cancelConnectionRequest(ConnectionDecl decl) {
//		ArrayList removeList = new ArrayList();
//		for (Iterator iter = pendingConnectionRequests.iterator(); iter.hasNext();) {
//			ConnectionRequest req = (ConnectionRequest) iter.next();
//			if(req.getDeclaration().equals(decl)) {
//				//it should only occur once, but just to be safe we continue
//				//iterating over the whole list
//				removeList.add(req);
//			}
//		}
////		pendingConnectionRequests.removeAll(removeList);
//		for (Iterator iter2 = removeList.iterator(); iter2.hasNext();) {
//			ConnectionRequest o = (ConnectionRequest) iter2.next();
//    		DebugDispatcher.debug("ResourceManager.cancelConnectionRequest(pending):"+o, DebugDispatcher.CAT_RESOURCE);
//			pendingConnectionRequests.remove(o);
//		}
//
//		removeList.clear();
//		for (Iterator iter = establishedConnectionRequests.iterator(); iter.hasNext();) {
//			ConnectionRequest req = (ConnectionRequest) iter.next();
//			if(req.getDeclaration().equals(decl)) {
//				removeList.add(req);
//			}
//		}
////		establishedConnectionRequests.removeAll(removeList);
//		for (Iterator iter2 = removeList.iterator(); iter2.hasNext();) {
//			ConnectionRequest o = (ConnectionRequest) iter2.next();
//    		DebugDispatcher.debug("ResourceManager.cancelConnectionRequest(established):"+o, DebugDispatcher.CAT_RESOURCE);
//			establishedConnectionRequests.remove(o);
//			o.unsatisfied();
//		}
//	}
//	
//	/* (non-Javadoc)
//	 * @see ist.palcom.assemblies.IResourceManager#servicesAreSingle(ist.palcom.communication.ast.ServiceExp, ist.palcom.communication.ast.ServiceExp)
//	 */
//	public boolean servicesAreSingle(ServiceExp prov, ServiceExp cust) {
//		boolean provIsSingle = serviceExpRefersToSingle(prov);
//		boolean custIsSingle = serviceExpRefersToSingle(cust);
//		return provIsSingle && custIsSingle;
//	}
//
//	/* (non-Javadoc)
//	 * @see ist.palcom.assemblies.IResourceManager#serviceExpRefersToSingle(ist.palcom.communication.ast.ServiceExp)
//	 */
//	public boolean serviceExpRefersToSingle(ServiceExp se) {
//		boolean seIsSingle=false;;
//		if(se instanceof ThisService) {
//			DebugDispatcher.debug("ResourceManager.serviceExpRefersToSingle: this=true", DebugDispatcher.CAT_RESOURCE);
//			seIsSingle = true;
//		} else if(se instanceof ServiceUse) {
//			seIsSingle = serviceUseRefersToSingle((ServiceUse)se);
//			DebugDispatcher.debug("ResourceManager.serviceExpRefersToSingle:"+se.toString()+"="+seIsSingle, DebugDispatcher.CAT_RESOURCE);
//		} else {
//			DebugDispatcher.debug("ResourceManager.serviceExpRefersToSingle:"+se.toString()+" not this or ServiceExp?", DebugDispatcher.CAT_RESOURCE);			
//		}
//		return seIsSingle;
//	}
//	
//	private boolean serviceUseRefersToSingle(ServiceUse svc) {
//		if(svc != null) {
//			ServiceDecl decl = svc.getDecl();
//			if(decl != null) {
//				AbstractServiceDecl inner = decl.getDecl();
//				return inner instanceof SingleServiceDecl;
//			} else {
//				DebugDispatcher.debug("ResourceManager.serviceUseRefersToSingle: no declaration found for "+svc, DebugDispatcher.CAT_RESOURCE);
//				throw new IllegalArgumentException("No declaration found for Service "+svc.getIDforEditing());
//			}
//		}
//		return false;
//	}
//
//	private static ServiceProxy createServiceProxy(String controlName) {
//        ServiceProxy sd = new ServiceProxy(controlName);
//        return sd;
//    }
//
//	/* (non-Javadoc)
//	 * @see ist.palcom.assemblies.IResourceManager#putEvent(ist.palcom.assemblies.ResourceRequestEvent)
//	 */
//	public void putEvent(ResourceRequestEvent event) {
//		resourceThread.putEvent(event);
//	}
//	
//	/** 
//	 * Notice object for connection requests 
//	 */
//	private abstract class ConnectionRequest {
//		private ConnectionDecl cd;
//		private Assembly assembly;
//		private PalcomThread recv;
//		
//		ConnectionRequest(ConnectionDecl cd, Assembly assembly, PalcomThread recv) {
//			this.cd = cd;
//			this.assembly = assembly;
//			this.recv = recv;
//		}
//		public abstract void satisfied();
//		
//		public abstract void unsatisfied();
//		
//		public abstract PRDConnection getMatchingConnectionInfo(PRDData info);
//		
//		ConnectionDecl getDeclaration() {
//			return cd;
//		}
//		
//		Assembly getAssembly() {
//			return assembly;
//		}
//		
//		PalcomThread getReceiver() {
//			return recv;
//		}
//
//		/**
//		 * Checks if an InfoEvent affects this ConnectionRequest
//		 * 
//		 * @param ie - the InfoEvent to check against
//		 * @return a reference to a ConnectionRequest that has now been fulfilled, or null 
//		 */
//
//		public abstract ConnectionRequest checkAgainstInfo(InfoEvent ie);
//		
//		public abstract boolean isAlternative();
//		
//		public String toString() {
//			return this.getClass().toString() + ":"+getDeclaration().toString();
//		}
//	}
//	
//	private class SingleConnectionRequest extends ConnectionRequest {
//		
//		SingleConnectionRequest(ConnectionDecl cd, Assembly assembly, PalcomThread recv) {
//			super(cd, assembly, recv);
//		}
//
//		/** Aux method
//		 * 
//		 * @param rr The Resource request to be examined
//		 * @param sl The disovered service list
//		 */
//		private void fireConnectionResponseIfPossible(PRDServiceList sl) {
//			for(int i=0; i< sl.getNumPRDServiceListItem(); i++) {
//				URN discoveredURN = sl.getPRDServiceListItem(i).getURN();
//				ConnectionDecl cd = (ConnectionDecl) getDeclaration();
////				DebugDispatcher.debug("fireConnectionResponseIfPossible:"+discoveredURN+ "("+cd.getProvider().getURN()+" | "+cd.getCustomer().getURN()+")", DebugDispatcher.CAT_RESOURCE);
//				
//				// ConnectionDecl includes discovered service
//				if((cd.getProvider().getURN().equals(discoveredURN) || 
//						cd.getCustomer().getURN().equals(discoveredURN)) ) {
////					DebugDispatcher.debug("***Match", DebugDispatcher.CAT_RESOURCE);
//
//					PRDConnection ci = cd.asInfo();
//					// Look up the current connectionInfo
//					PRDConnection current = (PRDConnection) remoteInfoHandler
//					.getInfoThatIsAboutTheSameEntity(ci);
//					
//					if (current == null) {
////									statusStr = statusStr+"\n"+"[**] "+cd.getProvider()+" --> "+cd.getCustomer();
////						System.out.println("[NO] "+cd.getProvider()+" --> "+cd.getCustomer());
//						if(remoteInfoHandler.connectionIsPossible(ci)) {
////							DebugDispatcher.debug("fireConnectionResponseIfPossible: putEvent...", DebugDispatcher.CAT_RESOURCE);
//							getReceiver().putEvent(new ResourceResponseEvent(ci,ResourceResponseEvent.ADD));
//						} else {
////							DebugDispatcher.debug("connection not possible --- strange..."+ci, DebugDispatcher.CAT_RESOURCE);							
//						}
//					} else {
////						DebugDispatcher.debug("current != null, we should be connected...", DebugDispatcher.CAT_RESOURCE);													
//					}
//				}
//			}
//		}
//
//
//		
//		public ConnectionRequest checkAgainstInfo(InfoEvent ie) {
//			ConnectionRequest result = null;
//			if((ie.getInfo() instanceof PRDServiceList))  {
//				PRDServiceList sl = (PRDServiceList) ie.getInfo();
//				fireConnectionResponseIfPossible(sl);
//			} else if(ie.getInfo() instanceof PRDConnection) {
//				ConnectionDecl cd = (ConnectionDecl) getDeclaration();
//				// A pending connection has been discovered
//				if(cd.asInfo().equals(ie.getInfo())) {
////					discoveredConnections.add(this);
//					result = this;
//				}
//			} else if ( (ie.getInfo() instanceof PRDDevice)) {
//				// For a new device, if we have a pending requests for a service on that
//				// device, which has not been discovered, we attempt to contact its 
//				// SoftwareComponentManager and see if it has a matching URL.
//				DebugDispatcher.debug("checkAgainstInfo..."+ie.getInfo()+"  [DEVICE]", DebugDispatcher.CAT_RESOURCE);
//				ConnectionDecl cd = (ConnectionDecl) getDeclaration();
//				ServiceExp prov = cd.getProvider();
//
//				if(prov instanceof ServiceUse) {
//					ServiceDecl provDecl = ((ServiceUse)prov).getDecl();
//					if(provDecl != null) {
//						SingleServiceDecl asd = (SingleServiceDecl) provDecl.getDecl();
//						if(asd.getURN().getDeviceURN().equals(new URN(((PRDDevice)ie.getInfo()).getDeviceID()))) {
//							//FIXME: is this a good idea:
//							DebugDispatcher.debug("checkAgainstInfo..."+ie.getInfo()+"  try loading...", DebugDispatcher.CAT_RESOURCE);							
//							PRDAssembly assemblyInfo=cd.ai();
//							tryLoadingSoftwareComponent(getAssembly(),assemblyInfo,asd.getURN());
//						}
//					}
//				}
//			} else if( ie.getInfo() instanceof PRDConnection) {  
//				DebugDispatcher.debug("SingleConnectionRequest:got ConnectionInfo:"+ie.getInfo(),DebugDispatcher.CAT_RESOURCE);
//				if( ((PRDConnection)ie.getInfo()).getProviderURN().getString().contains(ServiceManagerConstants.URN_SUFFIX)) {
//					DebugDispatcher.debug("SingleConnectionRequest:got ConnectionInfo for SWCMan:"+ie.getInfo(),DebugDispatcher.CAT_RESOURCE);
//					ConnectionDecl cd = (ConnectionDecl) getDeclaration();
//					ServiceExp prov = cd.getProvider();
//					ServiceDecl provDecl = ((ServiceUse)prov).getDecl();
//					if(provDecl != null) {
//						SingleServiceDecl asd = (SingleServiceDecl) provDecl.getDecl();
//						//FIXME: is this a good idea:
//						PRDAssembly assemblyInfo=cd.ai();
//						tryLoadingSoftwareComponent(getAssembly(),assemblyInfo,asd.getURN());
//					}
//				}
//			}
//			return result;
//		}
//
//		public PRDConnection getMatchingConnectionInfo(PRDData info) {
//			PRDConnection ci = getDeclaration().asInfo();
//			if( info.equals(ci)) {
//				return ci;
//			}
//			return null;
//		}
//
//		public boolean isAlternative() {
//			return false;
//		}
//
//		public void satisfied() {
//		}
//
//		public void unsatisfied() {
//		}
//	}
//	
//	private class AltConnectionRequest extends ConnectionRequest {
//		
//		private AltServiceDecl currentAlternative;
//		private PRDConnection currentConnectionInfo;
//
//		AltConnectionRequest(ConnectionDecl cd, Assembly assembly, PalcomThread recv) {
//			super(cd, assembly, recv);
//		}
//
//		/** Aux method	
//		 * 
//		 * @param rr The Resource request to be examined
//		 * @param sl The disovered service list
//		 */
//		private synchronized boolean fireConnectionResponseIfPossible(AltServiceDecl asd, PRDServiceList sl, boolean altProvider) {
//			boolean result = false;
//			for(int i=0; i< sl.getNumPRDServiceListItem(); i++) {
//				URN discoveredURN = sl.getPRDServiceListItem(i).getURN();
//				URN asdURN = asd.getURN();
//				// ConnectionDecl includes discovered service
//				DebugDispatcher.debug("AltConnectionRequest.fireConnectionResponseIfPossible: disovered service:"+discoveredURN, DebugDispatcher.CAT_RESOURCE);
//				if((asdURN.equals(discoveredURN) || 
//						asdURN.equals(discoveredURN)) ) {
//					DebugDispatcher.debug("...matches "+asd.getAssemblyLabel(),DebugDispatcher.CAT_RESOURCE);
//					PRDConnection ci = connectionInfoForASD(altProvider, getDeclaration(), asd);
//					// Look up the current connectionInfo
//					PRDConnection current = (PRDConnection) remoteInfoHandler
//					.getInfoThatIsAboutTheSameEntity(ci);
//					
//					if (current == null) {
////									statusStr = statusStr+"\n"+"[**] "+cd.getProvider()+" --> "+cd.getCustomer();
////						System.out.println("[NO] "+cd.getProvider()+" --> "+cd.getCustomer());
//						if(remoteInfoHandler.connectionIsPossible(ci)) {
//							if(currentAlternative == null) {
//								DebugDispatcher.debug("AltConnectionRequest.fireConnectionResponseIfPossible: no previous alternative:", DebugDispatcher.CAT_RESOURCE);
//								currentAlternative = asd;
//								currentConnectionInfo = ci;
//								getReceiver().putEvent(new ResourceResponseEvent(ci,ResourceResponseEvent.ADD));
//							} else {
//								DebugDispatcher.debug("AltConnectionRequest.fireConnectionResponseIfPossible: previous priority:"+currentAlternative.getPriority()+" ["+asd.getPriority()+"]", DebugDispatcher.CAT_RESOURCE);
//								if(asd.getPriority() < currentAlternative.getPriority()) {
//									PRDConnection oldCi = currentConnectionInfo;
//									currentAlternative = asd;
//									currentConnectionInfo = ci;
//									getReceiver().putEvent(new ResourceResponseEvent(ci, oldCi,ResourceResponseEvent.CHANGE));								
//								}
//							}
//							result =true;
//						} 
//					}
//				}
//			}
//			return result;
//		}
//		
//		public ConnectionRequest checkAgainstInfo(InfoEvent ie) {
//			ConnectionRequest result = null;
//			AltServiceDeclList asdl = null;
//			boolean altProvider = false;
//			
//			ConnectionDecl cd = getDeclaration();
//			try {
//				if(!serviceExpRefersToSingle(cd.getProvider())) {
//					asdl = (AltServiceDeclList)cd.getProvider().getDecl().getDecl();
//					altProvider = true;
//				} else if(!serviceExpRefersToSingle(cd.getCustomer())) {
//					//use else if just in case: one has to be true, or else
//					//this should be a SingleConnectionRequest: Perhaps add a
//					//check for this in the constructor?
//					asdl = (AltServiceDeclList)cd.getCustomer().getDecl().getDecl();
//					altProvider = false;
//				}
//			} catch (Exception e) {
//				DebugDispatcher.debug("AltConnectionRequest.checkAgainstInfo:"+e.toString(), DebugDispatcher.CAT_RESOURCE);
//			}
//			// just in case
//			if(asdl==null) {
//				DebugDispatcher.debug("AltConnectionRequest.checkAgainstInfo: adsl == null", DebugDispatcher.CAT_RESOURCE);
//				return null;
//			}
////			for(int i=0; i<asdl.getNumServiceDecl();i++) {
////				AltServiceDecl asd = asdl.getServiceDecl(i);
//			ArrayList al = asdl.getChildrenInPriorityOrder();
//			for (Iterator iter = al.iterator(); iter.hasNext();) {
//				AltServiceDecl asd = (AltServiceDecl) iter.next();
//			
//				if((ie.getInfo() instanceof PRDServiceList))  {
//					PRDServiceList sl = (PRDServiceList) ie.getInfo();
//					if(fireConnectionResponseIfPossible(asd, sl, altProvider))
//						break;
//				} else if(ie.getInfo() instanceof PRDConnection) {
//					PRDConnection asdInfo;
//					asdInfo = connectionInfoForASD(altProvider, cd, asd);
//					// A pending connection has been discovered
//					if(asdInfo.equals(ie.getInfo())) {
//						result = this;
//						break;
//					}
//				}
//			}
//			return result;
//		}
//		// XXX There's some redundant code  here (related to the .asInfo() equivalent).
//
//		private PRDConnection connectionInfoForASD(boolean altProvider, ConnectionDecl cd, AltServiceDecl asd) {
//			PRDConnection asdInfo;
//			if(altProvider) {
//				asdInfo = new PRDConnection(asd.getURN(), cd.getCustomer().getURN());
//			} else {
//				asdInfo = new PRDConnection(cd.getProvider().getURN(), asd.getURN());						
//			}
//			return asdInfo;
//		}
//		
//		public PRDConnection getMatchingConnectionInfo(PRDData info) {
//			ConnectionDecl cd = getDeclaration();
//			AltServiceDeclList asdl=null;
//			boolean altProvider=false;;
//			if(!serviceExpRefersToSingle(cd.getProvider())) {
//				asdl = (AltServiceDeclList)cd.getProvider().getDecl().getDecl();
//				altProvider = true;
//			} else if(!serviceExpRefersToSingle(cd.getCustomer())) {
//				//use else if just in case: one has to be true, or else
//				//this should be a SingleConnectionRequest: Perhaps add a
//				//check for this in the constructor?
//				asdl = (AltServiceDeclList)cd.getCustomer().getDecl().getDecl();
//				altProvider = false;
//			} else {
//				// This is an error
//				DebugDispatcher.debug("AltConnectionRequest.checkAgainstInfo: neither provider nor customer is alternative???",DebugDispatcher.CAT_RESOURCE);
//				return null;
//			}
//			//XXX HERE BE DRAGONS!
//			//If there is a currently active alternative, only match against that
//			//(This is to avoid triggering duplicate alternatives. In particular, we
//			//don't care about "replaced" connections going down.)
////			if(currentAlternative!=null) {
////				ConnectionInfo ci = connectionInfoForASD(altProvider, cd, currentAlternative);
//			if(currentConnectionInfo != null) {
//				if(currentConnectionInfo.equals(info))
//					return currentConnectionInfo;				
//			}else{
//				for(int i=0; i<asdl.getNumServiceDecl();i++) {
//					AltServiceDecl asd = asdl.getServiceDecl(i);
//					PRDConnection ci = connectionInfoForASD(altProvider, cd, asd);
//					if(ci.equals(info))
//						return ci;
//				}
//			} 
//			return null;
//		}
//
//		public boolean isAlternative() {
//			return true;
//		}
//
//		public synchronized void satisfied() {
//		}
//
//		public synchronized void unsatisfied() {
//			currentAlternative = null;
//			currentConnectionInfo = null;
//		}
//	}
}
