/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.base.storage;

import java.io.IOException;
import java.io.RandomAccessFile;

public class RandomAccessFileWrapper {
	private RandomAccessFile raf = null;
	private FileWrapper file = null;
	
	public RandomAccessFileWrapper(FileWrapper pfile, int mode) throws IOException {
		file = pfile;
		if(open(pfile, mode) < 0) {
			throw new IOException("Could not open RandomAccessFile");
		}
	}
	
	public int open(FileWrapper pfile, int mode) {
		try { 
			raf = new RandomAccessFile(pfile.getFile(), "rw"); 
		} catch (IOException e) {return -1;}
		return 0;
	}

	public void close() throws IOException  {
		raf.close(); 
	}
	
	public long getFilePointer() throws IOException  {
		return raf.getFilePointer();
	}
	
	public void seek(long pos) throws IOException  {
		raf.seek(pos); 
	}
	
	public int read()  throws IOException {
		return raf.read(); 
	}
	
	public void write(int b) throws IOException {
		raf.write(b); 
	}
	
	public int read(String file, byte b[], int off, int len)  throws IOException {
		return raf.read(b, off, len);
	}
	
	public void write(String file, byte b[], int off, int len) 
	throws IOException 
	{
		raf.write(b, off, len); 
	}
	
	public long length() throws IOException {
		return raf.length();
	}
	
	public boolean delete() {
		return file.getFile().delete();
	}
}
