/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.collections;

import java.util.ArrayList;
import java.util.Iterator;

/** A simple map 
 * 
 * @author sven
 *
 */
public class LinearMap {

	private ArrayList notices;
	
	public LinearMap() {
		notices = new ArrayList();
	}
	
	public boolean containsKey(Object key) {
		for (Iterator iter = notices.iterator(); iter.hasNext();) {
			Notice n = (Notice) iter.next();
			if(n.key.equals(key)) {
				return true;
			}
		}
		return false;
	}

	private Notice findNotice(Object key) {
		for (Iterator iter = notices.iterator(); iter.hasNext();) {
			Notice n = (Notice) iter.next();
			if (key == null && n.key == null) {
			    return n;
            }
            if(n.key != null && n.key.equals(key)) {
				return n;
			}
		}
		return null;
	}	

	public Object get(Object key) {
		Notice n = findNotice(key);
		if(n!=null)
			return n.val;
		else
			return null;
	}

	public LinearSet keySet() {
		LinearSet s = new LinearSet();
		for (Iterator iter = notices.iterator(); iter.hasNext();) {
			Notice n = (Notice) iter.next();
			s.add(n.key);
		}
		return s;
	}

	public ArrayList values() {
		ArrayList s = new ArrayList();
		for (Iterator iter = notices.iterator(); iter.hasNext();) {
			Notice n = (Notice) iter.next();
			s.add(n.val);
		}
		return s;
	}

	
	public void put(Object key, Object val) {
		Notice n = findNotice(key);
		if(n!=null)
			n.val = val;
		else
			notices.add(new Notice(key,val));
	}

	public Object remove(Object key) {
		Object result = null;
		Notice n = findNotice(key);
		if(n!=null) {
			result = n.val;
			notices.remove(n);
		}
		return result;
	}

	private class Notice {
		
		Notice(Object k, Object v) {
			key=k;
			val=v;
		}
		Object key;
		Object val;
	}

	public void clear() {
		notices = new ArrayList();
	}

	public int size() {
		return notices.size();
	}

	public boolean isEmpty() {
		return notices.isEmpty();
	}
}
