/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.collections;

public class LinkedList {
    private Link first = null;

    private Link last = null;

    public Link getFirst() {
        return first;
    }

    public Link getLast() {
        return last;
    }

    public Link removeFirst() {
        if (first == null) {
            return null;
        } else {
            Link result = first;
            remove(first);
            return result;
        }
    }

    public void insertAfter(Link newLink, Link linkInList) {
        newLink.insertAfter(linkInList);
        if (last == linkInList) {
            last = newLink;
        }
    }

    public void remove(Link link) {
        if (link == first) {
            first = first.getNext();
        }
        if (link == last) {
            last = last.getPrev();
        }
        link.out();
    }

    public void insertFirst(Link link) {
        if (isEmpty()) {
            first = link;
            last = link;
        } else {
            link.insertBefore(first);
            first = link;
        }
    }

    public void insertLast(Link link) {
        if (isEmpty()) {
            first = link;
            last = link;
        } else {
            link.insertAfter(last);
            last = link;
        }
    }

    public void clear() {
        while (!isEmpty()) {
            removeFirst();
        }
    }

    public boolean isEmpty() {
        return first == null;
    }

    public int getSize() {
        int size = 0;
        Link link = getFirst();
        while (link != null) {
            size++;
            link = link.getNext();
        }
        return size;
    }
}