/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.device;

import ist.palcom.communication.CommunicationManager;
import ist.palcom.connections.ConnectionsManager;
import ist.palcom.discovery.DiscoveryManager;
import ist.palcom.hgraph.manager.HgraphManager;
import ist.palcom.hgraph.IHgraphNode;
import ist.palcom.mal.MediaManager;
import ist.palcom.messaging.PointToPointManager;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.pubsub.PubSubManager;
import ist.palcom.resource.descriptor.LocalSID;
import ist.palcom.resource.descriptor.PRDDevice;
import ist.palcom.resource.descriptor.PRDSubList;
import ist.palcom.resource.descriptor.ServiceID;
import ist.palcom.resource.manager.ResourceManager;
import ist.palcom.resourceAddressing.ResourceAddressingManager;
import ist.palcom.routing.RoutingManager;

import java.util.ArrayList;
import java.util.Iterator;

public interface DeviceContext extends IHgraphNode{

    PRDDevice getDeviceInfo();

    /** @return An iterator over the AbstractServices on the device. */
    Iterator serviceIterator();

    /** Set the time interval between heartbeats. 
	 * This method has to be called in order for the device to send out heartbeats requesting answers,
	 * otherwise it will only send out heartbeats in reply to other devices.
	 */
    void setHeartBeatPeriod(long period);
    
    /** @return A, for this serviceID, unique instance number. */
    public String getNewInstanceNumber(ServiceID serviceID);
    
    /** @return A new unique string. */
    public String getNewLocalSIDString();
    
    /** @return A new unique LocalSID. */
    public LocalSID getNewLocalSID();
    
    void setName(String name);

    String getName();

    void setDeviceIO(IDeviceIO io);

    IDeviceIO getDeviceIO();

    /**
     * Adds a service to the device. Will add info about the service to the info
     * of the device. The DiscoveryManager will be notified if the service is
     * active.
     */
    void addServiceAtDeviceLevel(Service service);

    /**
     * Adds a service to a service list identified by listLocalSID. 
     * Will add info about the service to the info of the device. 
     * The DiscoveryManager will be notified if the service is active.
     * If there are no list with listLocalSID nothing happens.
     */
    void addServiceToList(Service service, LocalSID listLocalSID);
    
    /**
     * Adds subList to the service list idenfitied by listLocalSID. ArrayList services is a list
     * containing objects of Service. A PRDServiceList will be created pointing at subList and the 
     * services will be added to it.
     * Use a listLocalSID with deviceID and null as servicePartID to get the top level list.
     * If there are no list with listLocalSID nothing happens.
     */
    void addListOfServicesToList(LocalSID listLocalSID, ArrayList services, PRDSubList subList);
    
    /**
     * EXPERIMENTAL! Removes a service from the device, and notify the
     * DiscoveryManager.
     */
    void removeService(Service service);

    PalcomScheduler getScheduler();

    MediaManager getMediaManager();

    PointToPointManager getPointToPointManager();

    ConnectionsManager getConnectionsManager();

    PubSubManager getPubSubManager();

    DiscoveryManager getDiscoveryManager();
    
    ResourceManager getResourceManager2();

    RemoteConnectManager getRemoteConnectManager();
    
    CommunicationManager getCommunicationManager();
    
    RoutingManager getRoutingManager();
    
    HgraphManager getHgraphManager();
    
    ResourceAddressingManager getResourceAddressingManager();

	public void addListOfServicesAtDeviceLevel(ArrayList newServices, PRDSubList subList);

	public void removeSubList(PRDSubList subList);    
}