/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.device;

import java.io.IOException;

import ist.palcom.resource.descriptor.PRDConnection;
import ist.palcom.discovery.InfoEvent;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.Monitor;
import ist.palcom.palcomthreads.PalcomPriorityThread;

/**
 * 
 * A helper class to provide a blocking method for requesting remote
 * connections.
 * 
 * @author sven
 *
 */
public class RemoteConnectHelper extends Monitor{
	
	private Condition result;
	private Condition free;
	private boolean success;
	private ConnectHelperThread cht;
	private DeviceContext context;
	
	public RemoteConnectHelper(DeviceContext dc) {
		super(dc.getScheduler());
		this.context = dc;
		result = new Condition();
		free = new Condition();
	}

	public boolean doConnect(PRDConnection ci, long period, long timeout) {
		enterMonitor();
		while(cht != null) { // while stmt just in case...
			//wait for previous connection to finish
			//(we currently only handle one request at a time)
			free.await();
		}
		setupHelperThread(ci, period, timeout);
		success=false;
		result.await();
		boolean retVal = success;
		terminateHelperThread();
		exitMonitor();
		return retVal;
	}
	
	private void setSuccess(boolean val) {
		enterMonitor();
		success=val;
		result.causeAll(); //just in case, should be exactly one blocked thread. 
		exitMonitor();
	}
	
	private void setupHelperThread(PRDConnection ci, long period, long timeout) {
		cht = new ConnectHelperThread(ci, period, timeout);
		context.getScheduler().scheduleThread(cht);
	}

	private void terminateHelperThread() {
		cht.terminate();
		if(!success) {
			//TODO perhaps send disconnect request, just in case.
//			context.getRemoteConnectThread().putEvent(new InfoEvent(ci,false,true));
		}
		cht=null;
		free.causeOne();
	}
	
	private class ConnectHelperThread extends PalcomPriorityThread {
		private PRDConnection ci;
		private long timeout;
		private long period;
		ConnectHelperThread(PRDConnection ci, long period, long timeout) {
			this.ci = ci;
			this.timeout = timeout;
			this.period = period;
			context.getResourceManager2().updatePRDConnectionReceiver(ci.getDeviceID(), this);
			try {
                context.getRemoteConnectManager().requestConnect(ci);
            } catch (IOException e) {
                e.printStackTrace();
            }
		}
		
		public void run() {
			try {
				int t = ist.palcom.base.System.instance().time();
//				long t = java.lang.System.currentTimeMillis();
				enablePeriodicEvents(period);
//				while(java.lang.System.currentTimeMillis()-t < timeout) {
				while(ist.palcom.base.System.instance().time() -t < timeout) {
//					System.out.println("ConnectHelperThread.run");
					Event e = waitEvent();
//					System.out.println("ConnectHelperThread.got event:"+e);
					if(e instanceof InfoEvent) {
						InfoEvent ie = (InfoEvent)e;
						// TODO: rewrite with virtual method
						if(ie.getInfo() instanceof PRDConnection) {
							PRDConnection tmp = (PRDConnection) ie.getInfo();
							if(tmp.equals(ci)) {
								setSuccess(true);
								disablePeriodicEvents();
								return;
							}
						}
					} else {
						//retry periodically
						try {
                            context.getRemoteConnectManager().requestConnect(ci);
                        } catch (IOException e1) {
                            e1.printStackTrace();
                        }
					}
				}
				setSuccess(false);
				disablePeriodicEvents();
				return;
			} finally {
				context.getResourceManager2().removePRDConnectionReceiver(ci.getDeviceID(), this);
			}
		}
	}
}
