/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.discovery;

import ist.palcom.resource.descriptor.LocalServiceIDReply;
import ist.palcom.resource.descriptor.LocalServiceIDRequest;
import ist.palcom.resource.descriptor.PRDConnectionListReply;
import ist.palcom.resource.descriptor.PRDConnectionListRequest;
import ist.palcom.resource.descriptor.PRDDeviceReply;
import ist.palcom.resource.descriptor.PRDDeviceRequest;
import ist.palcom.resource.descriptor.PRDServiceFMDescriptionReply;
import ist.palcom.resource.descriptor.PRDServiceFMDescriptionRequest;
import ist.palcom.resource.descriptor.PRDServiceListReply;
import ist.palcom.resource.descriptor.PRDServiceListRequest;
import ist.palcom.resource.descriptor.SelectorReply;
import ist.palcom.resource.descriptor.SelectorRequest;
import ist.palcom.resource.descriptor.ServiceInstanceIDReply;
import ist.palcom.resource.descriptor.ServiceInstanceIDRequest;
import ist.palcom.resource.descriptor.ServiceStatusReply;
import ist.palcom.resource.descriptor.ServiceStatusRequest;
import ist.palcom.resource.function.FunctionInfoXMLFactory;
import ist.palcom.xml.XMLException;
import ist.palcom.xml.XMLRepresentable;

public class DiscoveryInfoXMLFactory extends FunctionInfoXMLFactory{

    protected XMLRepresentable createFromTagName(String tagName) throws XMLException {

    	if (tagName.equals("SLQ")) {
    		return new PRDServiceListRequest();
    	} else if (tagName.equals("SLA")) {
    		return new PRDServiceListReply();
    	} else if (tagName.equals("SDQ")) {
    		return new PRDServiceFMDescriptionRequest();
    	} else if (tagName.equals("SDA")) {
    		return new PRDServiceFMDescriptionReply();
    	} else if (tagName.equals("CLA")) {
    		return new PRDConnectionListReply();
    	} else if (tagName.equals("CLQ")) {
    		return new PRDConnectionListRequest();
    	} else if (tagName.equals("InfoEvent")) {
    		return new InfoEvent();
    	} else if (tagName.equals("DQ")) {
    		return new PRDDeviceRequest();
    	} else if (tagName.equals("DA")) {
        	return new PRDDeviceReply();
    	} else if (tagName.equals("SelQ")) {
    		return new SelectorRequest();
    	} else if (tagName.equals("SelA")) {
        	return new SelectorReply();
        } else if (tagName.equals("SIIQ")) {
        	return new ServiceInstanceIDRequest();
        } else if (tagName.equals("SIIA")) {
        	return new ServiceInstanceIDReply();
        } else if (tagName.equals("LSIQ")) {
        	return new LocalServiceIDRequest();
        } else if (tagName.equals("LSIA")) {
        	return new LocalServiceIDReply();
        } else if (tagName.equals("SSQ")) {
        	return new ServiceStatusRequest();
        } else if (tagName.equals("SSA")) {
        	return new ServiceStatusReply();
        } else {
            return super.createFromTagName(tagName);
        }
    }
}
