/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.discovery;

import ist.palcom.resource.descriptor.AbstractXMLRepresentable;
import ist.palcom.resource.descriptor.PRDData;
import ist.palcom.palcomthreads.Event;
import ist.palcom.xml.XMLRepresentable;

import java.io.IOException;

import org.kxml2.io.KXmlSerializer;
import org.xmlpull.v1.XmlPullParser;


/**
 * @author David Svensson, david@cs.lth.se
 */
public class InfoEvent extends Event implements XMLRepresentable {

    private PRDData info;

    private boolean keepInfo;

    private boolean request;

    private XMLDataProvider xmlDataProvider;

    public InfoEvent() {
        this(null, false);
    }

    public InfoEvent(PRDData info, boolean keepInfo) {
        this(info, keepInfo, false);
    }

    public InfoEvent(PRDData info, boolean keepInfo, boolean isRequest) {
        this.info = info;
        this.keepInfo = keepInfo;
        this.request = isRequest;
        xmlDataProvider = new XMLDataProvider();
    }

    public void initializeFromElement(XmlPullParser parser) {
        keepInfo = "true".equals(parser.getAttributeValue(null, "keepInfo"));
        request = "true".equals(parser.getAttributeValue(null, "isRequest"));
    }

    public PRDData getInfo() {
        return info;
    }

    public boolean infoShouldBeKept() {
        return keepInfo;
    }

    public boolean isRequest() {
        return request;
    }

    public byte[] getXMLData() {
        return xmlDataProvider.getXMLData();
    }

    public String toString() {
        return new String(getXMLData());
    }

    public void addChildFromXMLElement(XMLRepresentable child) {
        info = (PRDData) child;
    }

    public String getTagName() {
        return "InfoEvent";
    }

    public void writeXMLElement(KXmlSerializer serializer) throws IOException {
        serializer.startTag(null, getTagName());
        serializer.attribute(null, "keepInfo", new Boolean(keepInfo).toString());
        if (isRequest()) {
            serializer.attribute(null, "isRequest", new Boolean(true).toString());
        }
        info.writeXMLElement(serializer);
        serializer.endTag(null, getTagName());
    }

    /**
     * Inner class used for getting the implementation of getXMLData in
     * AbstractXMLRepresentable.
     */
    private class XMLDataProvider extends AbstractXMLRepresentable {
        public String getTagName() {
            return InfoEvent.this.getTagName();
        }

        public void writeXMLElement(KXmlSerializer serializer)
                throws IOException {
            InfoEvent.this.writeXMLElement(serializer);
        }

        public void initializeFromElement(XmlPullParser parser) {
            InfoEvent.this.initializeFromElement(parser);
        }

        public void addChildFromXMLElement(XMLRepresentable child) {
            InfoEvent.this.addChildFromXMLElement(child);
        }
    }
}