/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.hgraph;

import ist.palcom.base.StringBuilder;

/**
 * ChildSiblingReferences act for a parent node as a mapping between refering and refered child nodes.
 * This is necessary so as to maintain the sibling references, which is partly done by the parent 
 */
public class ChildSiblingReference {

	private String referer;
	private String referee;
	
	/**
	 * Creates a Child SiblingReference with the global name og the referer and the referee
	 * @param referer the global name of the referrer
	 * @param referee the global name of the referee
	 */
	public ChildSiblingReference(String referer, String referee){
		this.referer = referer;
		this.referee = referee;
	}
	
	/**
	 * @return The global name of the referer
	 */
	public String getReferer(){
		return referer;
	}
	
	/**
	 * @return The global name of the referee
	 */
	public String getReferee(){
		return referee;
	}
	
	/**
	 * Sets the global name of the referer
	 * @param referer
	 */
	public void setReferer(String referer){
		this.referer = referer;
	}
	
	/**
	 * Sets the global name of the referee
	 * @param referee
	 */
	public void setReferee(String referee){
		this.referee = referee;
	}
	
	/**
	 * @return an hgraphXML version of the ChildSiblingReference
	 */
	public String toXML(){
		StringBuilder build = StringBuilder.instantiate();
		build.append("<");
		build.append(getClass().toString());
		build.append(" referer=\"");
		build.append(referer);
		build.append("\" referee=\"");
		build.append(referee);
		build.append("\"/>\n");
		return build.toString();
	}
}
