/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.hgraph;


/**
 * ValueHandler handles commands related to the manipulation of node values
 */
public class ValueHandler extends AbstractHandler{
	
	/**
	 * Adds a new value to the node
	 */
	public static final String CMD_ADDVALUE = "cmd_addvalue";
	
	/**
	 * Removes an existing value from the node
	 */
	public static final String CMD_REMOVEVALUE = "cmd_removevalue";
	
	/**
	 * Sets an existing value of the node
	 */
	public static final String CMD_SETVALUE = "cmd_setvalue";
	
	/**
	 * Gets an existing value from the node
	 */
	public static final String CMD_GETVALUE = "cmd_getvalue";

	private static final String name = "Value Handler";
	private static final String description = "Handles events related to the values of the related node";
	private Signature[] handlerSignatures;
	
	private HgraphNode node;
	
	public ValueHandler(HgraphNode node){
		super(name, description);
		this.node = node;
		handlerSignatures = new Signature[4];
		handlerSignatures[0] = new Signature(new String[]{"Value"}, CMD_ADDVALUE, "int");
		handlerSignatures[1] = new Signature(new String[]{"String"}, CMD_GETVALUE, "Value");
		handlerSignatures[2] = new Signature(new String[]{"String"}, CMD_REMOVEVALUE, "int");
		handlerSignatures[3] = new Signature(new String[]{"String", "Value"}, CMD_SETVALUE, "int");
	}
	
	public Signature[] getSignatures(){
		return handlerSignatures;
	}

	public void handle(HgraphEvent event) {
		if(event.getCommand().equals(CMD_ADDVALUE) && event.getValue() != null){
			int tmp = node.getValueFunc().addValue((Value)event.getValue(), event.getFromPath());
			if(event.getReturnCommand() != null){
				node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_ADDVALUE, new Integer(tmp)));
			}
		}else if(event.getCommand().equals(CMD_REMOVEVALUE) && event.getValue() != null){
			int tmp = node.getValueFunc().removeValue((String)event.getValue().getValue(), event.getFromPath());
			if(event.getReturnCommand() != null){
				node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value((String)event.getValue().getValue(), new Integer(tmp)));				
			}			
		}else if(event.getCommand().equals(CMD_SETVALUE) && event.getValue() != null){
			int tmp = node.getValueFunc().setValue(((Value)event.getValue()).getName(), ((Value)event.getValue()).getValue(), event.getFromPath());
			if(event.getReturnCommand() != null){
				node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_SETVALUE, new Integer(tmp)));
			}
		}else if(event.getCommand().equals(CMD_GETVALUE) && event.getReturnCommand() != null && event.getValue() != null){
			Object tmp = node.getValueFunc().getValue((String)event.getValue().getValue());
			node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value((String)event.getValue().getValue(), tmp));
		}
	}
}
