/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.hgraph.manager;

import ist.palcom.hgraph.AbstractHandler;
import ist.palcom.hgraph.Signature;
import ist.palcom.hgraph.HgraphEvent;

/**
 * Handler for the Hgraph Manager handling manage requests to the rest of the H-Graph.
 * This is in principle only really important when adapting a multi process model with process separation 
 *
 */
public class HgraphManagerQueryHandler extends AbstractHandler{

	public static final String CMD_MG_CD = "cmd_manage_cd";
	public static final String CMD_MG_DIR = "cmd_manage_dir";
	public static final String CMD_MG_DIRFULL = "cmd_manage_dirfull";
	public static final String CMD_MG_DIRRECURSIVE = "cmd_manage_dirrec";
	public static final String CMD_MG_DIRFULLRECURSIVE = "cmd_manage_dirfullrec";
	public static final String CMD_MG_NEWNODE = "cmd_manage_newnode";
	public static final String CMD_MG_REMOVENODE = "cmd_manage_rmnode";
	public static final String CMD_MG_SETNAME = "cmd_manage_setname";
	
	private static final String name = "HgraphManager Query Handler";
	private static final String description = "Handles query events for browsing the hgraph";

	private Signature[] handlerSignatures;
	private HgraphManager manager;
	
	public HgraphManagerQueryHandler(HgraphManager manager){
		super(name, description);
		this.manager = manager;
		
		handlerSignatures = new Signature[8];
		handlerSignatures[0] = new Signature(new String[]{"string"}, CMD_MG_CD, "boolean");
		handlerSignatures[1] = new Signature(new String[]{"string"}, CMD_MG_DIR, "string");
		handlerSignatures[2] = new Signature(new String[]{"string"}, CMD_MG_DIRFULL, "string");
		handlerSignatures[3] = new Signature(new String[]{"string"}, CMD_MG_DIRRECURSIVE, "string");
		handlerSignatures[4] = new Signature(new String[]{"string"}, CMD_MG_DIRFULLRECURSIVE, "string");
		handlerSignatures[5] = new Signature(new String[]{"string"}, CMD_MG_NEWNODE, "int");
		handlerSignatures[6] = new Signature(new String[]{"string"}, CMD_MG_REMOVENODE, "int");
		handlerSignatures[7] = new Signature(new String[]{"string", "string"}, CMD_MG_SETNAME, "int");
	}
	
	public Signature[] getSignatures(){
		return handlerSignatures;
	}
	
	public void handle(HgraphEvent event) {
		manager.queryEvent(event);
	}
}
