/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.hmap.test;

import ist.palcom.hmap.AbstractHandler;
import ist.palcom.hmap.HmapDir;
import ist.palcom.hmap.HmapEvent;
import ist.palcom.hmap.HmapLeaf;

public class Test {

	public static void main(String[] args) {
                //ist.palcom.base.System.instance().enableConstantGC();

		System.out.println("<h-map test>");
		new Test().run();
	}

	void assertEquals(Object o1, Object o2) {

		if (o1 == null) {
			if (o2 != null)
				throw new AssertFailure("assertEquals");
		} else if (!o1.equals(o2))
			throw new AssertFailure("assertEquals");
	}

	void assertTrue(boolean condition) {
		if (!condition)
			throw new AssertFailure("assertTrue");
	}

	public void run() {
		try {
			testDir();
			testLeaf();

			System.out.println("<hmap test finished successfully>");
		} catch (Throwable t) {
			System.out.println(t.getMessage());
		}
	}

	public void testDir() {
		System.out.println("<testing Dir>");
		HmapDir root = new HmapDir("root");
		HmapDir dir1 = new HmapDir("dir1");
		HmapDir dir2 = new HmapDir("dir2");
		HmapDir dir21 = new HmapDir("dir2.1");
		HmapDir dir22 = new HmapDir("dir2.2");
		HmapDir dir221 = new HmapDir("dir2.2.1");
		HmapDir dir222 = new HmapDir("dir2.2.2");

		assertEquals(root.getName(), "root");

		dir22.addNode(dir221);
		dir22.addNode(dir222);
		dir2.addNode(dir21);
		dir2.addNode(dir22);
		root.addNode(dir1);
		root.addNode(dir2);
		
		assertTrue(root.isDir());
		assertTrue(dir1.isDir());
		assertTrue(dir2.isDir());

		assertEquals(root.resolve("#root#dir1"), dir1);
		assertEquals(root.resolve("#root#dir2"), dir2);

		HmapDir x0 = new HmapDir("xx");
		HmapLeaf l0 = new HmapLeaf("0", "pony");
		x0.addNode(l0);
		HmapDir x1 = new HmapDir("xx");
		x1.addNode(new HmapLeaf("1", ""));
		HmapDir x2 = new HmapDir("xx");
		x2.addNode(new HmapLeaf("2", ""));
		HmapDir x3 = new HmapDir("xx");
		x3.addNode(new HmapLeaf("3", ""));

		dir2.addNode(x0);
		dir2.addNode(x1);
		dir2.addNode(x2);
		dir2.addNode(x3);
	
		//dir22.addHandler(new printHandler());
		
		//dir22.nameInvoator.getName(new HmapPath("/root/dir1"), "");
		
		//System.out.println(root.toStringAll(0));
		
//		System.out.println(HmapPath.deSerialize(tmp));
		
//		System.out.println(root);
	}

	public void testLeaf() {
		System.out.println("<testing Leaf>");
		HmapDir root = new HmapDir("root");
		HmapLeaf leaf1 = new HmapLeaf("leaf1", "value1");
		HmapLeaf leaf2 = new HmapLeaf("leaf2", new Integer(42));

		assertEquals(root.getName(), "root");
		
		root.addNode(leaf1);
		root.addNode(leaf2);

		assertTrue(root.isDir());
		assertTrue(!leaf1.isDir());
		assertTrue(!leaf2.isDir());
		
		assertEquals(root.resolve("#root#leaf1"), leaf1);
		assertEquals(root.resolve("#root#leaf2"), leaf2);

		assertEquals(((HmapLeaf) root.resolve("#root#leaf1")).getValue(), "value1");
		assertEquals(((HmapLeaf) root.resolve("#root#leaf2")).getValue(), new Integer(42));
		
		assertEquals(root.resolve("no_such_entry"), null);
		
	}

	class AssertFailure extends RuntimeException {

		AssertFailure(String message) {
			super(message);
		}
	}
	
	public class printHandler extends AbstractHandler{

		public void handle(HmapEvent event) {
			System.out.println(event.toString());			
		}

		public String getDescription() {
			return "Prints out any event sent to the handler";
		}

		public String getName() {
			return "Print Handler";
		}
	}

}
