/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.jtst;

import java.util.ArrayList;
import java.util.AbstractList;
import java.util.List;

public class ArrayTest extends Test {
  public String name() { return "Array"; }
  public void run() {
    run1();
    run2();
  }

  private void run1() {
    //ist.palcom.base.System.instance().enableConstantGC();

    // Remove these when static initializers are working!
    //Throwable aioob = new java.lang.ArrayIndexOutOfBoundsException();
    //aioob.nullStackTrace();
    //ist.palcom.base.System.instance().addExceptionMapping("ArrayIndexOutOfBoundsException", aioob);
    //ist.palcom.base.System.instance().addExceptionMapping("ArrayStoreException", new java.lang.ArrayStoreException());

    startLine();
    print(A);

    Object [] oarray = new Object[1];
    print(B);
    if (oarray[0] == null) print(C);
    oarray[0] = new Object();
    if (oarray[0] != null) print(D);
    oarray[0] = oarray;
    if (oarray[0] == oarray) print(E);

    if (new ArrayList() instanceof List) { print(F); }

    try { print(oarray[1]); } catch(Throwable e) { print(G); }
    try { print(oarray[-1]); } catch(Throwable e) { print(H); }
    try { oarray[1] = null; } catch(Throwable e) { print(I); }
    try { oarray[-1] = null; } catch(Throwable e) { print(J); }

    // Put superclass and subclass in array
    AbstractList [] alarray = new AbstractList[1];
    print(K);
    oarray = alarray;
    print(L);
    if (alarray[0] == null) print(M);
    alarray[0] = new ArrayList();
    print(N);
    if (alarray[0] != null) print(O);
    oarray[0] = new ArrayList();
    print(P);
    try { oarray[0] = new Object(); } catch (Throwable t) { print(Q); }


    // Put superinterface and subinterface in array
    List [] larray = new List[1];
    print(R);
    oarray = larray;
    print(S);
    if (larray[0] == null) print(T);

    // Test store in interface-typed array TODO
    larray[0] = new ArrayList();
    print(U);
    if (larray[0] != null) print(V);
    larray[0].add(W);
    print(larray[0].get(0));

    try { oarray[0] = oarray; } catch (Throwable e) { print(X); }
    try { print(larray[1]); } catch(Throwable e) { print(Y); }
    try { print(larray[-1]); } catch(Throwable e) { print(Z); }
    try { larray[1] = null; } catch(Throwable e) { print(0); }
    try { larray[-1] = null; } catch(Throwable e) { print(1); }

    // 2d array
    AbstractList [][] al2array = new AbstractList[1][1];
    print(2);
    al2array[0] = alarray;
    print(3);
    al2array[0] = new AbstractList[1]; //
    al2array[0] = new ArrayList[1];   // ArrayList is a subclass of AbstractList
    print(4);
    Object [][] o2array = al2array;
    try { o2array[0] = new Object[1]; } catch (Throwable t) {
      print(5);
    }
    al2array[0][0] = new ArrayList();
    try {o2array[0][0] = new Object(); } catch (Throwable t) {
      print(6);
    }

    println("789.");
  }

  private void run2() {
    startLine();
    print(A);

    Object[][] o1 = new Object[1][1];
    print(B);
    Object[] o2 = (Object[])o1; // Ok to cast to a lower or equal dimension
    try {
      print(C);
      o2 = (Object[][][])o1; // Not ok to cast to a larger dimension
    } catch (ClassCastException e) {
      print(D);
    }
    
    if (o2 != null)
        print(E);
    
    List[][] l1 = new List[1][1];
    print(F);
    Object[] l2 = (Object[])l1; // Ok to cast to a lower or equal dimension
    //l2 = (Object[][][])l1; // Type error - won't compile with javac or PalJ

    if (l2 == l1)
      print(G);

    byte[][] b1 = new byte[2][];
    b1[0] = new byte[1]; 
    b1[1] = new byte[2]; // Ok to assign arrays with different dimensions
    print(H);

    
    println("ijklmnopqrstuvwxyz0123456789.");

  }
}

