/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.jtst;

import java.util.HashSet;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.util.Map;
//import java.util.MapEntry;

//import java.util.HashCodeNotOverriddenException;

public class HashTest extends Test {
  public String name() { return "Hash"; }
  public void run() {
    run1();
    run2();
  }

  private void run1() {
    //ist.palcom.base.System.instance().enableConstantGC();

    startLine();
    print(A);

    HashSet h = new HashSet();

    print(B);

    h.add("foo");

    print(C);
    if (h.size() == 1) print(D);

    h.add("bar");

    if (h.size() == 2) print(E);

    if (h.contains("foo")) print(F);
    if (h.contains("f" + "oo")) print(G);

    try {
      h.add(new NoHashCode());
      print(H); // On JVM
    } catch(RuntimeException e) {  // actually HashCodeNotOverriddenException
      print(H); // On pal-vm
    }

    h.add(new Equals());
    
    if (h.contains(new Equals())) print(I);

    if (!h.isEmpty()) print(J);
    h.clear();
    if (h.isEmpty()) print(K);
    if (!h.contains("foo")) print(L);

    HashSet h2 = new HashSet();
    h2.add("Erik");
    h2.add("Peter");
    h.add("Michael");
    
    if(h.size() == 1) print(M);
    if(h.contains("Michael")) print(N);

    h.addAll(h2);

    if(h.size() == 3) print(O);
    if(h.contains("Michael")) print(P);

    if (h.containsAll(h2)) print(Q);
    h2.add("David");

    if (!h.containsAll(h2)) print(R);

    boolean me = false;
    int c = 0;
    for (Iterator i = h.iterator(); i.hasNext(); ) {
      String s = (String)(i.next());
      if (s.equals("Michael")) print(S);
      if (s.equals("Erik")) me = true;
      c++;
    }
    if (me) print(T);
    if (c == 3) print(U);

    h2.remove("David");
    if (h.containsAll(h2)) print(V);

    if (!h.remove("David")) print(W);

    HashSet primes = multiples(1, 100);

    print(X);

    if (primes.size() == 99) print(Y);

    for (int i = 2; i <= 10; i++) {
      primes.removeAll(multiples(i, 100));
    }

    if (primes.size() == 25) print(Z);

    if (!primes.contains("ninetyone")) print(0);

    if (primes.contains("thirtyone")) print(1);

    String parray[] = new String[25];

    primes.toArray(parray);

    Object oarray[] = primes.toArray();

    sort(parray);
    sort(oarray);

    if (parray[0].equals("eightynine")) print(2);

    for (int i = 3; i <= 9; i++) {
      if (parray[i].equals(oarray[i])) print(i);
    }

    println(".");

  }

  private void sort(Object array[]) {
    boolean done = false;
    while (!done) {
      done = true;
      for (int i = 1; i < array.length; i++) {
        String prev = (String)(array[i-1]);
        String curr = (String)(array[i]);
        if (curr.compareTo(prev) < 0) {
          array[i-1] = curr;
          array[i] = prev;
          done = false;
        }
      }
    }
  }

  public HashSet multiples(int of, int max) {
    HashSet h = new HashSet();
    for (int i = of * 2; i <= max; i += of) {
      h.add(inWords(i));
    }
    return h;
  }

  public String inWords(int i) {
    if (i == 0) return "zero";
    String a= "";
    switch(i % 10) {
      case 0: a = ""; break;
      case 1: a = "one"; break;
      case 2: a = "two"; break;
      case 3: a = "three"; break;
      case 4: a = "four"; break;
      case 5: a = "five"; break;
      case 6: a = "six"; break;
      case 7: a = "seven"; break;
      case 8: a = "eight"; break;
      case 9: a = "nine"; break;
    }
    switch(i / 10) {
      case 0: return a;
      case 2: return "twenty" + a;
      case 3: return "thirty" + a;
      case 4: return "fourty" + a;
      case 5: return "fifty" + a;
      case 6: return "sixty" + a;
      case 7: return "seventy" + a;
      case 8: return "eighty" + a;
      case 9: return "ninety" + a;
    }
    switch(i) {
      case 11: return "eleven";
      case 12: return "twelve";
      case 13: return "thirteen";
      case 14: return "fourteen";
      case 15: return "fifteen";
      case 16: return "sixteen";
      case 17: return "seventeen";
      case 18: return "eighteen";
      case 19: return "nineteen";
      case 100: return "onehundred";
    }
    return "big number";
  }

  private class NoHashCode {
  }

  private class Equals {
    public boolean equals(Object other) {
      if (other != null)
        return true;
      return false;
    }
    public int hashCode() {
      return 1;
    }
  }

  private static int counter = 0;

  private class Ordinary {
    public boolean equals(Object other) {
      return this == other;
    }
    public int hashCode() {
      return h;
    }
    private int h;
    public Ordinary() {
      h = counter++;
    }
  }

  private void run2() {
    startLine();
    print(A);

    HashMap m = new HashMap();

    if (m.size() == 0) print(B);

    m.put("j", "foo");

    print(C);

    if (m.size() == 1) print(D);

    if (m.get("j").equals("foo")) print(E);

    for (Iterator it = m.entrySet().iterator(); it.hasNext(); ) {
      print(F);
      Object o = it.next();
      //if (o instanceof Map.Entry)   -- this doesn't work with pal-j :-(
        print(G);
    }

    for (Iterator it = m.keySet().iterator(); it.hasNext(); ) {
      print(H);
      Object key = it.next();
      if (key instanceof java.lang.String)
        print(I);
      print(key);
      if (m.get(key).equals("foo"))
        print(K);
    }

    m.put("m", "bar");
    m.put("lala", "baz");
    m.put(null, "fizz");
    m.put(new Ordinary(), "toot");
    m.put(new Ordinary(), "hoot");
    
    if (m.get(null).equals("fizz"))
      print(L);

    if (m.size() == 6) print(M);

    Object ne = new Ordinary();
    m.put(ne, "woot");

    if (m.size() != 7) print("!");

    if (m.containsKey(null)) print(N);
    if (m.containsValue("woot")) print(O);
    if (m.containsValue("fizz")) print(P);

    m.remove(null);
    if (m.containsKey(null)) print("*");
    if (m.containsValue("fizz")) print("-");

    if (m.containsKey(ne)) print(Q);

    m.clear();

    Ordinary key1 = new Ordinary();
    m.put(key1, key1);
    Ordinary key2 = new Ordinary();
    m.put(key2, key2);
    for (int i = 0; i < 1000; i++) {
      m.put(new Ordinary(), new java.lang.Integer(i));
    }
    Ordinary key3 = new Ordinary();
    m.put(key3, key3);
    Ordinary key4 = new Ordinary();
    m.put(key4, key4);
    if (m.size() == 1004) print(R);

    if (m.get(key1).equals(key1)) print(S);
    if (m.get(key2).equals(key2)) print(T);
    if (m.get(key3).equals(key3)) print(U);
    if (m.get(key4).equals(key4)) print(V);
    if (m.get(key1).equals(key2)) print("%");
    if (m.get(key2).equals(key1)) print("%");
    if (m.get(key3).equals(key1)) print("%");
    if (m.get(key4).equals(key1)) print("%");

    m.remove(key1);
    m.remove(key2);
    m.remove(key3);
    if (m.get(key4).equals(key4)) print(W);

    if (m.size() == 1001) print(X);

    Object keys[] = m.keySet().toArray();
    Object values[] = m.values().toArray();

    print(Y);

    if (keys.length == 1001) print(Z);
    if (values.length == 1001) print(0);

    m.keySet().remove(key4);

    if (m.size() == 1000)
      print(1);

    m.keySet().clear();

    if (m.size() == 0)
      print(2);

    m.put("foo", "bar");
    m.put("bar", "baz");
    m.put("baz", "fizz");

    HashSet s = new HashSet();
    s.add("foo");
    if (m.keySet().containsAll(s)) print(3);
    s.add("baz");
    if (m.keySet().containsAll(s)) print(4);
    s.add("fizz");
    if (!m.keySet().containsAll(s)) print(5);

    m.put("buzz", null);

    s.addAll(m.keySet());

    if (s.size() == 5) print(6);

    if (m.get("baa") == null) print(7);
    if (m.get("buzz") == null) print(8);

    if (!m.containsKey("baa") && m.containsKey("buzz")) print(9);

    m.put("buzz", "exigy");

    s.addAll(m.values());

    if(s.contains("exigy")) print(".");

    println("");
  }
}
