/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal;

import ist.palcom.resource.descriptor.URL;
import ist.palcom.palcomthreads.PalcomPriorityThread;

/**
 * A CommunicationThread is responsible for sending and receiving messages over
 * one medium or protocol (such as UDP or IR). Subclasses implement
 * protocol/medium specifics. In the case of UDP, one CommunicationThread sends
 * and receives on one socket. In general, one CommunicationThread is needed for
 * each independent sequence of incoming messages.
 * <p>
 * With the exception of the getURL method, the interface of this class is given
 * by the set of events handled. A CommunicationThread supports the following
 * incoming events:
 * 
 * <ul>
 * <li> MessageWithURLEvent. The message in the event is sent to the URL in the
 * event.
 * <li> StopEvent. The thread terminates itself.
 * </ul>
 * 
 * The only outgoing event is a MessageWithURLEvent, which is sent to the
 * DistributorThread of a MediaManager when a message has been received. The URL
 * in the event is the URL <em>on which the message has been received</em>.
 * 
 * @see ist.palcom.mal.MediaManager MediaManager
 * 
 * @author David Svensson, david@cs.lth.se
 */
public abstract class CommunicationThread extends PalcomPriorityThread {
    /**
     * @return The URL for sending messages to this thread, a URL this thread
     *         listens at (protocol-specific URL).
     */
    public abstract URL getURL();
}
