/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal;

import ist.palcom.resource.descriptor.Selector;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.resource.descriptor.URN;

public class DeviceCacheObject {

	private URL deviceURL;
	private String deviceID;
	private String wireProtocolVersion;
	private String discoveryProtocolVersion;
	private Selector discoverySelector;
	private String shortID;
	private String deviceCache;
	private String deviceStatus;

	public DeviceCacheObject(URL deviceURL, String deviceID, 
			String wireProtocolVersion, String discoveryProtocolVersion, 
			Selector discoverySelector) {
		this.deviceURL = deviceURL;
		this.deviceID = deviceID;
		this.wireProtocolVersion = wireProtocolVersion;
		this.discoveryProtocolVersion = discoveryProtocolVersion;
		this.discoverySelector = discoverySelector;
		shortID = null;
		deviceCache = null;
		deviceStatus = null;
	}

	/**
	 * Used for creating an object to compare with the list of registered devices
	 * to find a certain device with deviceURL.
	 * @param deviceURL the URL of the device which is sought.
	 */
	public DeviceCacheObject(URL deviceURL, String shortID) {
		this.deviceURL = deviceURL;
		deviceID = null;
		wireProtocolVersion = null;
		discoveryProtocolVersion = null;
		discoverySelector = null;
		this.shortID = shortID;
		deviceCache = null;
		deviceStatus = null;
	}

	public String getWireVersion() {
		return wireProtocolVersion;
	}

	public void setWireVersion(String wireProtocolVersion) {
		this.wireProtocolVersion = wireProtocolVersion;
	}

	public URL getDeviceURL() {
		return deviceURL;
	}

	public Selector getDiscoverySelector() {
		return discoverySelector;
	}

	public boolean equals(Object o) {
		if (o instanceof DeviceCacheObject) {
			if((((DeviceCacheObject) o).getDeviceURL()).equals(getDeviceURL())) {
				if(((DeviceCacheObject) o).getShortID() == null && getShortID() == null) {
					return true;
				} else if(((DeviceCacheObject) o).getShortID() != null && getShortID() != null) {
					return ((DeviceCacheObject) o).getShortID().equals(getShortID());
				}
			}
		} 
		return false;
	}

	public String getDeviceID() {
		return deviceID;
	}

	public String getDiscoveryVersion() {
		return discoveryProtocolVersion;
	}

	public String getShortID() {
		return shortID;
	}

	public void setShortID(String shortID) {
		this.shortID = shortID;
	}
	
	public String toString() {
		return deviceURL.toString() + " " + deviceID + " " + shortID;
	}
	
	public String getDeviceCache() {
		return deviceCache;
	}
	
	public void updateDeviceInfo(String deviceCache, String deviceStatus) {
		this.deviceCache = deviceCache;
		this.deviceStatus = deviceStatus;
	}
	
	public String getDeviceStatus() {
		return deviceStatus;
	}
}
