/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal.bt;

import ist.palcom.base.networking.BluetoothPacket;
import ist.palcom.base.networking.BluetoothSocket;
import ist.palcom.mal.CommunicationThread;
import ist.palcom.mal.MALErrorEvent;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageArrayWithURLEvent;
import ist.palcom.mal.MessageWithURLEvent;
import ist.palcom.mal.StopEvent;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.IOAvailableEvent;
import ist.palcom.palcomthreads.IOTimeoutEvent;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.URL;

import java.io.IOException;

/**
 * This class implements ist.palcom.mal.BluetoothCommunicationThread
 * 
 * BluetoothCommunicationThread is a CommunicationThread for Bluetooth communication. It
 * sends an receives on one Bluetooth socket and is thus the rfcomm connection to a specific
 * host
 * 
 * @see ist.palcom.mal.CommunicationThread CommunicationThread
 * 
 * @author Jacob Frolund Pedersen, frolund@daimi.au.dk
 */


//TODO: This is not tested and probably have to be reimplemented as it is just a copy of the UDPCommunicationThread
public class BluetoothCommunicationThread extends CommunicationThread {

    protected static final String URL_PREFIX = "bluetooth://";

    private PalcomScheduler scheduler;

    protected BluetoothSocket socket;

    private PalcomThread distributor;

    protected BluetoothCommunicationThread(PalcomScheduler scheduler, PalcomThread distributor,
    		String URL) throws IOException {
    	this.socket = new BluetoothSocket(URL);
    	this.distributor = distributor;
        this.scheduler = scheduler;
    }

    protected BluetoothCommunicationThread(PalcomScheduler scheduler, PalcomThread distributor,
    		BluetoothSocket socket){
    	this.distributor = distributor;
    	this.scheduler = scheduler;
    	this.socket = socket;
    }
    
    /** Perform socket set-up if needed. */
    protected void setUp() {
    	socket.startReceiving();
    }

    /** Close socket if needed. */
    protected void tearDown() {
    	socket.close();
    }

    public void run() {
        setUp();
        final int timeout = 1000;
        enableIOEvent(socket, timeout);
        while (true) {
            Event event = waitEvent();
            // System.out.println(hashCode() + " " + event);
            if (event instanceof IOAvailableEvent) {		// Either data or IOException is ready to be handled
                BluetoothPacket receivedPacket = new BluetoothPacket(
                        new byte[81920], BluetoothSocket.getLocalAddress());
                try {
                	socket.receive(receivedPacket);
                } catch (IOException e) {
                	System.out.println("BTCommThread: conn closed");
                	// Connection was closed
                	break;
                }
                	
                byte[] data = receivedPacket.getData();
//                Message message = Message.parse(data, 0, receivedPacket
//                        .getLength());
//                message.setSenderURL(new URL(URL_PREFIX+socket.getDeviceAddress()));
//                message.setReceiverURL(new URL(receivedPacket.getReceiverHost()));
                if (distributor != null) {
                    distributor.putEvent(new MessageArrayWithURLEvent(data,
                            new URL(URL_PREFIX+socket.getDeviceAddress())));
                }
                enableIOEvent(socket, timeout);
            } else if (event instanceof IOTimeoutEvent) {
                // Try again
                enableIOEvent(socket, timeout);
            } else if (event instanceof MessageArrayWithURLEvent) {
            	try {
//	            	Message message = ((MessageWithURLEvent)event).getMessage();
//	                byte[] messageData = new byte[message.getTotalLength()];
//	                message.write(messageData, 0);
            		byte[] messageData = ((MessageArrayWithURLEvent)event).getMessageArray();
	            	socket.send(new BluetoothPacket(messageData,
	                                ((MessageArrayWithURLEvent)event).getDeviceURL().toString()));
	            	enableIOEvent(socket,timeout);
            	
            	} catch (IOException e) {
            		// Terminate the connection, if an error occurs, to prevent dead connections
            		distributor.putEvent(new MALErrorEvent(e));
            		break;
            	}
            } else if (event instanceof StopEvent) {
                break;
            }
        }
        tearDown();
        terminate();
    }

	public URL getURL() {
		return new URL(URL_PREFIX + socket.getDeviceAddress());
	}
}