/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.messaging.test;

import ist.palcom.communication.CommunicationManager;
import ist.palcom.mal.AbstractMediaManager;
import ist.palcom.mal.DeviceCacheObject;
import ist.palcom.mal.Message;
import ist.palcom.mal.udp.UDPMediaManager;
import ist.palcom.mal.udp.test.UDPCommunicationTest2;
import ist.palcom.messaging.PointToPointManager;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.Selector;
import ist.palcom.routing.RoutingManager;
import ist.palcom.routing.util.SimpleRoutingManager;

import java.util.ArrayList;

/**
 * @author David Svensson, david@cs.lth.se
 */
public class MessagingTest {

    public static void main(String[] args) {
        // ist.palcom.base.System.instance().enableConstantGC();
        MessagingTest test = new MessagingTest();

        test.testSendAndReceiveMessageWithURNs();
        System.out.println();
    }

    private void testSendAndReceiveMessageWithURNs() {
        System.out.println("--- testSendAndReceiveMessage ---");

        final PalcomScheduler scheduler = new PalcomScheduler();
        PalcomThread t = new PalcomPriorityThread() {
            public void run() {
                AbstractMediaManager mediaManager = new UDPMediaManager(scheduler, null);
                ArrayList mediaManagers = new ArrayList();
                mediaManagers.add(mediaManager);

                              
//              Generate hopefully unique mark ID for router.
        		String mark = "abc";
                
        		RoutingManager routingManager = new SimpleRoutingManager(mediaManagers, scheduler, mark);
                CommunicationManager comManager = new CommunicationManager(scheduler, 
                		routingManager);
                PointToPointManager pointToPointManager = new PointToPointManager(
                        comManager);

                Selector receiverSelector = pointToPointManager
                        .startReceiving2(this);
                Selector senderSelector = pointToPointManager
                        .startReceiving2(this);
                // Sender also starts to listen, so a selector is allocated

                byte[] sentData = "Some data".getBytes();
                Message message = new Message(sentData, 0,
                        sentData.length);

                System.out.println("Sending message of total length "
                        + message.getTotalLength()
                        + " (selectors not set)");

                final DeviceID receiverDeviceID = new DeviceID("X:receiver");

                // Manually register device URL, so the manager knows about the
                // receiver. In real use, this information is spread via
                // broadcast messages
                mediaManager.registerDevice(receiverDeviceID.toString(), new DeviceCacheObject(mediaManager
                        .getDefaultUnicastURL(), receiverDeviceID.toString(), "W2", "D2", new Selector("1")));

                boolean sent = pointToPointManager.send(message, senderSelector,
                        receiverDeviceID, receiverSelector, false);

                UDPCommunicationTest2.waitForMessageAndPrintIt(this);

                comManager.stop();
                routingManager.stop();
            }
        };
        scheduler.scheduleThread(t);
        scheduler.run();
    }
}
