/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.palcomthreads;

import ist.palcom.collections.LinkedList;

/**
 * @author David Svensson, david@cs.lth.se
 */
class SortedPalcomThreadQueue {
    private LinkedList notices;

    SortedPalcomThreadQueue() {
        notices = new LinkedList();
    }

    PalcomThreadNotice getFirst() {
        return (PalcomThreadNotice) notices.getFirst();
    }

    PalcomThreadNotice removeFirst() {
        return (PalcomThreadNotice) notices.removeFirst();
    }

    boolean isEmpty() {
        return notices.isEmpty();
    }

    /**
     * Insert n into the queue, in sorted order. n will be placed after other
     * notices which are "equal" to n in the order.
     */
    void sortInto(PalcomThreadNotice n) {
        PalcomThreadNotice noticeInQueue = (PalcomThreadNotice) notices
                .getLast();
        while (noticeInQueue != null && n.isBefore(noticeInQueue)) {
            noticeInQueue = (PalcomThreadNotice) noticeInQueue.getPrev();
        }
        if (noticeInQueue == null) {
            notices.insertFirst(n);
        } else {
            notices.insertAfter(n, noticeInQueue);
        }
    }

    void remove(PalcomThreadNotice n) {
        notices.remove(n);
    }

    /**
     * @return The first PeriodicNotice for t in the list, or null if there is
     *         none.
     */
    PeriodicNotice findPeriodicNotice(PalcomThread t) {
        PalcomThreadNotice n = (PalcomThreadNotice) notices.getFirst();
        while (n != null
                && (!(n instanceof PeriodicNotice) || n.getThread() != t)) {
            n = (PalcomThreadNotice) n.getNext();
        }
        return (PeriodicNotice) n;
    }

    /**
     * @return The first TimeoutNotice for t in the list, or null if there is
     *         none.
     */
    TimeoutNotice findTimeoutNotice(PalcomThread t) {
        PalcomThreadNotice n = (PalcomThreadNotice) notices.getFirst();
        while (n != null
                && (!(n instanceof TimeoutNotice) || n.getThread() != t)) {
            n = (PalcomThreadNotice) n.getNext();
        }
        return (TimeoutNotice) n;
    }
}