/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;


public abstract class ActionWithParams extends Action implements Cloneable {
    public void flushCache() {
        super.flushCache();
        getErrorList_visited = false;
        ci_visited = false;
    }
    public Object clone() throws CloneNotSupportedException {
        ActionWithParams node = (ActionWithParams)super.clone();
        node.getErrorList_visited = false;
        node.ci_visited = false;
    return node;
    }
    // Declared in AssemblyTree.jadd at line 140

	
	public Object[] getChildArray() {
		return (getNumParamValue() > 0 ? getParamValueList().getChildArray() : null);
	}

    // Declared in Assembly.ast at line 3
    // Declared in Assembly.ast line 84

    public ActionWithParams() {
        super();

        setChild(new List(), 0);

    }

    // Declared in Assembly.ast at line 11


    // Declared in Assembly.ast line 84
    public ActionWithParams(String p0, List p1) {
        setCommand(p0);
        setChild(p1, 0);
    }

    // Declared in Assembly.ast at line 16


  protected int numChildren() {
    return 1;
  }

    // Declared in Assembly.ast at line 2
    // Declared in Assembly.ast line 84
    private String tokenString_Command;

    // Declared in Assembly.ast at line 3

    public void setCommand(String value) {
        tokenString_Command = value;
    }

    // Declared in Assembly.ast at line 6

    public String getCommand() {
        return tokenString_Command;
    }

    // Declared in Assembly.ast at line 2
    // Declared in Assembly.ast line 84
    public void setParamValueList(List list) {
        setChild(list, 0);
    }

    // Declared in Assembly.ast at line 6


    public int getNumParamValue() {
        return getParamValueList().getNumChild();
    }

    // Declared in Assembly.ast at line 10


    public Use getParamValue(int i) {
        return (Use)getParamValueList().getChild(i);
    }

    // Declared in Assembly.ast at line 14


    public void addParamValue(Use node) {
        List list = getParamValueList();
        list.setChild(node, list.getNumChild());
    }

    // Declared in Assembly.ast at line 19


    public void setParamValue(Use node, int i) {
        List list = getParamValueList();
        list.setChild(node, i);
    }

    // Declared in Assembly.ast at line 23

    public List getParamValueList() {
        return (List)getChild(0);
    }

    // Declared in Assembly.ast at line 27


    public List getParamValueListNoTransform() {
        return (List)getChildNoTransform(0);
    }

    protected boolean getErrorList_visited = false;
    // Declared in ErrorChecking.jrag at line 157
    public ArrayList getErrorList() {
        if(getErrorList_visited)
            throw new RuntimeException("Circular definition of attr: getErrorList in class: ");
        getErrorList_visited = true;
        ArrayList getErrorList_value = getErrorList_compute();
        getErrorList_visited = false;
        return getErrorList_value;
    }

    private ArrayList getErrorList_compute()  {
		ArrayList l = new ArrayList();
		try {
		for(int i=0; i < getNumParamValue();i++) {
			//check if parameters are declared
			if(!getParamValue(i).isDeclared()) {
				l.add(l.size(),new SemanticError(SemanticError.TYPE_ERROR, getParamValue(i).getUseName() + " undeclared", this));											
			}
			//check parameter types
			if(getParamValue(i).typeError()) {
				l.add(l.size(),new SemanticError(SemanticError.TYPE_WARNING,"Type mismatch for param "+i+":"+ getParamValue(i).getUseName()+"["+getParamValue(i).actualType()+"] != "+getParamValue(i).expectedType(), this));								
			}
		}
		}catch(NullPointerException e) {e.printStackTrace();}
		return l;
	}

    protected boolean ci_visited = false;
    // Declared in ErrorChecking.jrag at line 113
    public CommandInfo ci() {
        if(ci_visited)
            throw new RuntimeException("Circular definition of attr: ci in class: ");
        ci_visited = true;
        CommandInfo ci_value = getParent().Define_CommandInfo_ci(this, null);
        ci_visited = false;
        return ci_value;
    }

    // Declared in AssemblyTypeAnalysis.jrag at line 74
    public String Define_String_expectedType(ASTNode caller, ASTNode child) {
        if(caller == getParamValueListNoTransform()) {
      int i = caller.getIndexOfChild(child);
            return  "unknown";
        }
        return getParent().Define_String_expectedType(this, caller);
    }

}
