/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;


public class Command extends ControlItem implements Cloneable {
    public void flushCache() {
        super.flushCache();
        getBrowserLabel_visited = false;
    }
    public Object clone() throws CloneNotSupportedException {
        Command node = (Command)super.clone();
        node.getBrowserLabel_visited = false;
    return node;
    }
    public ASTNode copy() {
      try {
          Command node = (Command)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        Command res = (Command)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in DeviceTree.jadd at line 248


	public Object[] getChildArray() {
		if(getNumParam() > 0) {
			return getParamList().getChildArray();
		} else {
//			Object tmp[] = new Object[1];
//			tmp[0] = new String("No parameters");
//			return tmp;
			return null;
		}
	}

    // Declared in ServiceDescription.jadd at line 45


    public static final String DIRECTION_IN = "in";

    // Declared in ServiceDescription.jadd at line 47


    public static final String DIRECTION_OUT = "out";

    // Declared in ServiceDescription.jadd at line 87


    public Command(String id, String direction) {
        super();
        setChild(new List(), 1);
        getChildNoTransform(1).setParent(this);
        setInfo(new CommandInfo(id, null, direction, null, new List()));
    }

    // Declared in ServiceDescription.jadd at line 122

    
    public PRDData getInfoAsInfo() {
        return getInfo();
    }

    // Declared in ServiceDescription.jadd at line 158


    public String getDirection() { 
        return getInfo().getDirection();
    }

    // Declared in ServiceDescription.jadd at line 174


    public String getHelp() {
        return getInfo().getHelp();
    }

    // Declared in ServiceDescription.jadd at line 198


    public void setHelp(String help) {
        getInfo().setHelp(help);
    }

    // Declared in ServiceDescription.jadd at line 248


    public ControlItem findItem(String id) {
        if (id.equals(getID())) {
            return this;
        } else {
            return getParam(id);
        }
    }

    // Declared in ServiceDescription.jadd at line 329



    /**
     * Implements this method for Commands (overrides the one in ControlItem).
     * 
     * Returns this Command if it matches command, and null if not.
     */
    public Command findMatchingCommand(Command command) {
        if (this.matches(command)) {
            return this;
        } else {
            return null;
        }
    }

    // Declared in ServiceDescription.jadd at line 365


    public void setDataRefsAndAddDataToList(ArrayList dataList) {
        for (int i = 0; i < getNumParam(); i++) {
            getParam(i).setDataRefsAndAddDataToList(dataList);
        }
    }

    // Declared in ServiceDescription.jadd at line 398


    public void setDataAccordingToDataRefs(byte[][] data) {
        for (int i = 0; i < getNumParam(); i++) {
            getParam(i).setDataAccordingToDataRefs(data);
        }
    }

    // Declared in ServiceDescription.jadd at line 427


    /**
     * Returns a copy of this command, with the same ID and the same parameters
     * and parameter values.
     */
    public Command getCopy() {
        Command copy = new Command(getID(), getDirection());
        for (int i = 0; i < getNumParam(); i++) {
            copy.addParamAndInfo(getParam(i).getCopy());
        }
        return copy;
    }

    // Declared in ServiceDescription.jadd at line 450

    
    /** Returns a param with ID id, or null if none is found. */
    public Param getParam(String id) {
        for (int i = 0; i < getNumParam(); i++) {
            if (id.equals(getParam(i).getID())) {
                return getParam(i);
            }
        }
        return null;
    }

    // Declared in ServiceDescription.jadd at line 467


    /**
     * Invoke this command. This means that it will be sent to registered
     * receivers in the surrounding ServiceProxy, with the current
     * parameter values. Which receivers that are notified depends on the actual
     * type of this object.
     * 
     * If there is no surrounding ServiceProxy, nothing will happen.
     */
    public void invoke() {
        ServiceProxy sp = getServiceProxy();
        if (sp != null) {
            sp.invokeCommand(this);
        }
    }

    // Declared in ServiceDescription.jadd at line 480


    /**
     * Returns true if this Command matches command.
     * 
     * A Command is considered to match if it has the same ID, and if all params
     * have the same ID and the same types, and are in the same order.
     */
    protected boolean matches(Command command) {
        if (getID().equals(command.getID())) {
            if (getNumParam() == command.getNumParam()) {
                for (int i = 0; i < getNumParam(); i++) {
                    if (!getParam(i).getID()
                            .equals(command.getParam(i).getID())
                            || !getParam(i).getType().equals(
                                    command.getParam(i).getType())) {
                        return false;
                    }
                }

                // We have a match!
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    // Declared in ServiceDescription.jadd at line 506


    /**
     * Update this Command with the parameter values in command. If command does
     * not match this Command, nothing happens.
     */
    public void update(Command command) {
        if (this.matches(command)) {
            for (int i = 0; i < getNumParam(); i++) {
                getParam(i).setData(command.getParam(i).getData());
            }
        }
    }

    // Declared in ServiceDescription.jadd at line 615


    public void addParamAndInfo(Param param) {
        addParam(param);
        getInfo().addParamInfo(param.getInfo());
    }

    // Declared in Comparison.jadd at line 239

    public boolean isAboutTheSameEntityAs(PRDData data) {
        return getInfo().isAboutTheSameEntityAs(data);
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 77

    public Command() {
        super();

        setChild(null, 0);
        setChild(new List(), 1);

    }

    // Declared in PalcomResourceDescriptor.ast at line 12


    // Declared in PalcomResourceDescriptor.ast line 77
    public Command(CommandInfo p0, List p1) {
        setChild(p0, 0);
        setChild(p1, 1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 17


  protected int numChildren() {
    return 2;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 77
    public void setInfo(CommandInfo node) {
        setChild(node, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public CommandInfo getInfo() {
        return (CommandInfo)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public CommandInfo getInfoNoTransform() {
        return (CommandInfo)getChildNoTransform(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 77
    public void setParamList(List list) {
        setChild(list, 1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 6


    public int getNumParam() {
        return getParamList().getNumChild();
    }

    // Declared in PalcomResourceDescriptor.ast at line 10


    public Param getParam(int i) {
        return (Param)getParamList().getChild(i);
    }

    // Declared in PalcomResourceDescriptor.ast at line 14


    public void addParam(Param node) {
        List list = getParamList();
        list.setChild(node, list.getNumChild());
    }

    // Declared in PalcomResourceDescriptor.ast at line 19


    public void setParam(Param node, int i) {
        List list = getParamList();
        list.setChild(node, i);
    }

    // Declared in PalcomResourceDescriptor.ast at line 23

    public List getParamList() {
        return (List)getChild(1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 27


    public List getParamListNoTransform() {
        return (List)getChildNoTransform(1);
    }

    protected boolean getBrowserLabel_visited = false;
    // Declared in LabelProviders.jrag at line 109
    public String getBrowserLabel() {
        if(getBrowserLabel_visited)
            throw new RuntimeException("Circular definition of attr: getBrowserLabel in class: ");
        getBrowserLabel_visited = true;
        String getBrowserLabel_value = getBrowserLabel_compute();
        getBrowserLabel_visited = false;
        return getBrowserLabel_value;
    }

    private String getBrowserLabel_compute()  {
		return getID() + " [" + getDirection()+"]";
	}

}
