/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;


public class CommandInfo extends ControlInfo implements Cloneable {
    public void flushCache() {
        super.flushCache();
        getBrowserLabel_visited = false;
        lookupCommand_String_visited = new java.util.HashSet(4);
        lookupParam_String_visited = new java.util.HashSet(4);
        encloseInBraces_visited = false;
        parentService_visited = false;
    }
    public Object clone() throws CloneNotSupportedException {
        CommandInfo node = (CommandInfo)super.clone();
        node.getBrowserLabel_visited = false;
        node.lookupCommand_String_visited = new java.util.HashSet(4);
        node.lookupParam_String_visited = new java.util.HashSet(4);
        node.encloseInBraces_visited = false;
        node.parentService_visited = false;
    return node;
    }
    public ASTNode copy() {
      try {
          CommandInfo node = (CommandInfo)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        CommandInfo res = (CommandInfo)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in DeviceTree.jadd at line 298


	public Object[] getChildArray() {
		if(getNumParamInfo() > 0) {
			return getParamInfoList().getChildArray();
		} else {
			return null;
		}
	}

    // Declared in ServiceDescription.jadd at line 55


    public CommandInfo(String id, String direction) {
        this(id, null, direction, null, new List());
    }

    // Declared in ServiceDescription.jadd at line 648


    public ControlItem createInstance() {
        Command instance = new Command(this, new List());
        for (int i = 0; i < getNumParamInfo(); i++) {
            instance.addParam((Param)getParamInfo(i).createInstance());
        }
        return instance;
    }

    // Declared in Comparison.jadd at line 256


    public boolean isAboutTheSameEntityAs(PRDData data) {
        return equals(data);
    }

    // Declared in Comparison.jadd at line 453


    public boolean equals(Object o) {
        if (!super.equals(o)) {
            return false;
        }

        if (o instanceof CommandInfo) {
            CommandInfo c = (CommandInfo) o;
            if (!PRDData.safeEqual(c.getDirection(), getDirection())) {
                return false;
            }
            for (int i = 0; i < getNumParamInfo(); i++) {
                if (!c.getParamInfo(i).equals(getParamInfo(i))) {
                    return false;
                }
            }
            return true;
        } else {
            return false;
        }
    }

    // Declared in XML.jadd at line 120


    public String getTagName() {
        return "CmdI";
    }

    // Declared in XML.jadd at line 316

    
    public void initializeFromElement(XmlPullParser parser) {
        super.initializeFromElement(parser);
        setDirection(parser.getAttributeValue(null, "direction"));
    }

    // Declared in XML.jadd at line 466


    public void addChildFromXMLElement(XMLRepresentable child) {
        addParamInfo((ParamInfo) child);
    }

    // Declared in XML.jadd at line 643



    public void writeXMLElement(KXmlSerializer serializer) throws IOException {
        writeStartTagAndAttributes(serializer);
        serializer.attribute(null, "direction", getDirection());
        for (int i = 0; i < getNumParamInfo(); i++) {
            getParamInfo(i).writeXMLElement(serializer);
        }
        serializer.endTag(null, getTagName());
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 71

    public CommandInfo() {
        super();

        setChild(new List(), 0);

    }

    // Declared in PalcomResourceDescriptor.ast at line 11


    // Declared in PalcomResourceDescriptor.ast line 71
    public CommandInfo(String p0, String p1, String p2, String p3, List p4) {
        setID(p0);
        setHelp(p1);
        setDirection(p2);
        setName(p3);
        setChild(p4, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 19


  protected int numChildren() {
    return 1;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 69
    private String tokenString_ID;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setID(String value) {
        tokenString_ID = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getID() {
        return tokenString_ID;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 69
    private String tokenString_Help;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setHelp(String value) {
        tokenString_Help = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getHelp() {
        return tokenString_Help;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 71
    private String tokenString_Direction;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setDirection(String value) {
        tokenString_Direction = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getDirection() {
        return tokenString_Direction;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 71
    private String tokenString_Name;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setName(String value) {
        tokenString_Name = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getName() {
        return tokenString_Name;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 71
    public void setParamInfoList(List list) {
        setChild(list, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 6


    public int getNumParamInfo() {
        return getParamInfoList().getNumChild();
    }

    // Declared in PalcomResourceDescriptor.ast at line 10


    public ParamInfo getParamInfo(int i) {
        return (ParamInfo)getParamInfoList().getChild(i);
    }

    // Declared in PalcomResourceDescriptor.ast at line 14


    public void addParamInfo(ParamInfo node) {
        List list = getParamInfoList();
        list.setChild(node, list.getNumChild());
    }

    // Declared in PalcomResourceDescriptor.ast at line 19


    public void setParamInfo(ParamInfo node, int i) {
        List list = getParamInfoList();
        list.setChild(node, i);
    }

    // Declared in PalcomResourceDescriptor.ast at line 23

    public List getParamInfoList() {
        return (List)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 27


    public List getParamInfoListNoTransform() {
        return (List)getChildNoTransform(0);
    }

    // Declared in Hgraph.jadd at line 80

	
	  public void setID(String ID){
		refined_PalcomResourceDescriptor_setID(ID);
		if(ID != null){
			_node.setNodeName(getTagName() + " - " + ID, null);
		}
	}

    // Declared in Hgraph.jadd at line 98

	  public void setHelp(String help){
		refined_PalcomResourceDescriptor_setHelp(help);
		if(help != null){
			_node.setNodeName(getTagName() + " - " + help, null);
		}
	}

    // Declared in Hgraph.jadd at line 116

	  public void setDirection(String direc){
		refined_PalcomResourceDescriptor_setDirection(direc);
		if(direc != null){
			_node.getValueFunc().setValue("direction", direc, null);
		}
	}

    protected boolean getBrowserLabel_visited = false;
    // Declared in LabelProviders.jrag at line 98
    public String getBrowserLabel() {
        if(getBrowserLabel_visited)
            throw new RuntimeException("Circular definition of attr: getBrowserLabel in class: ");
        getBrowserLabel_visited = true;
        String getBrowserLabel_value = getBrowserLabel_compute();
        getBrowserLabel_visited = false;
        return getBrowserLabel_value;
    }

    private String getBrowserLabel_compute() {  return  getID() + " [" + getDirection()+"]";  }

    protected java.util.Set lookupCommand_String_visited = new java.util.HashSet(4);
    // Declared in NameAnalysis.jrag at line 220
    public ControlInfo lookupCommand(String id) {
        Object _parameters = id;
        if(lookupCommand_String_visited.contains(_parameters))
            throw new RuntimeException("Circular definition of attr: lookupCommand in class: ");
        lookupCommand_String_visited.add(_parameters);
        ControlInfo lookupCommand_String_value = lookupCommand_compute(id);
        lookupCommand_String_visited.remove(_parameters);
        return lookupCommand_String_value;
    }

    private ControlInfo lookupCommand_compute(String id)  {
	if(id.equals(getID()))
		return this;
	else
		return null;
}

    protected java.util.Set lookupParam_String_visited = new java.util.HashSet(4);
    // Declared in NameAnalysis.jrag at line 227
    public ParamInfo lookupParam(String id) {
        Object _parameters = id;
        if(lookupParam_String_visited.contains(_parameters))
            throw new RuntimeException("Circular definition of attr: lookupParam in class: ");
        lookupParam_String_visited.add(_parameters);
        ParamInfo lookupParam_String_value = lookupParam_compute(id);
        lookupParam_String_visited.remove(_parameters);
        return lookupParam_String_value;
    }

    private ParamInfo lookupParam_compute(String id)  {
	for(int i=0; i < getNumParamInfo(); i++) {
		ParamInfo pi = getParamInfo(i).lookupParam(id);
		if(pi instanceof ParamInfo)
			return pi;
	}
	return null;
}

    protected boolean encloseInBraces_visited = false;
    // Declared in PrettyPrint.jrag at line 75
    public boolean encloseInBraces() {
        if(encloseInBraces_visited)
            throw new RuntimeException("Circular definition of attr: encloseInBraces in class: ");
        encloseInBraces_visited = true;
        boolean encloseInBraces_value = encloseInBraces_compute();
        encloseInBraces_visited = false;
        return encloseInBraces_value;
    }

    private boolean encloseInBraces_compute() {  return  getNumParamInfo() > 0;  }

    protected boolean parentService_visited = false;
    // Declared in AssemblyTree.jadd at line 180
    public Service parentService() {
        if(parentService_visited)
            throw new RuntimeException("Circular definition of attr: parentService in class: ");
        parentService_visited = true;
        Service parentService_value = getParent().Define_Service_parentService(this, null);
        parentService_visited = false;
        return parentService_value;
    }

}
