/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;



public abstract class ControlItem extends ASTNode implements Cloneable {
    public void flushCache() {
        super.flushCache();
        getBrowserLabel_visited = false;
    }
    public Object clone() throws CloneNotSupportedException {
        ControlItem node = (ControlItem)super.clone();
        node.getBrowserLabel_visited = false;
    return node;
    }
    // Declared in ServiceDescription.jadd at line 120

    
    /** Workaround for static typing. */
    public abstract PRDData getInfoAsInfo();

    // Declared in ServiceDescription.jadd at line 147

        
    /** 
     * Forwarding methods
     */
    
    //TODO: forwarding for getInfo() ? speciellt, ControlItem.getID()...
    public String getID() {
        if(this instanceof Command) {
            return ((Command)this).getInfo().getID();
        }else if(this instanceof Group) {
            return ((Group)this).getInfo().getID();
        }else if(this instanceof Param) {
            return ((Param)this).getInfo().getID();
        }
        return null;
    }

    // Declared in ServiceDescription.jadd at line 212

    
    /**
     * Returns the ControlItem within the sub-tree rooted at this ControlItem
     * that has the sought ID, or null if none exists.
     * 
     * This method is overridden in appropriate subclasses.
     */
    public ControlItem findItem(String id) {
        if (id.equals(getID())) {
            return this;
        } else {
            return null;
        }
    }

    // Declared in ServiceDescription.jadd at line 264


    /**
     * Returns the closest surrounding ServiceProxy, or null if there is
     * none. If this object is a ServiceProxy, <code>this</code> is
     * returned.
     * 
     * The method is overriden in ServiceProxy, to give the right
     * behaviour.
     */
    public ServiceProxy getServiceProxy() {
        if (parent == null) {
            return null;
        } else {
            List parentList = (List) parent;
            ControlItem parentControlItem = (ControlItem) parentList.parent;
            return parentControlItem.getServiceProxy();
        }
    }

    // Declared in ServiceDescription.jadd at line 290


    /**
     * Returns a matching command in the ControlItem sub-tree rooted at this
     * ControlItem, or null if none exists.
     * 
     * A Command is considered to match if it has the same Java type, the same
     * ID, and if all params have the same ID and the same types (param type),
     * and are in the same order.
     * 
     * This method is overridden in appropriate sub-classes, to handle the
     * actual matching.
     */
    public Command findMatchingCommand(Command command) {
        // No match for a general ControlItem
        return null;
    }

    // Declared in ServiceDescription.jadd at line 337


    public Message getMessage() {
        ArrayList dataList = new ArrayList();
        setDataRefsAndAddDataToList(dataList);
        Message result = new Message(Message.TYPE_MULTIPART);
        Message xmlMessage = new Message(getInfoAsInfo().getXMLData());
        result.addPart(xmlMessage);
        for (int i = 0; i < dataList.size(); i++) {
            byte[] data = (byte[]) dataList.get(i);
            result.addPart(new Message(data));
        }
        return result;
    }

    // Declared in ServiceDescription.jadd at line 350


    public abstract void setDataRefsAndAddDataToList(ArrayList dataList);

    // Declared in ServiceDescription.jadd at line 383

    
    public abstract void setDataAccordingToDataRefs(byte[][] data);

    // Declared in XML.jadd at line 105


    public String getTagName() {
        if(this instanceof Command) {
            return ((Command)this).getInfo().getTagName();
        }else if(this instanceof Group) {
            return ((Group)this).getInfo().getTagName();
        }else if(this instanceof Param) {
            return ((Param)this).getInfo().getTagName();
        }
        return null;
    }

    // Declared in XML.jadd at line 295

    
    public void initializeFromElement(XmlPullParser parser) {
        if(this instanceof Command) {
            ((Command)this).getInfo().initializeFromElement(parser);
        }else if(this instanceof Group) {
            ((Group)this).getInfo().initializeFromElement(parser);
        }else if(this instanceof Param) {
            ((Param)this).getInfo().initializeFromElement(parser);
        }
    }

    // Declared in XML.jadd at line 856

	
    protected void writeStartTagAndAttributes(KXmlSerializer serializer) throws IOException {
        if(this instanceof Command) {
            ((Command)this).getInfo().writeStartTagAndAttributes(serializer);
        }else if(this instanceof Group) {
            ((Group)this).getInfo().writeStartTagAndAttributes(serializer);
        }else if(this instanceof Param) {
            ((Param)this).getInfo().writeStartTagAndAttributes(serializer);
        }

    }

    // Declared in XML.jadd at line 890

        
    public void writeXMLElement(KXmlSerializer serializer) throws IOException {
        if(this instanceof Command) {
            ((Command)this).getInfo().writeXMLElement(serializer);
        }else if(this instanceof Group) {
            ((Group)this).getInfo().writeXMLElement(serializer);
        }else if(this instanceof Param) {
            ((Param)this).getInfo().writeXMLElement(serializer);
        }
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 75

    public ControlItem() {
        super();


    }

    // Declared in PalcomResourceDescriptor.ast at line 9


  protected int numChildren() {
    return 0;
  }

    protected boolean getBrowserLabel_visited = false;
    // Declared in LabelProviders.jrag at line 127
    public String getBrowserLabel() {
        if(getBrowserLabel_visited)
            throw new RuntimeException("Circular definition of attr: getBrowserLabel in class: ");
        getBrowserLabel_visited = true;
        String getBrowserLabel_value = getBrowserLabel_compute();
        getBrowserLabel_visited = false;
        return getBrowserLabel_value;
    }

    private String getBrowserLabel_compute()  {
		String s = getID();
		return (s != null ? s : new String("null"));
	}

}
