/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;


public class ParamInfo extends ControlInfo implements Cloneable {
    public void flushCache() {
        super.flushCache();
        getBrowserLabel_visited = false;
        getAssemblyLabel_visited = false;
        lookupParam_String_visited = new java.util.HashSet(4);
        encloseInBraces_visited = false;
    }
    public Object clone() throws CloneNotSupportedException {
        ParamInfo node = (ParamInfo)super.clone();
        node.getBrowserLabel_visited = false;
        node.getAssemblyLabel_visited = false;
        node.lookupParam_String_visited = new java.util.HashSet(4);
        node.encloseInBraces_visited = false;
    return node;
    }
    public ASTNode copy() {
      try {
          ParamInfo node = (ParamInfo)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        ParamInfo res = (ParamInfo)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in ServiceDescription.jadd at line 104

    
    public ParamInfo(String id, String type) {
        super(id, null);
        setID(id);
        setType(type);
    }

    // Declared in ServiceDescription.jadd at line 656

    
    public ControlItem createInstance() {
        return new Param(this);
    }

    // Declared in Comparison.jadd at line 260


    public boolean isAboutTheSameEntityAs(PRDData data) {
        return equals(data);
    }

    // Declared in Comparison.jadd at line 474


    public boolean equals(Object o) {
        if (!super.equals(o)) {
            return false;
        }

        if (o instanceof ParamInfo) {
            ParamInfo p = (ParamInfo) o;
            if (!PRDData.safeEqual(p.getType(), getType())) {
                return false;
            }
            if (p.getDataRef() != getDataRef()) {
                return false;
            }
            return true;
        } else {
            return false;
        }
    }

    // Declared in XML.jadd at line 124


    public String getTagName() {
        return "PI";
    }

    // Declared in XML.jadd at line 321


    public void initializeFromElement(XmlPullParser parser) {
        super.initializeFromElement(parser);
        setType(parser.getAttributeValue(null, "type"));
        String dataRefString = parser.getAttributeValue(null, "dataRef");
        if (dataRefString == null) {
            setDataRef(-1);
        } else {
            setDataRef(Integer.parseInt(dataRefString));
        }
    }

    // Declared in XML.jadd at line 652


    public void writeXMLElement(KXmlSerializer serializer) throws IOException {
        writeStartTagAndAttributes(serializer);
        serializer.attribute(null, "type", getType());
        if (getDataRef() != -1) {
            serializer.attribute(null, "dataRef", Integer.toString(getDataRef()));
        }
        serializer.endTag(null, getTagName());
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 72

    public ParamInfo() {
        super();


    }

    // Declared in PalcomResourceDescriptor.ast at line 10


    // Declared in PalcomResourceDescriptor.ast line 72
    public ParamInfo(String p0, String p1, String p2, String p3, int p4) {
        setID(p0);
        setHelp(p1);
        setType(p2);
        setName(p3);
        setDataRef(p4);
    }

    // Declared in PalcomResourceDescriptor.ast at line 18


  protected int numChildren() {
    return 0;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 69
    private String tokenString_ID;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setID(String value) {
        tokenString_ID = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getID() {
        return tokenString_ID;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 69
    private String tokenString_Help;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setHelp(String value) {
        tokenString_Help = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getHelp() {
        return tokenString_Help;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 72
    private String tokenString_Type;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setType(String value) {
        tokenString_Type = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getType() {
        return tokenString_Type;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 72
    private String tokenString_Name;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setName(String value) {
        tokenString_Name = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getName() {
        return tokenString_Name;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 72
    private int tokenint_DataRef;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setDataRef(int value) {
        tokenint_DataRef = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public int getDataRef() {
        return tokenint_DataRef;
    }

    // Declared in Hgraph.jadd at line 92

	  public void setID(String ID){
		refined_PalcomResourceDescriptor_setID(ID);
		if(ID != null){
			_node.setNodeName(getTagName() + " - " + ID, null);
		}
	}

    // Declared in Hgraph.jadd at line 110

	  public void setHelp(String help){
		refined_PalcomResourceDescriptor_setHelp(help);
		if(help != null){
			_node.setNodeName(getTagName() + " - " + help, null);
		}
	}

    // Declared in Hgraph.jadd at line 122

	  public void setType(String type){
		refined_PalcomResourceDescriptor_setType(type);
		if(type != null){
			_node.getValueFunc().setValue("type", type, null);
		}
	}

    // Declared in Hgraph.jadd at line 128

	  public void setDataRef(int ref){
		refined_PalcomResourceDescriptor_setDataRef(ref);
		_node.getValueFunc().setValue("data ref", new Integer(ref), null);
	}

    protected boolean getBrowserLabel_visited = false;
    // Declared in LabelProviders.jrag at line 100
    public String getBrowserLabel() {
        if(getBrowserLabel_visited)
            throw new RuntimeException("Circular definition of attr: getBrowserLabel in class: ");
        getBrowserLabel_visited = true;
        String getBrowserLabel_value = getBrowserLabel_compute();
        getBrowserLabel_visited = false;
        return getBrowserLabel_value;
    }

    private String getBrowserLabel_compute() {  return  getID();  }

    protected boolean getAssemblyLabel_visited = false;
    // Declared in LabelProviders.jrag at line 192
    public String getAssemblyLabel() {
        if(getAssemblyLabel_visited)
            throw new RuntimeException("Circular definition of attr: getAssemblyLabel in class: ");
        getAssemblyLabel_visited = true;
        String getAssemblyLabel_value = getAssemblyLabel_compute();
        getAssemblyLabel_visited = false;
        return getAssemblyLabel_value;
    }

    private String getAssemblyLabel_compute() {  return  getType().toString() + " " + getID();  }

    protected java.util.Set lookupParam_String_visited = new java.util.HashSet(4);
    // Declared in NameAnalysis.jrag at line 236
    public ParamInfo lookupParam(String id) {
        Object _parameters = id;
        if(lookupParam_String_visited.contains(_parameters))
            throw new RuntimeException("Circular definition of attr: lookupParam in class: ");
        lookupParam_String_visited.add(_parameters);
        ParamInfo lookupParam_String_value = lookupParam_compute(id);
        lookupParam_String_visited.remove(_parameters);
        return lookupParam_String_value;
    }

    private ParamInfo lookupParam_compute(String id)  {
	if(id.equals(getID()))
		return this;
	else
		return null;
}

    protected boolean encloseInBraces_visited = false;
    // Declared in PrettyPrint.jrag at line 76
    public boolean encloseInBraces() {
        if(encloseInBraces_visited)
            throw new RuntimeException("Circular definition of attr: encloseInBraces in class: ");
        encloseInBraces_visited = true;
        boolean encloseInBraces_value = encloseInBraces_compute();
        encloseInBraces_visited = false;
        return encloseInBraces_value;
    }

    private boolean encloseInBraces_compute() {  return  false;  }

}
