/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;


public class ServiceInstanceID extends AbstractXMLRepresentable implements Cloneable {
    public void flushCache() {
        super.flushCache();
    }
    public Object clone() throws CloneNotSupportedException {
        ServiceInstanceID node = (ServiceInstanceID)super.clone();
    return node;
    }
    public ASTNode copy() {
      try {
          ServiceInstanceID node = (ServiceInstanceID)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        ServiceInstanceID res = (ServiceInstanceID)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in Addressing.jadd at line 64

    
    public String toString() {
    	if(hasDeviceID() && getServiceID() != null) {
        	return getDeviceID().getString() + " " + getServiceID().toString() + getInstanceNumber();
        } else if(hasDeviceID()) {
        	return getDeviceID().getString()+ getInstanceNumber();
        } else if(getServiceID() != null){
        	return getServiceID().toString()+ getInstanceNumber();
        } 
        return null;
    }

    // Declared in Addressing.jadd at line 201

    
    public ServiceInstanceID(DeviceID deviceID, ServiceID serviceID, 
    		String instanceNumber) {
    	this(new Opt(), serviceID, instanceNumber);
    	setDeviceID(deviceID);
    	if(!hasDeviceID()) {
    		System.err.println("********ERROR! created SIID without deviceID!");
    		throw new Error("********ERROR! created SIID without deviceID!");
    	}
 	
    }

    // Declared in Comparison.jadd at line 549

	
	public boolean equals(Object o) {
		if(o instanceof ServiceInstanceID) {
			ServiceInstanceID sid = (ServiceInstanceID) o;
			if (!PRDData.safeEqual(sid.getInstanceNumber(), getInstanceNumber())) {
                return false;
            }
            if (!sid.getServiceID().equals(getServiceID())) {
                return false;
            }
            if (!PRDData.safeEqual(sid.getDeviceID(), getDeviceID())) {
                return false;
            }
            return true;
		} else {
			return false;
		}
	}

    // Declared in XML.jadd at line 152

	
	public String getTagName() {
		return "SIID";
	}

    // Declared in XML.jadd at line 370

	
	public void initializeFromElement(XmlPullParser parser) {
		setInstanceNumber(parser.getAttributeValue(null, "in"));
	}

    // Declared in XML.jadd at line 470

    
    public void addChildFromXMLElement(XMLRepresentable child) {
        if (child instanceof ServiceID) {
            setServiceID((ServiceID) child);
        }
        if (child instanceof DeviceID) {
        	setDeviceID((DeviceID)child);
        } 
    }

    // Declared in XML.jadd at line 706

	
	public void writeXMLElement(KXmlSerializer serializer) throws IOException {
		serializer.startTag(null, getTagName());
		serializer.attribute(null, "in", getInstanceNumber());
		if(hasDeviceID()) {
//			serializer.attribute(null, "did", getDeviceID().toString());
			getDeviceID().writeXMLElement(serializer);
		}
		if(getServiceID() != null) {
			getServiceID().writeXMLElement(serializer);
		}
		serializer.endTag(null, getTagName());
	}

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 97

    public ServiceInstanceID() {
        super();

        setChild(new Opt(), 0);
        setChild(null, 1);

    }

    // Declared in PalcomResourceDescriptor.ast at line 12


    // Declared in PalcomResourceDescriptor.ast line 97
    public ServiceInstanceID(Opt p0, ServiceID p1, String p2) {
        setChild(p0, 0);
        setChild(p1, 1);
        setInstanceNumber(p2);
    }

    // Declared in PalcomResourceDescriptor.ast at line 18


  protected int numChildren() {
    return 2;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 97
    public void setDeviceIDOpt(Opt opt) {
        setChild(opt, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 6


    public boolean hasDeviceID() {
        return getDeviceIDOpt().getNumChild() != 0;
    }

    // Declared in PalcomResourceDescriptor.ast at line 10


    public DeviceID getDeviceID() {
        return (DeviceID)getDeviceIDOpt().getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 14


    public void setDeviceID(DeviceID node) {
        getDeviceIDOpt().setChild(node, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 17

    public Opt getDeviceIDOpt() {
        return (Opt)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 21


    public Opt getDeviceIDOptNoTransform() {
        return (Opt)getChildNoTransform(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 97
    public void setServiceID(ServiceID node) {
        setChild(node, 1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public ServiceID getServiceID() {
        return (ServiceID)getChild(1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public ServiceID getServiceIDNoTransform() {
        return (ServiceID)getChildNoTransform(1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 97
    private String tokenString_InstanceNumber;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setInstanceNumber(String value) {
        tokenString_InstanceNumber = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getInstanceNumber() {
        return tokenString_InstanceNumber;
    }

}
