package ist.palcom.resourceAddressing;

import ist.palcom.collections.LinearMap;
import ist.palcom.collections.LinearSet;
import ist.palcom.device.DeviceContext;
import ist.palcom.discovery.InfoEvent;
import ist.palcom.discovery.ServiceInstanceIDEvent;
import ist.palcom.discovery.ServiceStatusEvent;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.Monitor;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.AbstractResourceAddress;
import ist.palcom.resource.descriptor.ConnectionInfo;
import ist.palcom.resource.descriptor.ConnectionInfoList;
import ist.palcom.resource.descriptor.DeviceAddress;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.DeviceInfo;
import ist.palcom.resource.descriptor.Info;
import ist.palcom.resource.descriptor.List;
import ist.palcom.resource.descriptor.LocalSID;
import ist.palcom.resource.descriptor.Opt;
import ist.palcom.resource.descriptor.PRDConnection;
import ist.palcom.resource.descriptor.PRDConnectionList;
import ist.palcom.resource.descriptor.PRDData;
import ist.palcom.resource.descriptor.PRDDevice;
import ist.palcom.resource.descriptor.PRDService;
import ist.palcom.resource.descriptor.PRDServiceFMDescription;
import ist.palcom.resource.descriptor.PRDServiceList;
import ist.palcom.resource.descriptor.PRDServiceListItem;
import ist.palcom.resource.descriptor.PRDSubList;
import ist.palcom.resource.descriptor.ResourceStatusInfo;
import ist.palcom.resource.descriptor.ServiceAddress;
import ist.palcom.resource.descriptor.ServiceDescription;
import ist.palcom.resource.descriptor.ServiceInfo;
import ist.palcom.resource.descriptor.ServiceInfoList;
import ist.palcom.resource.descriptor.ServiceInfoListItem;
import ist.palcom.resource.descriptor.ServiceInstanceID;
import ist.palcom.resource.descriptor.SubListInfo;
import ist.palcom.resource.descriptor.TemporaryAddress;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

public class ResourceAddressingManager {
	private DeviceContext context;
	private DiscoveryThread discoveryThread;
	private PalcomThread statusThread;
	private LinearSet discoveryReceivers;
	private Monitor drmon; // monitor protecting (iterators for) discoveryReceiver
	private LinearMap statusReceivers;
	private Monitor srmon; // D:o


	public ResourceAddressingManager(DeviceContext context) {
		this.context = context;
		discoveryReceivers = new LinearSet();
		drmon = new Monitor(context.getScheduler());
		statusReceivers = new LinearMap(); // AbstractResourceAddress --> LinearSet<PalcomThread>
		srmon = new Monitor(context.getScheduler());
		discoveryThread = new DiscoveryThread();
		context.getScheduler().scheduleThread(discoveryThread);
		statusThread = new StatusThread();
		context.getScheduler().scheduleThread(statusThread);
	}

	public void requestConnection(ConnectionInfo info) {
		AbstractResourceAddress pa = info.getProviderAddress();
		AbstractResourceAddress ca = info.getCustomerAddress();
		System.err.println("RAM.requestConnection:"+
				pa + " --> "+ ca + "\n"+ca.getDeviceAddress().getPRDDevice().getDeviceID());
		LocalSID plsid = null;
		LocalSID clsid = null;
		PRDService ps = null;
		PRDService cs = null;
		if(pa instanceof ServiceAddress) {
			ps = getPRDSForSIID(((ServiceAddress)pa).getServiceInstanceID());
//			ps = ((ServiceAddress)pa).getPRDService();
//			plsid = getLocalSIDForPRDService(ps);
			plsid = ps.getLocalSID();
		} else if (pa instanceof TemporaryAddress) {
			plsid = ((TemporaryAddress)pa).getLocalSID();
			ps = getPRDServiceForLocalSID(plsid);
		}
		if(ca instanceof ServiceAddress) {
			cs = getPRDSForSIID(((ServiceAddress)ca).getServiceInstanceID());
			System.err.println("... cs:"+cs.getLocalSID().getDeviceID());
			//			cs = ((ServiceAddress)ca).getPRDService();
			clsid = cs.getLocalSID(); // getLocalSIDForPRDService(cs);
//			System.err.println("... cls:"+clsid.getDeviceID());
		} else if (ca instanceof TemporaryAddress) {
			clsid = ((TemporaryAddress)ca).getLocalSID();
			cs = getPRDServiceForLocalSID(clsid);
		}
		System.err.println("RAM:requestConnection from "+ps+":"+plsid+" to "+cs+":"+clsid);
		PRDConnection prdc;
		//XXX This logic should really not be here, but rather in the remote connect manager.
		if(ps != null && cs != null) {
			if(ps.getRemoteConnect())
				prdc = new PRDConnection(clsid, plsid);
			else if(cs.getRemoteConnect())
				prdc = new PRDConnection(plsid, clsid);
			else {
				System.err.println("WARNING! ResourceAddressingManager.requestConnection: no service has RemoteConnect!");
				return;
			}
		} else {
			System.err.println("WARNING! ResourceAddressingManager.requestConnection: no PRDS?");
			prdc = new PRDConnection(plsid, clsid);
		}
		try {
			context.getRemoteConnectManager().requestConnect(prdc);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void requestDisconnection(ConnectionInfo info) {
		PRDConnection prdc=null;
		if(info.hasPRDConnection()) {
			prdc = info.getPRDConnection();
		}else {
			LocalSID plsid = null;
			LocalSID clsid = null;
			PRDService ps = null;
			PRDService cs = null;
			AbstractResourceAddress pa = info.getProviderAddress();
			AbstractResourceAddress ca = info.getCustomerAddress();
			if(pa instanceof ServiceAddress) {
//				ps = ((ServiceAddress)pa).getPRDService();
//				plsid = getLocalSIDForPRDService(ps);
				ps = getPRDSForSIID(((ServiceAddress)ca).getServiceInstanceID());
				plsid = ps.getLocalSID(); // getLocalSIDForPRDService(cs);
			} else if (pa instanceof TemporaryAddress) {
				plsid = ((TemporaryAddress)pa).getLocalSID();
				ps = getPRDServiceForLocalSID(plsid);
			}
			if(ca instanceof ServiceAddress) {
//				cs = ((ServiceAddress)ca).getPRDService();
//				clsid = getLocalSIDForPRDService(cs);
				cs = getPRDSForSIID(((ServiceAddress)ca).getServiceInstanceID());
				clsid = cs.getLocalSID(); // getLocalSIDForPRDService(cs);
			} else if (ca instanceof TemporaryAddress) {
				clsid = ((TemporaryAddress)ca).getLocalSID();
				cs = getPRDServiceForLocalSID(clsid);
			}
			prdc = new PRDConnection(plsid, clsid);
		}
		try {
			context.getRemoteConnectManager().requestDisconnect(prdc);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void listenForDevices(PalcomThread t) {
		try {
			drmon.enterMonitor();
			discoveryReceivers.add(t);
		} finally {
			drmon.exitMonitor();
		}
			context.getResourceManager2().updatePRDDeviceReceiver(discoveryThread);
	}

	public void listenForServices(PalcomThread t, DeviceAddress da) {
		addServiceReceiver(t, da);
		ArrayList services = knownPRDS.values();
		for (Iterator iter = services.iterator(); iter.hasNext();) {
			PRDService prds = (PRDService) iter.next();
			ServiceAddress sa = getAddressForService(prds);
			ServiceInfo si = new ServiceInfo(sa, prds.getName());
			dispatchServiceInfo(new AInfoEvent(si,true));
		}
	}

	private ServiceAddress getAddressForService(LocalSID lsid) {
		PRDService tmp = getPRDServiceForLocalSID(lsid);
		return getAddressForService(tmp);
	}

	private ServiceAddress getAddressForService(PRDService prds) {
		LocalSID lsid = prds.getLocalSID();
		ServiceInstanceID sid = getSIIDForLocalSID(lsid);
		ServiceAddress sa = new ServiceAddress(sid, prds, lsid);
		return sa;
	}

	public void listenForConnections(PalcomThread t, DeviceAddress da) {
		addConnectionReceiver(t, da);
		for (Iterator iter = knownPRDC.iterator(); iter.hasNext();) {
			PRDConnection prdc = (PRDConnection) iter.next();
			LocalSID lsid1 = prdc.getLocalSID1();
			LocalSID lsid2 = prdc.getLocalSID2();
			ServiceAddress sa1 = getAddressForService(lsid1);
			ServiceAddress sa2 = getAddressForService(lsid2);
			ConnectionInfo ci = new ConnectionInfo(sa1, sa2, new Opt(prdc));
			dispatchConnectionInfo(new AInfoEvent(ci,true));
		}
	}


	private void addServiceReceiver(PalcomThread t, DeviceAddress da) {
		System.err.println("RAM.addServiceReceiver:"+da);
		LinearSet tmp = (LinearSet) serviceReceivers.get(da);
		if(tmp == null) {
			tmp = new LinearSet();
			serviceReceivers.put(da,tmp);
		}
		tmp.add(t);
	}

	private void removeServiceReceiver(PalcomThread t, DeviceAddress da) {
		LinearSet tmp = (LinearSet) serviceReceivers.get(da);
		if(tmp != null)	tmp.remove(t);
	}

	private Iterator getServiceReceiverIterator(DeviceAddress da) {
		LinearSet tmp = (LinearSet) serviceReceivers.get(da);
		if(tmp!=null) return tmp.iterator();
		return null;
	}

	private void addConnectionReceiver(PalcomThread t, DeviceAddress da) {
		System.err.println("RAM.addConnectionReceiver:"+da);
		LinearSet tmp = (LinearSet) connectionReceivers.get(da);
		if(tmp == null) {
			tmp = new LinearSet();
			connectionReceivers.put(da,tmp);
		}
		tmp.add(t);
	}

	private void removeConnectionReceiver(PalcomThread t, DeviceAddress da) {
		LinearSet tmp = (LinearSet) connectionReceivers.get(da);
		if(tmp != null)	tmp.remove(t);
	}

	private Iterator getConnectionReceiverIterator(DeviceAddress da) {
		LinearSet tmp = (LinearSet) connectionReceivers.get(da);
		if(tmp!=null) return tmp.iterator();
		return null;
	}


	private LinearMap serviceReceivers = new LinearMap(); // DeviceAddress --> {PalcomThread}
	private LinearMap connectionReceivers = new LinearMap(); // DeviceAddress --> {PalcomThread}

	// The interface towards the palcom middleware managers follow:

	private LinearMap knownPRDS = new LinearMap(); //LocalSID --> PRDService
	private LinearMap knownLSID = new LinearMap(); // PRDService --> LocalSID
	private LinearMap knownSIID = new LinearMap(); // LocalSID --> ServiceInstanceID
	private LinearMap siidToPRDS = new LinearMap(); // SIID --> PRDService
	private LinearSet knownPRDC = new LinearSet(); // {PRDConnection}
	private LinearSet knownPRDD = new LinearSet(); // {PRDDevice}
	/** Wrapper class to replace "PRDService.equals" with a less exclusive one
	 * 
	 * @author sven
	 */
	private class PRDServiceNotice {
		private PRDService svc;

		public PRDServiceNotice(PRDService s) {
			this.svc = s;
		}
		//XXX This is a dirty hack to fix strange behaviour of PRDService.equals // .isAboutTheSameEntityAs
		public boolean equals(Object o) {
			if(o instanceof PRDService) {
				return svc.toString().equals(o.toString());
			} else {
				return false;
			}
		}

		public PRDService getService() {
			return svc;
		}
	}

	protected PRDService getPRDServiceForLocalSID(LocalSID sid) {
		return (PRDService) knownPRDS.get(sid);
	}
	protected LocalSID getLocalSIDForPRDService(PRDService s) {
		if(s.getLocalSID().getDeviceID()==null) {
			System.err.println("Warning! ResourceAddressingManager.getLocalSIDForPRDService: ignoring PRDService without DeviceID");
			return null;
		}
		return (LocalSID) knownLSID.get(s);
	}
	protected PRDService getPRDSForSIID(ServiceInstanceID siid) {
		PRDService s = (PRDService) siidToPRDS.get(siid);
		if(!siid.hasDeviceID() || siid.hasDeviceID() && siid.getDeviceID()==null) {
			System.err.println("Error! RAM.getPRDSForSIID. sid has no device ID");
		}
		if(s != null && s.getLocalSID().getDeviceID()==null) {
			System.err.println("RAM.getPRDSForSIID. PRDS has no DeviceID to "+s);
			s = null;
		}
		if( s == null) {
			System.err.println("WARNING! RAM.getPRDSForSIID("+siid+") returns null");
		}
		return s;
	}

	protected ServiceInstanceID getSIIDForLocalSID(LocalSID sid) {
		ServiceInstanceID siid = (ServiceInstanceID) knownSIID.get(sid);
		if(siid == null) {
			siid = discoveryThread.getTranslator().getSIDForLSID(sid);
		}
		return siid;
	}

	protected LocalSID getLocalSIDForSIID(ServiceInstanceID siid) {
		//XXX rewrite
		Iterator i = knownSIID.keySet().iterator();
		while(i.hasNext()) {
			LocalSID lsid = (LocalSID) i.next();
			if(knownSIID.get(lsid).equals(siid))
				return lsid;
		}
		return null;
	}

	private void updateKnownServices(boolean keepInfo, PRDService service) {
		LocalSID lsid = service.getLocalSID();
		if(lsid.getDeviceID()==null) {
			System.err.println("Warning! ResourceAddressingManager.updateKnownServices: ignoring PRDService without DeviceID");
			return;
		}
		if(keepInfo) {
//			System.err.println("adding ["+service.getLocalSID()+"] to knownServices");
			knownPRDS.put(lsid, service);
			knownLSID.put(new PRDServiceNotice(service), lsid);
		} else {
//			System.err.println("removing ["+service.getLocalSID()+"] from knownServices");
			PRDService tmp = (PRDService) knownPRDS.remove(lsid);
			if(tmp != null) {
				knownLSID.remove(tmp);
			}
			knownSIID.remove(lsid);
		}
	};

	private void updateKnownServices(InfoEvent event) {
		PRDData data = event.getInfo();
		if(data instanceof PRDService) {
			updateKnownServices(event.infoShouldBeKept(), ((PRDService)data));
		} else if (data instanceof PRDServiceList) {
			PRDServiceList sl = (PRDServiceList) data;
			for(int i=0; i<sl.getNumPRDServiceListItem();i++) {
				PRDServiceListItem sli = sl.getPRDServiceListItem(i);
				if(sli instanceof PRDService) {
					updateKnownServices(event.infoShouldBeKept(), (PRDService)sli);
				} else if(sli instanceof PRDSubList) {
//					System.err.println("Warning! ResourceAddressingManager.updateKnownServices: Ignoring sublist "+sli.getName());
				} else {
					System.err.println("Warning! ResourceAddressingManager.updateKnownServices doesn't handle "+sli);
				}
			}
		}
	}

	private void updateKnownConnections(PRDConnection c, boolean keep) {
		if(keep) {
			knownPRDC.add(c);
		} else {
			knownPRDC.remove(c);
		}
	}

	private void updateKnownConnections(InfoEvent event) {
		PRDData data = event.getInfo();
		if(data instanceof PRDConnectionList) {
			PRDConnectionList l = (PRDConnectionList)data;
			for(int i=0; i < l.getNumPRDConnection();i++) {
				updateKnownConnections(l.getPRDConnection(i), event.infoShouldBeKept());
			}
		} else if (data instanceof PRDConnection) {
			updateKnownConnections((PRDConnection) data, event.infoShouldBeKept());
		} 
	}

	private void updateKnownSIID(ServiceInstanceID siid, LocalSID lsid) {
		knownSIID.put(lsid, siid);
		PRDService tmp = getPRDServiceForLocalSID(lsid);
		siidToPRDS.put(siid, tmp);
	}

	/** A class responsible for translating from the discovery layer messages to
	 *  the corresponding high level addressing entity.
	 *  
	 * @author sven
	 */
	private class HLATranslator {
		protected AInfoEvent translate(InfoEvent e) {
			//XXX introduce factory
			PRDData tmp = e.getInfo();
			Info tmpi=null;
			if(tmp instanceof PRDDevice) {
				PRDDevice dtmp = (PRDDevice)tmp;
				DeviceAddress da = new DeviceAddress(dtmp);
				tmpi = new DeviceInfo(da,dtmp.getName());
//				} else if (tmp instanceof PRDService) {
//				PRDService stmp = (PRDService)tmp;
//				LocalSID ltmp = getLocalSIDForPRDService(stmp);
//				ServiceAddress sa;

//				tmpi = new ServiceInfo(sa, stmp.getName());
			} else if (tmp instanceof PRDServiceList) {
				tmpi = translate(((PRDServiceList) tmp));
			} else if (tmp instanceof PRDConnectionList) {
				tmpi = translate((PRDConnectionList)tmp);
			} else if (tmp instanceof PRDConnection) {
				tmpi = translate((PRDConnection)tmp);
			} else if (tmp instanceof PRDServiceFMDescription) {
				tmpi = translate((PRDServiceFMDescription) tmp);
			} else {
				throw new Error("HLATranslator: Unhandled type of PRDData: "+tmp.getClass());
			}
			if(tmpi == null){
				throw new Error("HLATranslator: tmpi==null?");
			}
			AInfoEvent result = new AInfoEvent(tmpi, e.infoShouldBeKept());
			return result;
		}

		private ServiceDescription translate(PRDServiceFMDescription description) {
			LocalSID localSID = description.getLocalSID();
			PRDService prds = getPRDServiceForLocalSID(localSID);
			ServiceInstanceID siid = getSIIDForLocalSID(localSID);
			if(prds != null) {
				ServiceAddress sa = new ServiceAddress(siid, prds, localSID);
				return new ServiceDescription(sa, description);
			} else {
				TemporaryAddress ta = new TemporaryAddress(localSID);
				return new ServiceDescription(ta, description);
				//throw new Error("HLAT.translate(PRDServiceFMDescription) prds==null for lsid ["+localSID+"]");
			}

		}

		private ConnectionInfoList translate(PRDConnectionList cl) {
			ConnectionInfoList result = new ConnectionInfoList(new List());
			for(int i = 0; i < cl.getNumPRDConnection(); i++) {
				result.addConnectionInfo(translate(cl.getPRDConnection(i)));
			}
			return result;
		}

		private ConnectionInfo translate(PRDConnection connection) {
			LocalSID cust = connection.getLocalSID2();
			LocalSID prov = connection.getLocalSID1();
			PRDService cs = getPRDServiceForLocalSID(cust);
			PRDService ps = getPRDServiceForLocalSID(prov);
			ServiceInstanceID csiid = getSIIDForLocalSID(cust);
			ServiceInstanceID psiid = getSIIDForLocalSID(prov);
			if(cs != null && ps != null) {
				ServiceAddress sa1 = new ServiceAddress(csiid, cs, cust);
				ServiceAddress sa2 = new ServiceAddress(psiid, ps, prov);
				ConnectionInfo result = new ConnectionInfo(sa1, sa2, new Opt(connection));
				return result;
			} else {
//				throw new Error("HLAT.translate(PRDConnection): cls="+cust+", cs="+cs+" , pls="+prov+" ps="+ps);
				TemporaryAddress sa1 = new TemporaryAddress(connection.getLocalSID1());
				TemporaryAddress sa2 = new TemporaryAddress(connection.getLocalSID2());
				ConnectionInfo result = new ConnectionInfo(sa1, sa2, new Opt(connection));
				return result;
			}
		}

		private ServiceInfoList translate(PRDServiceList list) {
			LocalSID parentLocalSID = list.getParentLocalSID();
			AbstractResourceAddress pa = null;
			if(parentLocalSID.getServicePartID() == null) {
				PRDDevice prdd = new PRDDevice(parentLocalSID.getDeviceID());
				pa = new DeviceAddress(prdd);
			} else {
				pa = new TemporaryAddress(parentLocalSID);
			}
			ServiceInfoList result = new ServiceInfoList(pa, new List());
			Iterator iterator = list.getChildIterator();
			while(iterator.hasNext()) {
				Object next = iterator.next();
				if(next instanceof PRDService) {
					PRDService s = (PRDService) next;
//					LocalSID ltmp = getLocalSIDForPRDService(s);
					LocalSID ltmp = s.getLocalSID();
					ServiceInstanceID sid = getSIDForLSID(ltmp);
					if(sid != null) {
						ServiceAddress sa;
						if(ltmp != null)  {
							sa = new ServiceAddress(sid, s, ltmp);				
						} else {
							sa = new ServiceAddress(sid, s);
						}
						ServiceInfo si = new ServiceInfo(sa, s.getName());
						result.addServiceInfoListItem(si);
					}
				} else if(next instanceof PRDSubList) {
					PRDSubList prd = (PRDSubList) next;
					getServiceListsForSubList(prd.getListLocalSID());
					TemporaryAddress tmpA = new TemporaryAddress(prd.getListLocalSID());
					SubListInfo ltmp = new SubListInfo(pa, tmpA, prd.getName(), prd.getKindAsString());
					result.addServiceInfoListItem(ltmp);
				}
			}
			return result;
		}

		private class SIIDThread extends PalcomPriorityThread {

			private Monitor mon = new Monitor(context.getScheduler());
			private Monitor.Condition cond = mon.new Condition();
			private LocalSID ls;
			private ServiceInstanceID result;
			private int timeout;

			public SIIDThread(LocalSID ls) {
				this.ls = ls;
			}
			private void enableTimeout(int t) {
				timeout = t;
			}

			public void run() {
				context.getResourceManager2().updateServiceInstanceIDReceiver(ls, this);
				enablePeriodicEvents(timeout);				
				Event e = waitEvent();
				if(e instanceof ServiceInstanceIDEvent) {
					mon.enterMonitor();
					result = ((ServiceInstanceIDEvent)e).getServiceInstanceID();
					cond.causeAll();
					mon.exitMonitor();
				} else {
					System.err.println("WARNING! SIIDThread got event:"+e);
					result = null;
					mon.enterMonitor();
					cond.causeAll();
					mon.exitMonitor();
				}
				terminate();
			}

			public ServiceInstanceID waitForSIID(int timeout) {
				ServiceInstanceID result;
				enableTimeout(timeout);
				context.getScheduler().scheduleThread(this);
				mon.enterMonitor();
				cond.await();
				result = this.result;
				mon.exitMonitor();
				return result;
			}
		}

		/** A blocking method requesting a service instance id matching the localSID
		 *  Current implementation has a timeout to avoid blocking forever
		 * @param ltmp - the LocalSID to look up
		 * @return a service instance ID, or null if the operation timed out
		 */
		public ServiceInstanceID getSIDForLSID(final LocalSID ltmp) {
			SIIDThread sit = new SIIDThread(ltmp);
//			System.err.println("RAM.getSIDForLSID. waiting: "+ltmp);
			ServiceInstanceID result = sit.waitForSIID(15000);
//			System.err.println("RAM.getSIDForLSID. returning: "+result);
			if(!result.hasDeviceID()) {
				System.err.println("**************************************");
				System.err.println("WARNING! RAM.getSIDForLSID: no DeviceID: lsid="+ltmp);
				System.err.println("**************************************");
			}
			if(result != null) {
				updateKnownSIID(result, ltmp);
			}
			return result;
		}
	}

	private void dispatchServiceInfo(AInfoEvent aie) {
		ServiceInfo info = (ServiceInfo) aie.getInfo();
		DeviceAddress da = info.getServiceAddress().getDeviceAddress();
		Iterator i = getServiceReceiverIterator(da);
//		System.err.println("RAM: dispatchServiceInfo");
		while(i != null && i.hasNext()) {
//			System.err.println("... putEvent");
			PalcomThread t = (PalcomThread) i.next();
			t.putEvent(new AInfoEvent(aie.getInfo(), aie.infoShouldBeKept()));
		}
	}

	private void dispatchConnectionInfo(AInfoEvent aie) {
		ConnectionInfo info = (ConnectionInfo) aie.getInfo();
		DeviceAddress da1 = info.getCustomerAddress().getDeviceAddress();
		DeviceAddress da2 = info.getCustomerAddress().getDeviceAddress();

		Iterator i = getConnectionReceiverIterator(da1);
//		System.err.println("RAM: dispatchServiceInfo");
		while(i != null && i.hasNext()) {
//			System.err.println("... putEvent");
			PalcomThread t = (PalcomThread) i.next();
			t.putEvent(new AInfoEvent(aie.getInfo(), aie.infoShouldBeKept()));
		}
		i = getConnectionReceiverIterator(da2);
//		System.err.println("RAM: dispatchServiceInfo");
		while(i != null && i.hasNext()) {
//			System.err.println("... putEvent");
			PalcomThread t = (PalcomThread) i.next();
			t.putEvent(new AInfoEvent(aie.getInfo(), aie.infoShouldBeKept()));
		}
	}


	/** Flatten service info lists, and send a number of service info events
	 **/
	private void checkAndDispatchServicesAndConnections(AInfoEvent translated) {
//		System.err.println("************************************");
//		System.err.println("RAM: checkAndDispatchServices:"+translated);
		if(translated.getInfo() instanceof ServiceInfo) {
			dispatchServiceInfo(translated);
		} else if(translated.getInfo() instanceof ServiceInfoList) {
			ServiceInfoList sli = (ServiceInfoList) translated.getInfo();
			for(int i =0; i<sli.getNumServiceInfoListItem(); i++) {
				ServiceInfoListItem tmp = sli.getServiceInfoListItem(i);
				if(tmp instanceof ServiceInfo) {
					dispatchServiceInfo(new AInfoEvent(tmp,translated.infoShouldBeKept()));
				}
			}
		} else if(translated.getInfo() instanceof ConnectionInfoList) {
			ConnectionInfoList cli = (ConnectionInfoList)translated.getInfo();
			for(int i=0; i<cli.getNumConnectionInfo();i++) {
				dispatchConnectionInfo(new AInfoEvent(cli.getConnectionInfo(i),translated.infoShouldBeKept()));
			}
		} else if(translated.getInfo() instanceof ConnectionInfo) {
//			System.err.println("RAM.checkAndDispatch: connectionInfo");
			dispatchConnectionInfo(new AInfoEvent(translated.getInfo(), translated.infoShouldBeKept()));
		}		
	}

	private class DiscoveryThread extends PalcomPriorityThread {

		private HLATranslator hlaTranslator = new HLATranslator();

		public HLATranslator getTranslator() {return hlaTranslator;}

		public void run() {
			while (true) {
				InfoEvent event = (InfoEvent) waitEvent();
//				System.err.println("RAM.DiscoveryThread: Got Info: "+event.getInfo().getClass());
				PRDData info = event.getInfo();
				if(info instanceof PRDDevice) {
					getServiceListsAndConnectionsForDevice(info);
					updateResourceStatus((PRDDevice) info);
				} else if (info instanceof PRDServiceList) {
					updateKnownServices(event);
					getServiceDescriptionsForServiceList(info);
				} else if (info instanceof PRDConnectionList || info instanceof PRDConnection) {
					updateKnownConnections(event);
				}
				dispatchEvent(event);
			}
		}

		/**
		 * Translate the event to the corresponding high level representation, and
		 * send an AInfoEvent to the registered listeners.
		 * 
		 * @param event
		 */
		private void dispatchEvent(InfoEvent event) {
			try {
				drmon.enterMonitor();

				for (Iterator iter = discoveryReceivers.iterator(); iter.hasNext();) {
					try {
						PalcomThread t = (PalcomThread) iter.next();
						AInfoEvent translated = hlaTranslator .translate(event);
						checkAndDispatchServicesAndConnections(translated);
						t.putEvent(translated);
					} catch (Throwable t) {
						t.printStackTrace();
					}
				}
			} finally {
				drmon.exitMonitor();
			}
		}
	}

//	private LinearMap knownStatus = new LinearMap(); // AbstractResourceAddress --> ResourceStatusInfo

	private void updateResourceStatus(PRDDevice prdd) {
		DeviceAddress d = new DeviceAddress(prdd.getDeviceID());
//		ResourceStatusInfo s = (ResourceStatusInfo) knownStatus.get(d);
//		if(s != null) {
//		if(s.getStatus() != prdd.getDeviceStatus()) {
//		System.err.println("********* RAM. changed Status ("+(char)prdd.getDeviceStatus()+") for "+d);
//		s.setStatus(prdd.getDeviceStatus());
//		s.setHelpText("");
//		fireStatusInfo(d,s);
//		}
//		} else {
		ResourceStatusInfo s = new ResourceStatusInfo(d, prdd.getDeviceStatus(), "");
//		System.err.println("********* RAM. new Status ("+(char)prdd.getDeviceStatus()+") for "+d);
		updateResourceStatus(d, s);
//		}

	}

	private void updateResourceStatus(AbstractResourceAddress a, ResourceStatusInfo i) {
//		knownStatus.put(a,i);
		fireStatusInfo(a, i);
	}

	public void listenForStatusChanges(AbstractResourceAddress a, PalcomThread t) {
//		System.err.println("*********** RAM.ListenForStatusChanges: "+a);
		LinearSet set = (LinearSet) statusReceivers.get(a);
		if(set == null) {
			set = new LinearSet();
			try {
				srmon.enterMonitor();
				statusReceivers.put(a, set);
			}finally {
				srmon.exitMonitor();
			}
		}
		set.add(t);
		if(a instanceof ServiceAddress || a instanceof TemporaryAddress) {
			LocalSID lsid;
			
			if(a instanceof ServiceAddress) {
				ServiceInstanceID siid = ((ServiceAddress)a).getServiceInstanceID();
				lsid = getLocalSIDForSIID(siid);
			} else {
				lsid = ((TemporaryAddress)a).getLocalSID();
			}
			
			context.getResourceManager2().updateServiceStatusReceiver(lsid, statusThread);
		}
//		fireStatusInfo(a);
	}

	//XXX this should perhaps iterate over all services on a device
//	private void fireStatusInfo(AbstractResourceAddress a) {
//	ResourceStatusInfo i = (ResourceStatusInfo) knownStatus.get(a);
//	if(i != null) {
////	System.err.println("*********** RAM.fireStatusInfo: "+a);
//	fireStatusInfo(a,i);
//	}
//	}

	private void fireStatusInfo(AbstractResourceAddress a, ResourceStatusInfo i) {
		try {
			srmon.enterMonitor(); //unneccessarily large critical section, but safe
			LinearSet set = (LinearSet) statusReceivers.get(a);
//			LinearSet set = discoveryReceivers;
			if(set != null) {
				Iterator iter = set.iterator();
				while(iter.hasNext()) {
//					System.err.println("********** RAM.fireStatusInfo "+a);
					((PalcomThread)iter.next()).putEvent(new AInfoEvent(i, true));
				}
			}
		}finally {
			srmon.exitMonitor();
		}
	}

	private class StatusThread extends PalcomPriorityThread {
		public void run() {
			while(true) {
				Event e = waitEvent();
				if(e instanceof ServiceStatusEvent) {
//					System.err.println("********** RAM.StatusThread. got "+(char)((ServiceStatusEvent)e).getServiceStatus());
					ServiceStatusEvent se = (ServiceStatusEvent) e;
					LocalSID lsid = se.getLocalSID();
					byte status = se.getServiceStatus();
					String helpTxt = se.getServiceHelpText();
					ServiceInstanceID siid = getSIIDForLocalSID(lsid);
					AbstractResourceAddress ta;
					if(siid != null) {
						ta = new ServiceAddress(siid, new Opt(), new Opt(lsid));
					} else {
						ta = new TemporaryAddress(lsid);
					}
					updateResourceStatus(ta, new ResourceStatusInfo(ta, status, helpTxt));
				} else {
					System.err.println("********** RAM.StatusThread. got "+e);
				}
			}
		}
	}

	private void getServiceDescriptionsForServiceList(PRDData info) {
		PRDServiceList list = (PRDServiceList) info;
		for (int i = 0; i < list.getNumPRDServiceListItem(); i++) {
			PRDServiceListItem sli = list.getPRDServiceListItem(i);
			if(sli instanceof PRDService && ((PRDService)sli).getHasDesc()) {
				LocalSID lsid = sli.getLocalSID();
				context.getResourceManager2().updatePRDServiceFMDescriptionReceiver(lsid, discoveryThread);
			}
		}
	}

	private void getServiceListsAndConnectionsForDevice(PRDData info) {
		DeviceID deviceID = ((PRDDevice)info).getDeviceID();
		context.getResourceManager2().updatePRDServiceListReceiver(new LocalSID(deviceID, null), discoveryThread);
		context.getResourceManager2().updatePRDConnectionReceiver(deviceID, discoveryThread);
	}


	protected void getServiceListsForSubList(LocalSID listLocalSID) {
		context.getResourceManager2().updatePRDServiceListReceiver(listLocalSID, discoveryThread);		
	}

	public AbstractResourceAddress validateAddress(AbstractResourceAddress sa1) {
		if(sa1 instanceof ServiceAddress) {
//			System.err.println("RAM.validateAddress: "+sa1);
			ServiceAddress sa = (ServiceAddress) sa1;
			ServiceInstanceID siid = sa.getServiceInstanceID();
			LocalSID lsid = getLocalSIDForSIID(siid);
			if(sa.hasLocalSID()) {
				if(lsid == null) {
//					System.err.println("...(1) no LSID found");
					return new ServiceAddress(siid, new Opt(), new Opt());
				} else 	if(!sa.getLocalSID().equals(lsid)) {
//					System.err.println("... changed LSID found");
					Opt o1;
					if(sa.hasPRDService()) {
						PRDService tmp = (PRDService) sa.getPRDService().fullCopy();
						tmp.setLocalSID((LocalSID)lsid.fullCopy());
						o1 = new Opt(tmp);
					} else {
						o1 = new Opt();
					}
					Opt o2 = new Opt(lsid.fullCopy());
					return new ServiceAddress(siid, o1, o2);
				}
			} else if (sa.hasPRDService()) {
				if(lsid == null) {
//					System.err.println("...(2) no LSID found");
					return new ServiceAddress(siid, new Opt(), new Opt());
				} else 	if(!sa.getPRDService().getLocalSID().equals(lsid)) {
//					System.err.println("... changed LSID found");
					PRDService tmp = (PRDService) sa.getPRDService().fullCopy();
					tmp.setLocalSID((LocalSID)lsid.fullCopy());
					Opt o1 = new Opt(tmp);
					Opt o2 = new Opt(lsid.fullCopy());
					return new ServiceAddress(siid, o1, o2);
				}
			}
		}
//		System.err.println("RAM.validateAddress: (default)"+sa1);
		return sa1;
	}

	public void stopListeningForConnections(PalcomThread thread, DeviceAddress da1) {
		System.err.println("WARNING! ResourceAddressingManager.stopListeningForConnections not implemented");
		
	}
}
