/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.test;

import ist.palcom.communication.CommunicationManager;
import ist.palcom.discovery.DiscoveryManager;
import ist.palcom.mal.MediaManager;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageEvent;
import ist.palcom.mal.udp.UDPMediaManager;
import ist.palcom.messaging.PointToPointManager;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.LocalSID;
import ist.palcom.resource.descriptor.PRDDevice;
import ist.palcom.resource.descriptor.PRDService;
import ist.palcom.resource.descriptor.PRDServiceList;
import ist.palcom.resource.descriptor.Selector;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.routing.util.AbstractRoutingManager;
import ist.palcom.routing.util.DSDVRoutingManager;
import ist.palcom.routing.util.RoutingPubSubManager;

/**
 * This is a simple receiver-service that use the DSDVRoutingManager.
 * 
 * @author jmahle
 *
 */
public class DSDVRoutingReceiver {
	
	private MediaManager mediaManager;
	
	private AbstractRoutingManager routingManager;
	
	private PointToPointManager pointToPointManager;
	
	private DiscoveryManager discoveryManager;
	
	private final PalcomScheduler scheduler = new PalcomScheduler();
	
	/**
	 *
	 */
	public DSDVRoutingReceiver() {
		PalcomThread t = new PalcomPriorityThread() {
        	public void run() {
        		// Setup
        		mediaManager = new UDPMediaManager(scheduler, null);
        		//mediaManager = new BluetoothMediaManager(scheduler, null);
        		final DeviceID receiverDeviceID = new DeviceID("X:device_10.11.23.191:2000");
        		routingManager = new DSDVRoutingManager(scheduler, mediaManager, new RoutingPubSubManager(scheduler, mediaManager), new URN(receiverDeviceID.toString()));
        		CommunicationManager comManager = new CommunicationManager(scheduler, routingManager);
//        		CommunicationManager comManager = new CommunicationManager(scheduler, new BasicRoutingManager(mediaManager)); //Without routing
        		pointToPointManager = new PointToPointManager(comManager);
        		
        		System.out.println("DSDV ReceiverURN: " + receiverDeviceID.toString());
        		final PRDDevice deviceInfo = new PRDDevice(receiverDeviceID);
            	deviceInfo.setName("DSDV Test Receiver Service");
                
            	Selector receiverSelector = pointToPointManager.startReceiving2(this);
            	
                
                discoveryManager = new DiscoveryManager(scheduler, pointToPointManager, 
                		comManager, deviceInfo);
        		
                PRDServiceList serviceList = new PRDServiceList(new LocalSID(deviceInfo.getDeviceID(),null));
                PRDService serviceInfo = new PRDService("Receiver", new LocalSID(deviceInfo.getDeviceID(),"test"), PRDService.UNICAST, 
                		false, false, "P1", false, null, null);
                serviceList.addPRDService(serviceInfo);
                discoveryManager.updateLocalServiceList(serviceList);
                
                // Start discovery and listening for incoming messages
                discoveryManager.start();
                
                System.out.println("Waiting for incoming messages");
                
        		while(true) {
        			Event event = waitEvent();
        			
        			if (event instanceof MessageEvent) {
        				MessageEvent e = (MessageEvent) event;
        				Message message = e.getMessage();
        				byte[] result = new byte[message.getTotalLength()];
        		        message.write(result, 0);
        		        Message sm = Message.parse(result, 0, result.length);
        		        String msg = new String(sm.getData());
        		        
        		        if (msg.equals("testdsdv")) {
        		        	System.out.println("Recieved test DSDV message");
        		        	System.out.println("- Sender device ID: " + message.getSenderDeviceID());
        		        	System.out.println("- Receiver device ID: " + message.getReceiverDeviceID());
        		        	System.out.println("- Sender URL: " + message.getSenderURL());
        		        	System.out.println("- Receiver URL: " + message.getReceiverURL());
        		        	System.out.println();
        		        }
        			}
        		}
        	}
        };
        scheduler.scheduleThread(t);
        scheduler.run();
	}
	
	public static void main(String[] args) {
		new DSDVRoutingReceiver();
	}

}
