/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util;

import ist.palcom.collections.LinearMap;
import ist.palcom.collections.LinkedList;
import ist.palcom.hgraph.FuncFunc;
import ist.palcom.hgraph.HgraphNode;
import ist.palcom.hgraph.IHgraphNode;
import ist.palcom.hgraph.Path;
import ist.palcom.hgraph.SiblingFunc;
import ist.palcom.hgraph.ValueFunc;
import ist.palcom.mal.MediaManager;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageEvent;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.palcomthreads.PeriodicEvent;
import ist.palcom.resource.descriptor.Selector;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.routing.RoutingManager;
import ist.palcom.routing.util.messages.AbstractRoutingMessage;

import java.util.Iterator;

/**
 * This class is work in progress :-)
 * 
 * @author Jacob Mahler-Andersen, jmahle@daimi.au.dk and Jacob Frolund, frolund@daimi.au.dk
 */
public abstract class AbstractRoutingManager implements RoutingManager{
	
	protected HgraphNode _node;

	protected final MediaManager manager;
	
	protected RoutingPubSubManager pubSubManager;
	
	protected PalcomScheduler scheduler;
	
	protected PalcomThread recieverThread;
	
	protected LinearMap registeredThreads;
	
	private MessageThread internalThread;

	private URL deviceURL;
	
	private URN deviceURN;
	
	private Selector deviceSelector;
	
	/**
	 * Create a new Routing manager.
	 * 
	 * @param scheduler - the palcom scheduler used in this service.
	 * @param manager - the mediamanager used in this service, this could be a compound media manager.
	 * @param pubSubManager - the RoutingPuSubManager
	 * @param deviceURN - information about the device e.g. the URN of the device
	 */
	public AbstractRoutingManager(PalcomScheduler scheduler, MediaManager manager, RoutingPubSubManager pubSubManager, URN deviceURN) {
		this.scheduler = scheduler;
		this.manager = manager;
		this.pubSubManager = pubSubManager;
		this.deviceURN = deviceURN;
		
		internalThread = new MessageThread();
		registeredThreads = new LinearMap();
		scheduler.scheduleThread(internalThread);
		
		/******hgraph impl*******/
		_node = new HgraphNode("routing manager");
		
//		deviceSelector = manager.startReceiving2(internalThread);
//		deviceURL = manager.getDefaultUnicastURL();
		
		startRouting();
	}
	
	/**
	 * Method that should be implemented to handle the different routing messages that 
	 * the specific implementation of a routing manager can handle.
	 * 
	 * @param message - message send between the nodes in the network to update/establish 
	 * routing.
	 */
	protected abstract void handleRoutingMessage(AbstractRoutingMessage message);
	
	protected abstract void startRouting();
	
	public abstract boolean send(Message message, URN senderURN, Selector senderSelector, URN receiverURN, Selector receiverSelector);
	
	/**
     * Start receiving messages, at an URL selected by the manager. Messages will
     * be sent to the receiver in MessageEvents.
     * @deprecated
     * @return PalCom URL for sending messages here.
     */
    public URL startReceiving(PalcomThread receiver) {   	
//    	URL url = manager.startReceiving(recieverThread);
//   
//    	return url; 
    	return null;
    }
    
    /**
     * Start receiving messages at a URL. Messages will be sent to the receiver
     * in MessageEvents.
     * 
     * @return PalCom URL for sending messages here.
     */
//    public URL startReceiving(URL u, PalcomThread receiver) {
//    	URL url = manager.startReceiving(u, recieverThread);
//    	
//    	return url;
//    }
    
//    public Selector startReceiving2(PalcomThread receiver) {
//    	URL url = startReceiving(receiver);
//    	Selector selector = manager.getSelector(url);
//    	registeredThreads.put(selector, receiver);
//    	System.out.println("Start receiving med selector " + selector.getString());
//        return selector;
//    }

    /** 
     * Stop all receiving of messages at a URL. 
     * @deprecated
     * */
//    public void stopReceiving(URL url) {
//        manager.stopReceiving(url);
//    }

    /** 
     * Stop receiving messages for a receiver at a URL. 
     * @deprecated
     * */
//    public void stopReceiving(URL url, PalcomThread receiver) {
//    	// TODO
//    	
//    	manager.stopReceiving(url, receiver);
//    }
    
    /** Stop receiving messages at a selector. */
    public void stopReceiving(Selector selector) {
//    	registeredThreads.remove(selector);
//        manager.stopReceiving(selector);
    }
    
    /** Stop receiving messages for a receiver. */
    public void stopReceiving(PalcomThread receiver) {
//    	ArrayList list = registeredThreads.values();
//    	for (int i = 0; i < list.size(); i++) {
//    		if (((PalcomThread)list.get(i)).equals(receiver))
//    			list.remove(i);
//    	}    	
//    	manager.stopReceiving(receiver);
    }

    /**
     * Stop the manager. This will stop all listening for messages.
     */
    public void stop() {
    	manager.stop();
    }
    
//	public URL[] getDefaultGroupAddresses() {
//		return manager.getDefaultGroupAddresses();
//	}
	
//	protected URL getDefaultUnicastURL() {
//		return deviceURL;
//	}
	
	public URN getRoutingURN() {
		return deviceURN;
	}
	
	public Selector getRoutingSelector() {
		return deviceSelector;
	}
	
	
//	The following methods supports the new wire protocol
	public boolean send(Message message, String receiverDeviceID) {
		return manager.send(message, receiverDeviceID);
	}
	public void addErrorHandler(PalcomThread handler) {
		manager.addErrorHandler(handler);
	}
	public void setLocalDeviceCache(String localDeviceCache) {
		manager.setLocalDeviceCache(localDeviceCache);
	}
	public String getLocalDeviceCache() {
		return manager.getLocalDeviceCache();
	}
	public void setLocalDeviceID(String localDeviceID) {
		manager.setLocalDeviceID(localDeviceID);
	}
    public void setLocalStatusFlag(byte statusFlag) {
        manager.setLocalStatusFlag(statusFlag);
    }
    // TODO Remove the following method.
    // There should only be the one with a byte param
    // (This one is currently needed for Pal-J)
    public void setLocalStatusFlag(int statusFlag) {
        manager.setLocalStatusFlag((byte) statusFlag);
    }
	public void setLocalDiscoveryVersion(String discoveryVersion) {
		manager.setLocalDiscoveryVersion(discoveryVersion);
	}
	public void setDiscoverySelector(Selector selector) {
		manager.setDiscoverySelector(selector);
	}
	public void sendBroadcast(Message message) {
		manager.sendBroadcast(message);
	}
	public void sendHeartAttack() {
		manager.sendHeartAttack();
	}
	
	/**
	 * Registers as a listener to messages received 
	 * by this manager from the network.
	 * @param communicationManager  The PalComThread used in CommunicationManager
	 */
	public void registerAsListener(PalcomThread communicationManager) {
		manager.registerAsListener(communicationManager);
	}
	
	public void registerForHeartBeats(PalcomThread resourceManager) {
		manager.registerForHeartBeats(resourceManager);
	}
	
	public void sendHeartBeat(boolean isHeartBeatAck) {
		manager.sendHeartBeat(isHeartBeatAck, new LinkedList());
	}
	
	/** Remove the device with deviceID from the list of registered devices. */
	public void unregisterDevice(String deviceID) {
		manager.unRegisterDevice(deviceID);
	}
	
    /**
     * This private class is the thread for receiving messages from the 
     * mediamanager, this thread will distribute messages based on their type
     * routing/ordinary.
     */
    private class MessageThread extends PalcomPriorityThread {

		private Message msg = null;

		private Event event = null;

		private AbstractRoutingMessage routingMessage = null;

		MessageThread() {
		}

		public void run() {
			while (true) {
				event = waitEvent();
				if (event instanceof MessageEvent) {
					msg = ((MessageEvent) event).getMessage();
					routingMessage = AbstractRoutingMessage.getRoutingMessage(msg);
					
					if (routingMessage != null) {
						//TODO: this should be changed so that unmarshaling only happens once.
						handleRoutingMessage(routingMessage);
					} else {
						System.out.println("##  ELSE  ##  " + msg.getSenderURL() + " - " + msg.getSenderSelector() + "  ##");
						//TODO: It's not a routing message send it to the correct receiving thread;
					}
				} else if (event instanceof PeriodicEvent) {

				}
			}
		}
	}
    
/****************hgraph impl below********************/
	
	public String getNodeName(){return _node.getNodeName();}
	public int setNodeName(String name, Path callerPath){return _node.setNodeName(name, callerPath);}
	public Path getPath(){return _node.getPath();}
	public void updatePath(){_node.updatePath();}
	public boolean hasChildNamed(String name){return _node.hasChildNamed(name);}
	public boolean isAncestorOf(IHgraphNode node) {return _node.isAncestorOf(node);}
	public int removeNodeChild(String name, Path callerPath){return _node.removeNodeChild(name, callerPath);}
	public String toXML(){return _node.toXML();}
	public String toXMLFull(){return _node.toXMLFull();}
	public String toXMLRecursively(){return _node.toXMLRecursively();}
	public String toXMLFullRecursively(){return _node.toXMLFullRecursively();}
	public int setNodeParent(IHgraphNode newParent, Path callerPath) {return _node.setNodeParent(newParent, callerPath);}
	public int addNodeChild(IHgraphNode node, Path callerPath) {return _node.addNodeChild(node, callerPath);}	
	public SiblingFunc getSiblingFunc(){return _node.getSiblingFunc();};
	public ValueFunc getValueFunc(){return _node.getValueFunc();};
	public FuncFunc getFuncFunc(){return _node.getFuncFunc();};	
	public IHgraphNode getNodeParent(){return _node.getNodeParent();}
	public Iterator getChildIterator(){return _node.getChildIterator();}

}
