/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util;


import ist.palcom.collections.LinkedList;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PeriodicEvent;
import ist.palcom.routing.util.messages.DSDVUpdateMessage;
import ist.palcom.routing.util.messages.PingMessage;

public class DSDVRoutine {
	
	private final static int pingInterval = 2000;
	
	private DSDVRoutingManager manager;
	
	public DSDVRoutine(DSDVRoutingManager manager) {
		this.manager = manager;
	}
	
	public PalcomPriorityThread startRoutine() {
		
		PalcomPriorityThread thread = new PalcomPriorityThread() {
			 
			public void run() {
				 
				// Create a ping message
				PingMessage message = new PingMessage(manager.getRoutingURN(), manager.getRoutingSelector());
				
				int greylistInterval = 0;
				
				// Routine checking if our neighbours are reachable
				while (true) {
					LinkedList list = manager.getDSDVTable().getPeers();
					DSDVPeer peer = (DSDVPeer) list.getFirst();
					System.out.println("DSDV-Table:");
					System.out.println("-----------");
					boolean more = peer != null;
					while (more) {
						System.out.println(peer);
						
						// Only ping neighbours we can reach directly
						if (peer.getMetric() == 0) {
							// Send our DSDVtable periodically (Because of rascal)
							if (greylistInterval % 2 == 0) {
								manager.sendDirect(new DSDVUpdateMessage(manager.getRoutingURN(), manager.getRoutingSelector(), manager.getDSDVTable()), 
										manager.getRoutingURN(), manager.getRoutingSelector(), peer.getRoutingURN(), peer.getRoutingSelector());
							}
							
							// If status = 3 remove the peer
							if (peer.getStatus() >= 3) {
								list.remove(peer);
								
								DSDVPeer tmppeer = (DSDVPeer) list.getFirst();
								boolean tmpmore = tmppeer != null;
								while (tmpmore) {
									if (tmppeer.getNextHopURN().equals(peer.getNextHopURN()))
											list.remove(tmppeer);
									
									tmppeer = (DSDVPeer) tmppeer.getNext();
									tmpmore = tmppeer != null;
								}
								
								
								peer.setIsActive(false);
								peer.setSequenceNumber(peer.getSequenceNumber() + 2);
								manager.getDSDVChangedPeers().addPeer(peer.copy());
								
							// Else try to ping the peer
							} else if (peer.getStatus() >= 1) {
								peer.setStatus(peer.getStatus() + 1);
								manager.sendDirect(message, manager.getRoutingURN(), manager.getRoutingSelector(),
										peer.getNextHopURN(), peer.getNextHopSelector());
							} else {
								peer.setStatus(peer.getStatus() + 1);
							}
							
						}

						peer = (DSDVPeer) peer.getNext();
						more = peer != null;
					}
					System.out.println("-----------");
					System.out.println("");
					
					
					greylistInterval++;

					
					// The DSDV-table has been modified - send update to our neighbours
					if (!manager.getDSDVChangedPeers().getPeers().isEmpty()) {
						DSDVUpdateMessage update = new DSDVUpdateMessage(manager.getRoutingURN(), manager.getRoutingSelector(), manager.getDSDVChangedPeers());
						
						list = manager.getDSDVTable().getPeers();
						peer = (DSDVPeer) list.getFirst();
		
						more = peer != null;
						while (more) {
							// Only send update to neighbours in our subnet
							if (peer.getMetric() == 0) {
								manager.sendDirect(update, manager.getRoutingURN(), manager.getRoutingSelector(), 
										peer.getRoutingURN(), peer.getRoutingSelector());
							}
						
							peer = (DSDVPeer) peer.getNext();
							more = peer != null;
						}
						
						manager.getDSDVChangedPeers().reset();
					}
					
					// Wait an amount of second until next round
					enablePeriodicEvents(pingInterval);
					Event event = waitEvent();
					
					while (!(event instanceof PeriodicEvent)) {
						event = waitEvent();
					}
						           
	                disablePeriodicEvents();
				}
				
			 }
			 
		};
		
		return thread;
	}

}
