/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util;


import ist.palcom.collections.Link;
import ist.palcom.collections.LinkedList;
import ist.palcom.resource.descriptor.URL;

public class SMURFNode extends Link {
	private URL address;
	private LinkedList neighbours;
	private boolean marked;
	public SMURFNode(URL address){
		this.address = address;
	}
	
	/**
	 * Return the neighbours of this node
	 * @return a LinkedList of neighbours
	 */
	public LinkedList getNeighbours(){
		return neighbours;
	}
	
	/**
	 * Merging the linkedlist neighbours into the local linked list
	 * containing known neighbours.
	 * @param neighbours
	 */
	
	public void updateNeighbours(LinkedList neighbours){
		SMURFNode nNode = (SMURFNode) neighbours.getFirst();
		while(nNode != null){
			if(findeNode(nNode.getAddress(),neighbours) == null){
				neighbours.insertLast(nNode);
			}
		}
	}
	
	/**
	 * Return the address of this Address SMURFNode
	 * @return - Address
	 */
	
	public URL getAddress() {
		return address;
	}
	
	public void unMark(){
		marked = false;
	}
	
	public void mark(){
		marked = true;
	}
	
	public boolean isMarked(){
		return marked;
	}

	/**
	 * Find the node in the linkedlist based on the input address
	 * 
	 * @param address - address of the node to finde.
	 * @param list - list where to finde the node.
	 * @return - null or the node
	 */
	private SMURFNode findeNode(URL address, LinkedList list) {
		SMURFNode node = (SMURFNode) list.getFirst();
		while (node != null) {
			if (node.getAddress().equals(address)) {
				return node;
			}
			node = (SMURFNode) node.getNext();
		}
		return null;
	}
}
