/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util.messages;

import ist.palcom.mal.Message;

public abstract class AbstractRoutingMessage extends Message {
	
	public AbstractRoutingMessage(byte[] information) {
		super(information);
	}
	
	public static AbstractRoutingMessage getRoutingMessage(Message message) {
		byte[] receivedData = new byte[message.getTotalLength()];
        message.write(receivedData, 0);
        
        String data = "";
        int index = -1;
        
        int offset = message.getTotalLength() - message.getDataLength();       
        for (int i = offset; i < receivedData.length; i++) {
        	char c = (char) receivedData[i];
        	if (c == ' ')
        		break;
        	data += c;
        }
		
		String msgType = data;
        
		
		if (DSDVRoutingMessage.type.equals(msgType))
        	return DSDVRoutingMessage.unmarshall(message);
        else if (DSDVUpdateMessage.type.equals(msgType))
        	return DSDVUpdateMessage.unmarshall(message);
        else if (SMURFFullAdvertisement.type.equals(msgType))
        	return new SMURFFullAdvertisement(data.substring(index,data.length()).getBytes());
        else if (SMURFUpdateAdvertisement.type.equals(msgType))
        	return new SMURFUpdateAdvertisement(data.substring(index,data.length()).getBytes());
        else if (PongMessage.type.equals(msgType))
        	return PongMessage.unmarshall(message);
        else if (BootstrapMessage.type.equals(msgType))
        	return BootstrapMessage.unmarshall(message);
        else if (PingMessage.type.equals(msgType))
        	return PingMessage.unmarshall(message);
		
		return null;
	}
}
