/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.simulated.device;

import ist.palcom.resource.descriptor.DeviceID;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Properties;

/** 
 * @author David Svensson, david@cs.lth.se 
 */
public class DeviceDescriptor {

    public final DeviceID deviceID;

    public String name;

    public String type;

    public DeviceDescriptor(DeviceID deviceID) {
        this.deviceID = deviceID;
    }

    public File getDeviceDirectory() {
        try {
            return new File(AbstractSimulatedDeviceGUI.DEVICES_DIRECTORY,
                    URLEncoder.encode(deviceID.toString(), "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            // Should not happen with UTF-8
            throw new RuntimeException(e);
        }
    }

    private File getPropertiesFile() {
        return new File(getDeviceDirectory(), "device.properties");
    }

    public static DeviceDescriptor createFromPropertiesFile(DeviceID deviceID) {
        DeviceDescriptor descriptor = new DeviceDescriptor(deviceID);
        File propertiesFile = descriptor.getPropertiesFile();
        if (propertiesFile.exists()) {
            try {
                FileInputStream inputStream = new FileInputStream(
                        propertiesFile);
                Properties properties = new Properties();
                properties.load(inputStream);
                inputStream.close();

                // Update attribute values
                descriptor.name = properties.getProperty("name");
                descriptor.type = properties.getProperty("type");
            } catch (IOException e) {
                // TODO Handle this error?
            }
        }
        return descriptor;
    }

    public void writeToPropertiesFile() {
        // Put attribute values into Properties table
        Properties properties = new Properties();
        properties.put("name", name);
        properties.put("type", type);

        try {
            getDeviceDirectory().mkdirs();
            FileOutputStream outputStream = new FileOutputStream(
                    getPropertiesFile());
            properties.store(outputStream, "Device properties");
            outputStream.close();
        } catch (IOException e) {
            // TODO Handle this error?
        }
    }
}
