/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.pfs;

/**
 * ByteStorage.java
 * 
 * ByteStorage is a very simple filesystem that is implemented 
 * on a byte array
 * 
 * @author Konrad Tollmar, konrad.tollmar@design.lth.se
 */

public class ByteStorage extends AbstractStorage {

	static final int msize = 1024;
	static final int wsize = 50;
	static final int fsize = 10;
	
	private byte [] fs; 
	private int [] filePos; 
	private int [][] fileIdx;
	private int writes = 0;
	private int free = 0;

	private String dirs[];
	private int files = 0;

	private String root = "";
	
	public ByteStorage(String root) {
		this.root = root;
		dirs = new String[fsize];
		fs = new byte[msize];
		fileIdx = new int [wsize][2];
		filePos = new int[fsize];
	}

	public int create(String filen) { 
		dirs[files] = filen;
		filePos[files] = 0;
		++files;
		return (files-1);
	}

	public int open(String filen, int mode) {
		int fd = fd(filen);
		if ( fd == -1) {
			return create(filen);
		} else {
			seek(filen,0);
			return fd;
		}
	}
	
	public int close(String filen) {
		return -1;
	}

	public int fd(String filen) {
		for (int i = 0; i<fsize; i++) {
			if (dirs[i] != null) {
				if (filen.equals(dirs[i]))
					return i;
			} else {
				return -1;
			}
		}	
		return -1;
	}
	
	public int seek(String file, long pos) {
		filePos[fd(file)] = (int)pos;
		return find(fd(file), (int)pos);
	}
	
	public long pos(String file) {
		return filePos[fd(file)];
	}

	private int find(int file, int pos) {
		int fpos = 0;
		for (int i = 0; i < writes; i++) {
			if (fileIdx[i][1] == file) {
				for (int j = fileIdx[i][0]; j < fileIdx[i+1][0]; j++) {
					if (fpos++ == pos) {						
						return j;
					}
				}
			}
		}
		return -1;
	}

	public int read(String filen) {
		int fd = fd(filen);
		int pos = (int)pos(filen);
		int find = find(fd,pos++);
		seek(filen,pos);
		return (int) fs[find] & 0xFF; // convert unsignedByteToInt
	}
		
	public int readBytes(String file, byte b[], int off, int len) {
		int fd = fd(file);
		int pos = (int)pos(file);
		for (int i=0; i<len; i++) {
	    	int find = find(fd,pos++);
			seek(file,pos);
	    	if (find != -1)
	    		b[i] = fs[find];
	    	else 
	    		return i;
		}
		return len;
	}
	
	public int write(String filen, int b) {
		int fd = fd(filen);
		int pos = (int)pos(filen);
	    int find = find(fd,pos);
		
		if (find == -1) {    
			fs[free] = (byte)b;
	 		fileIdx[writes][0] = free;
			fileIdx[writes][1] = fd;
			++writes;
			++free;
			fileIdx[writes][0] = free;
			fileIdx[writes][1] = -1;
			seek(filen,pos+1);
		} else {
			fs[find] = (byte)b;
			seek(filen, pos+1);
		}
		return 1;
	}

	public int writeBytes(String file, byte [] data, int off, int len) {
		int fd = fd(file);
		for (int i=0; i<len; i++) {
			int pos = (int)pos(file);
	    	int find = find(fd,pos);

	    	if (find == -1) {    
	    		int start = free;
	    		int j=0;
	    		for (j=i; j<len; j++) {
					fs[free++] = data[j];
		 		}
				fileIdx[writes][0] = start;
				fileIdx[writes][1] = fd;
				fileIdx[writes + 1][0] = free;
				fileIdx[writes + 1][1] = -1;
				++writes;
				seek(file,pos+j);
		 		return len;
			} else {
				fs[find] = data[i];
				seek(file, ++pos);
			} 
		}
		return len;
	}

	public void print() {
		System.out.println(new String(fs));	
	}
	
}

