/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.pfs;

/**
 * FileStorage.java
 * 
 * FileStorage is a filesystem that use the systems implemented file operations, ie see FileWrapper and RandomAccessFileWrapper
 * 
 * @author Konrad Tollmar, konrad.tollmar@design.lth.se
 */

import ist.palcom.base.storage.FileWrapper;
import ist.palcom.base.storage.RandomAccessFileWrapper;
import ist.palcom.collections.LinearMap;

import java.io.IOException;

public class FileStorage extends AbstractStorage {

	LinearMap files;

	String root;

	public FileStorage(String root) {
		this.root = root; // todo: read / store fs from root in files
		files = new LinearMap();
	}

	public int open(String name, int mode) {
		try {
			FileWrapper pfile = new FileWrapper(name); // fm.getFile(name);
			RandomAccessFileWrapper raf = new RandomAccessFileWrapper(pfile, 0);
			if (raf != null) {
				files.put(name, raf);
				return Storage.OK;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return Storage.ERROR;
	}

	public int close(String file) {
		try {
			if (files.get(file) != null) {
				((RandomAccessFileWrapper) files.get(file)).close();
				return Storage.OK;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return Storage.ERROR;
	}

	public long pos(String file) {
		RandomAccessFileWrapper raf;
		if ((raf = (RandomAccessFileWrapper) files.get(file)) != null) {
			try {
				return raf.getFilePointer();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return -1;
	}

	public int seek(String file, long pos) {
		RandomAccessFileWrapper raf;
		if ((raf = (RandomAccessFileWrapper) files.get(file)) != null) {
			try {
				raf.seek(pos);
				return Storage.OK;
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return Storage.ERROR;
	}

	public int read(String file) {
		RandomAccessFileWrapper in;
		int b = 0;
		if ((in = (RandomAccessFileWrapper) files.get(file)) != null) {
			try {
				b = in.read();
			} catch (IOException e) {
				e.printStackTrace();
			}
			return b;
		}
		return Storage.ERROR;
	}

	public int write(String file, int b) {
		RandomAccessFileWrapper out;
		if ((out = (RandomAccessFileWrapper) files.get(file)) != null) {
			try {
				out.write(b);
			} catch (IOException e) {
				e.printStackTrace();
				return Storage.ERROR;
			}
		}
		return Storage.OK;
	}

	public int readBytes(String file, byte b[], int off, int len) {
		RandomAccessFileWrapper in;
		int rb = 0;
		if ((in = (RandomAccessFileWrapper) files.get(file)) != null) {
			try {
				rb = in.read(file, b, off, len);
			} catch (IOException e) {
				e.printStackTrace();
				return Storage.ERROR;
			}
			return rb;
		}
		return Storage.ERROR;
	}

	public int writeBytes(String file, byte b[], int off, int len) {
		RandomAccessFileWrapper out;
		if ((out = (RandomAccessFileWrapper) files.get(file)) != null) {
			try {
				out.write(file, b, off, len);
			} catch (IOException e) {
				e.printStackTrace();
				return Storage.ERROR;
			}
		}
		return Storage.OK;
	}

	public long length(String file) {
		RandomAccessFileWrapper raf;
		if ((raf = (RandomAccessFileWrapper) files.get(file)) != null) {
			try {
				return raf.length();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return Storage.ERROR;
	}

	public int delete(String file) {
		RandomAccessFileWrapper raf;
		if ((raf = (RandomAccessFileWrapper) files.get(file)) != null) {
			if(raf.delete()) {
				return Storage.OK;
			}
		}
		return Storage.ERROR;
	}
}
