/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.pio;

/**
 * StorageOutputStream.java
 * StorageOutputStream enables the use of the Storage as a stream
 * This is (very) simmilar to the std java.io classes
 *  
 * @author Konrad Tollmar, konrad.tollmar@design.lth.se
 */

import ist.palcom.storage.pfs.Storage;
import ist.palcom.storage.pfs.StorageException;

import java.io.*;

public class StorageOutputStream extends java.io.OutputStream {

	Storage store;
	String filen;
	
	/**
	 * Create a stream
	 * @param store The Storage to use
	 * @param filen The File name
	 * @throws IOException
	 */
	public StorageOutputStream(Storage store, String filen)  
	throws IOException
	{
		this.store = store;
		open(filen);
	} 

	/**
	 * Open a stream 
	 * @param filen The File name
	 * @throws IOException
	 */
	private void open(String filen) 
	throws IOException 
	{
		this.filen = filen;
		try {
			store.open(filen, -1); // todo: implement rw mode
		} catch (StorageException e) {
			throw new IOException(); 
		}
	}
	
	/**
	 * Close the stream
	 */
	public void close() 
	throws IOException 
	{
		try {
//			store.write(filen, 27);
			store.close(filen);
		} catch (StorageException e) {
			throw new IOException(); 
		}
	}
	
	/**
	 * Write a byte
	 * @param b the Byte
	 * @throws IOException
	 */
	public void write(int b) throws IOException {
		store.write(filen, b);
    }

	/**
	 * Write a byte array
	 * @param b the byte array
	 * @param off offset in the stream
	 * @param len lenght of the stream
	 * @throws IOException
	 */
	public void write(byte b[], int off, int len) throws IOException {
		store.writeBytes(filen, b, off, len);
	}
	
	/**
	 * Write an int
	 * @param v the int
	 * @throws IOException
	 */
	public final void writeInt(int v) throws IOException {
		/*
  		write((v >>> 24) & 0xFF);
        write((v >>> 16) & 0xFF);
        write((v >>>  8) & 0xFF);
        write((v >>>  0) & 0xFF);
		*/
        
		byte []b = new byte[4];
		b[0] = (byte)((v >>> 24) & 0xFF);
		b[1] = (byte)((v >>> 16) & 0xFF);
		b[2] = (byte)((v >>> 8) & 0xFF);
		b[3] = (byte)((v >>> 0) & 0xFF);
		write(b,0,b.length);
		
    }
}	
