/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.test;

import ist.palcom.storage.test.TaggableEmployee;
import ist.palcom.storage.persistent.Persistent;
import ist.palcom.storage.pfs.FileStorage;
import ist.palcom.storage.pfs.Storage;
import ist.palcom.storage.pio.StorageInputStream;
import ist.palcom.storage.pio.StorageOutputStream;

import java.io.*;

public class Employee implements Persistent, Cloneable {
    protected int      employeeID;
    protected String   firstName;
    protected String   lastName;
    protected int      managerID;

    public Employee(){
    }

    public Employee( int employeeID, String firstName,
                     String lastName, int managerID ){
        this.employeeID = employeeID;
        this.firstName = firstName;
        this.lastName = lastName;
        this.managerID = managerID;
    }

    public Object clone() {
    	try {
    		return super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return null;
    }

    public int getID() { return employeeID; }

    public String getFirstName() {
        return firstName != null ? firstName : ""; 
    }

    public String getLastName() {
        return lastName != null ? lastName : "";
    }

    public int getManagerID() { return managerID; }

    public String toString() {
        StringBuilder b = new StringBuilder();
        b.append( "{" );
        b.append( employeeID );
        b.append( "," );
        b.append( firstName );
        b.append( "," );
        b.append( lastName );
        b.append( "," );
        b.append( managerID );
        b.append( "}" );
        return b.toString();
    }

    public byte[] persist() throws IOException {
        ByteArrayOutputStream bout = 
                           new ByteArrayOutputStream();
        
        bout.write(getID());
        bout.write(getFirstName().getBytes().length); //Length of the string which will be written
        bout.write(getFirstName().getBytes());
        bout.write(getLastName().getBytes().length); //Length of the string which will be written
        bout.write(getLastName().getBytes());
        bout.write(getManagerID());
        bout.flush();
        
        return bout.toByteArray();
    }

    public void resurrect( byte[] data ) 
                                   throws IOException {
        ByteArrayInputStream bin = 
                      new ByteArrayInputStream( data );
        
        employeeID = bin.read();
        int stringLength = bin.read();
        byte[] b = new byte[stringLength];
        int result = bin.read(b);
        if(result>=0) {
        	firstName = new String(b);
        } //else EOF reached
        stringLength = bin.read();
        b = new byte[stringLength];
        result = bin.read(b);
        if(result>=0) {
        	lastName = new String(b);
        } //else EOF reached
        managerID = bin.read();
        
    }
    
    public static void main(String[] args) {
    	
    	Employee emp;
    	
    	int employeeID = 1;
    	String firstName = "Konrad";
        String lastName = "Tollmar";
        int managerID = 2;
        
    	emp = new Employee(employeeID, firstName, lastName, managerID );
    	
    	try {
    			Storage store = new FileStorage("files");
    			StorageOutputStream out = new StorageOutputStream(store,"emp.txt");
    	        byte[] persisted = emp.persist();
    	        out.write(persisted, 0, persisted.length);
    	    	out.close();
    	} catch( java.io.IOException e ){}


		StringBuffer sb = new StringBuffer(); 
    	try {
			Storage store = new FileStorage("files");
    		StorageInputStream in = new StorageInputStream(store,"emp.txt");
    		byte[] buf = new byte[512];
    		int count = 0;
    		while ((count = in.read(buf)) > 0) {
    			sb.append(new String(buf));
    		}
    	} catch( java.io.IOException e ){}

    	byte[] persisted = sb.toString().getBytes(); // presistence info
    	emp = new Employee();
    	    
    	try {
    		emp.resurrect( persisted );
    	}
    	catch( java.io.IOException e ){}
    	
    	System.out.print(emp.toString());
    	
	}
}
