/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.test;

import ist.palcom.storage.tags.Taggable;

import java.io.*;

public class TNote implements Taggable {
	
    protected String   date;
    protected String   note;
	int tagid = 0;
	
    public TNote(){
    }

    public TNote(String date, String note, String id) {
        this.date = date;
        this.note = note;
        setid(Integer.parseInt(id));
    }
        
	public int getid() {
		return tagid;
	}

	public void setid(int id) {
		this.tagid = id;
	}
	
    public Object clone() {
    	try {
    		byte[] data = this.persist();
			TNote tnote = new TNote();
			tnote.resurrect(data);
			return tnote;

		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
    }

    public String getDate() {
        return date != null ? date : ""; 
    }

    public String getNote() {
        return note != null ? note : "";
    }

    public String toString() {
        StringBuilder b = new StringBuilder();
        b.append( "{" );
        b.append( date );
        b.append( "," );
        b.append( note );
        b.append( "," );
        b.append( tagid );
        b.append( "}" );
        return b.toString();
    }

    public byte[] persist() throws IOException {
        ByteArrayOutputStream bout = 
                           new ByteArrayOutputStream();
        
        bout.write(tagid);
        bout.write(getDate().getBytes().length); //Length of the string which will be written
        bout.write(getDate().getBytes());
        bout.write(getNote().getBytes().length);
        bout.write(getNote().getBytes());
        bout.flush();
        
        return bout.toByteArray();
    }

    public void resurrect( byte[] data ) 
                                   throws IOException {
        ByteArrayInputStream bin = 
                      new ByteArrayInputStream( data );
        
        tagid = bin.read();
        
        int stringLength = bin.read();
        byte[] b = new byte[stringLength];
        int result = bin.read(b);
        if(result>=0) {
        	date = new String(b);
        } //else EOF reached
        
        stringLength = bin.read();
        b = new byte[stringLength];
        result = bin.read(b);
        if(result>=0) {
        	note = new String(b);
        } //else EOF reached
        
    }
}
