/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.test;

import ist.palcom.storage.pfs.FileStorage;
import ist.palcom.storage.pfs.Storage;

import ist.palcom.storage.tags.TaggedPersistentStore;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This is a simple test of the TaggedPersistentStore 
 * 
 * @author Konrad Tollmar, konrad.tollmar@design.lth.se
 * 
 */

public class TNoteStorageTest {

	private Storage store; 
    // private PersistentStore ps; 
    private TaggedPersistentStore tps;
    
    TNoteStorageTest() {
        // init storage
        store = new FileStorage("files");
    	tps = new TaggedPersistentStore(store, "note.dat", "note.idx", true);
    }
    
    /** Add Note to Storage */
    private void addNote(byte[] noteData, byte[] idData) {
    	long ltime = System.currentTimeMillis();
        TNote tnote = new TNote(String.valueOf(ltime),
        		new String(noteData), new String(idData));
        System.out.println("storeNote: " + tnote.date + " " 
        			+ tnote.note + " " + tnote.getid());
        tps.add(tnote);
    }

    /** Tag Note in Storage */
    private void tagNote(byte[] tagData, byte[] idData) {
        TNote tnote = new TNote("", "", new String(idData));
        tps.setTag(tnote, new String(tagData));
        System.out.println("tagNote: " + tnote.getid() + " " 
        			+ new String(tagData));
    }

/*
    private void getTag(String tag) {
    	
    	// todo: fix this better, i.e. not have to make a full copy
    	TNote [] tnotes = new TNote[tps.size()];
    	for (int i=0; i < tps.size(); i++) {
    		tnotes[i] = new TNote();
    	}

    	// todo: stupid and heavy .. what to do?
    	Taggable[] tobj = tps.getTagged(tag, tnotes);
    	
    	System.out.println("GetTag:" + tag + " Obj:" + tobj.length);
    	
    	for (int j=0; j<tobj.length; j++) {
    		System.out.println(tobj[j].toString());
    	} 
    }

 */    
    
    private void getTag(String tag) {
    	TNote tnote = new TNote();
    	ArrayList tobj = tps.getTagged2(tag, tnote);
    	
    	System.out.println("GetTag:" + tag + " Obj:" + tobj.size());
    	
    	for (Iterator iter = tobj.iterator(); iter.hasNext();) {
			tnote = (TNote)iter.next();
    		System.out.println(
    				tnote.toString());
    	} 
    }

	/** Returns a Note from Storage */
    private byte[] getNote(int index) {    
    	byte[] noteData = null;
        TNote tnote = new TNote();
        tps.get(index, tnote);
    	if (tnote == null) 
        	return null;
		try {
			noteData = tnote.persist();
		} catch (IOException e) {
			e.printStackTrace();
		}        	 
        // System.out.println("getStoredNoteData: " + noteData.toString());
        return noteData;        	
    }

    /** Delete a Note from Storage */
    private void deleteNote(int index) {
    	tps.delete(index);
    }
    
    private byte[] getSize() {
        int size = tps.size();
        byte[] sizeData = Integer.toString(size).getBytes();
        return sizeData; 
    }
        
public static void main(String[] args) {
	TNoteStorageTest t = new TNoteStorageTest();
	
	t.addNote((new String("Hello")).getBytes(),(new String("1")).getBytes());
	t.addNote((new String("Hej")).getBytes(), (new String("2")).getBytes());
	t.addNote((new String("Howdy")).getBytes(), (new String("3")).getBytes());
	t.addNote((new String("Tjena")).getBytes(), (new String("4")).getBytes());
	t.addNote((new String("Cheers")).getBytes(), (new String("5")).getBytes());
	t.addNote((new String("Salut")).getBytes(), (new String("6")).getBytes());
        
	t.tagNote((new String("Eng")).getBytes(), (new String("1")).getBytes());
	t.tagNote((new String("Sve")).getBytes(), (new String("2")).getBytes());
	t.tagNote((new String("Eng")).getBytes(), (new String("3")).getBytes());
	t.tagNote((new String("Sve")).getBytes(), (new String("4")).getBytes());
	t.tagNote((new String("Eng")).getBytes(), (new String("5")).getBytes());
	t.tagNote((new String("Fra")).getBytes(), (new String("6")).getBytes());
	
	t.getTag("Fra");
}

}
