/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import ist.palcom.collections.LinearSet;
import ist.palcom.storage.tags.OrderedMap;
import ist.palcom.storage.persistent.Persistent;
import ist.palcom.storage.persistent.PersistentStore;
import ist.palcom.storage.pfs.FileStorage;
import ist.palcom.storage.pfs.Storage;
import ist.palcom.storage.tags.Taggable;
import ist.palcom.storage.tags.TaggedPersistentStore;
import ist.palcom.storage.tags.Tags;
import ist.palcom.storage.test.Employee;

public class TagTest {

	void test1() {
		// test1
		System.out.println("\nTest1");
		
		OrderedMap omtags = new OrderedMap();
		omtags.put("key1", "val1");
		omtags.put("key2", "val2");
		omtags.put("key1", "val2");

		Object [] sobj = omtags.get("key1");
		for (int i = 0; i<sobj.length; i++)
			System.out.println("Found key1: " + (String)sobj[i]);
		
		LinearSet ls = omtags.keySet();
		for (Iterator iter = ls.iterator(); iter.hasNext();) {
				String str = (String) iter.next();
				System.out.println("Keyset: " + str);
		}

		ArrayList al = omtags.values();
		for (Iterator iter = al.iterator(); iter.hasNext();) {
			String str = (String) iter.next();
			System.out.println("Values: " + str);
		}
	}
	
	void test2() {
		// test2
		System.out.println("\nTest2");

		TaggTest2 b1 = new TaggTest2(1,"b1");
		TaggTest2 b2 = new TaggTest2(2,"b2");
		TaggTest2 b3 = new TaggTest2(3,"b3");
		
		Tags tags = new Tags();
		
		//tags.set(b1, "b1");
		//tags.set(b2, "b2");
		//tags.set(b3, "b3");

		int [] tt = tags.get("b1");
		for (int i = 0; i<tt.length; i++)
			System.out.println("Found key1: " + tt[i]);
		
	}
	
	void test3() {
		
		System.out.println("\nTest3");

		Storage store = new FileStorage("files");        
    	
    	int employeeID; 
    	String firstName;
        String lastName;
        int managerID;
        
        TaggableEmployee empp = new TaggableEmployee( );

        employeeID = 1;
    	firstName = "K";
        lastName= "Tollmar";
        managerID = 2;

        TaggableEmployee emp1 = new TaggableEmployee(employeeID, firstName, 
        		lastName, managerID );
    	
    	employeeID = 2;
    	firstName = "T";
        lastName= "Kalling";
        managerID = 3;

        TaggableEmployee emp2 = new TaggableEmployee(employeeID, firstName, 
        		lastName, managerID );
        
    	employeeID = 3;
    	firstName = "A";
        lastName= "Malm";
        managerID = 0;

        TaggableEmployee emp3 = new TaggableEmployee(employeeID, firstName, 
        		lastName, managerID );

    	employeeID = 4;
    	firstName = "M";
        lastName= "Lathinen";
        managerID = 1;

        TaggableEmployee emp4 = new TaggableEmployee(employeeID, firstName, 
        		lastName, managerID );
    
        // 1: Add 
        
    	TaggedPersistentStore tps = new TaggedPersistentStore(store, 
    			"test.dat", "test.idx", true);

    	System.out.print("add:");
    	System.out.println(emp1.toString());
    	tps.add(emp1);    	
    	System.out.print("add:");
    	System.out.println(emp2.toString());
    	tps.add(emp2);
    	System.out.print("add:"); 
    	System.out.println(emp3.toString());
    	tps.add(emp3);

    	// 2: Tagg
    	
    	tps.setTag(emp1, "emp1");
    	tps.setTag(emp2, "emp2");
    	tps.setTag(emp3, "emp3");

    	tps.setTag(emp1, "lusax");
    	tps.setTag(emp2, "lusax");
    	
    	// todo: fix this better, i.e. not have to make a full copy
    	TaggableEmployee [] temp = new TaggableEmployee[tps.size()];
    	for (int i=0; i<tps.size(); i++) {
    		temp[i] = new TaggableEmployee();
    	}

    	// todo: stupid and heavy .. what to do?
    	Taggable[] tobj = tps.getTagged("lusax", temp);
    	for (int i=0; i<tobj.length; i++) {
    		System.out.println("Tagged: " + tobj[i].toString());
    	}

    	tps.print(empp);
    	tps.close();

	}
	
	public static void main(String[] args) {
		TagTest t = new TagTest();
		t.test1();	
		t.test2();	
		t.test3();	
	}
	
	class TaggTest2 implements Taggable {
		
		String b;
		int id;
		
		TaggTest2(int id, String b) {
			this.b = b;
			setid(id);
		}

		public int getid() {
			return id;
		}

		public void setid(int id) {
			this.id = id;
		}

		public byte[] persist() throws IOException {
			return null;
		}

		public void resurrect(byte[] data) throws IOException {			
		}
		
		public Object clone() { return null; }

	}
	
	

}
